@@ -96,7 +96,7 @@ var TabmixTabClickOptions = {
       this.clickAction(prefName, clickOutTabs, tab, aEvent);
   },
 
-  clearTabFlipTimeOut: function() {
+  clearTabFlipTimeOut() {
     clearTimeout(this._tabFlipTimeOut);
     this._tabFlipTimeOut = null;
   },
@@ -140,37 +140,37 @@ var TabmixTabClickOptions = {
     }
   },
 
-/// add option to open new tab after current one
-/// convert this switch to object
+  /// add option to open new tab after current one
+  /// convert this switch to object
   doCommand: function TMP_doCommand(command, aTab, clickOutTabs, event) {
     gBrowser.selectedBrowser.focus();
     switch (command) {
-      case 0 :
+      case 0:
         break;
-      case 1 :
+      case 1:
         BrowserOpenTab();
         break;
-      case 2 :
+      case 2:
         if (aTab && aTab.parentNode) {
           let byMouse = Tabmix.isVersion(520) ? event && event.mozInputSource == MouseEvent.MOZ_SOURCE_MOUSE : true;
-          gBrowser.removeTab(aTab, {animate: true, byMouse: byMouse});
+          gBrowser.removeTab(aTab, {animate: true, byMouse});
         }
         break;
-      case 3 :
+      case 3:
         gBrowser.duplicateTab(aTab);
         break;
-      case 4 :
+      case 4:
         if (aTab.linkedBrowser.__SS_restoreState != 2)
           gBrowser.reloadTab(aTab);
         break;
-      case 5 :
+      case 5:
         gBrowser.protectTab(aTab);
         break;
-      case 6 :
+      case 6:
         gBrowser.lockTab(aTab);
         break;
-      case 7 :
-        tablib.reloadTabs(gBrowser.visibleTabs);
+      case 7:
+        Tabmix.tablib.reloadTabs(gBrowser.visibleTabs);
         break;
       case 8:
         gBrowser.removeAllTabsBut(aTab);
@@ -225,6 +225,8 @@ var TabmixTabClickOptions = {
           if (typeof gIeTabObj.switchTabEngine == "function") {
             if (!aTab.selected)
               gBrowser.selectedTab = aTab;
+            // IeTab2.getBoolPref accept default value
+            // eslint-disable-next-line tabmix/no-useless-parameters
             gIeTabObj.switchTabEngine(aTab, gIeTabObj.getBoolPref(ieTab.folder + ".alwaysNewTab", false));
           }
         } else if (window.ieview && window.ieview.launch) {
@@ -288,7 +290,7 @@ var TabmixTabClickOptions = {
     return true;
   },
 
-  toggleEventListener: function(enable) {
+  toggleEventListener(enable) {
     let eventListener = enable ? "addEventListener" : "removeEventListener";
     document.getElementById("TabsToolbar")[eventListener]("dblclick", this.blockDblclick, false);
   },
@@ -297,7 +299,7 @@ var TabmixTabClickOptions = {
    * block dblclick on TabsToolbar when tabbar.dblclick_changesize is false
    * and tabbar.click_dragwindow is true
    */
-  blockDblclick: function(aEvent) {
+  blockDblclick(aEvent) {
     if (aEvent.button !== 0 || aEvent.target.localName == "tabs" ||
         Tabmix.prefs.getBoolPref("tabbar.dblclick_changesize") ||
         !Tabmix.prefs.getBoolPref("tabbar.click_dragwindow"))
@@ -345,8 +347,8 @@ var TabmixContext = {
       var aFunction = "createTabbarMenu" in IeTab.prototype ? "createTabbarMenu" : "init";
       if (aFunction in IeTab.prototype) {
         Tabmix.changeCode(IeTab.prototype, "IeTab.prototype." + aFunction)._replace(
-             'tabbarMenu.insertBefore(document.getElementById("ietab-tabbar-sep"), separator);',
-             'separator = document.getElementById("tm-separator-3"); $&'
+          'tabbarMenu.insertBefore(document.getElementById("ietab-tabbar-sep"), separator);',
+          'separator = document.getElementById("tm-separator-3"); $&'
         ).toCode();
       }
     }
@@ -367,7 +369,7 @@ var TabmixContext = {
     sep.parentNode.insertBefore(sep, $id("tm-content-closetab"));
   },
 
-  updateTabbarContextMenu: function(show) {
+  updateTabbarContextMenu(show) {
     let tabBar = gBrowser.tabContainer;
     if (show) {
       this._originalTabbarContextMenu = tabBar.getAttribute("context");
@@ -377,14 +379,14 @@ var TabmixContext = {
     }
   },
 
-  toggleEventListener: function(enable) {
+  toggleEventListener(enable) {
     var eventListener = enable ? "addEventListener" : "removeEventListener";
     document.getElementById("contentAreaContextMenu")[eventListener]("popupshowing", this, false);
     gBrowser.tabContextMenu[eventListener]("popupshowing", this, false);
     gBrowser.tabContextMenu[eventListener]("popupshown", this, false);
   },
 
-  handleEvent: function(aEvent) {
+  handleEvent(aEvent) {
     let id = aEvent.target.id;
     switch (aEvent.type) {
       case "popupshowing":
@@ -401,7 +403,7 @@ var TabmixContext = {
         break;
       case "popuphidden":
         if (id == "tabContextMenu") {
-          aEvent.target.removeEventListener("popuphidden", this, false);
+          aEvent.target.removeEventListener("popuphidden", this);
           Tabmix.hidePopup(aEvent.target);
         }
         break;
@@ -413,7 +415,7 @@ var TabmixContext = {
     if (event.originalTarget != gBrowser.tabContextMenu)
       return true;
 
-    gBrowser.tabContextMenu.addEventListener("popuphidden", this, false);
+    gBrowser.tabContextMenu.addEventListener("popuphidden", this);
 
     var item, triggerNode = gBrowser.tabContextMenu.triggerNode;
     if (triggerNode.parentNode)
@@ -442,8 +444,8 @@ var TabmixContext = {
     Tabmix.showItem("context_openTabInWindow", Tabmix.prefs.getBoolPref("detachTabMenu") && !Tabmix.singleWindowMode);
     if (Tabmix.isVersion(320)) {
       Tabmix.showItem("context_openNonRemoteWindow",
-                      Tabmix.prefs.getBoolPref("tabcontext.openNonRemoteWindow") &&
-                      !Tabmix.singleWindowMode && gMultiProcessBrowser);
+        Tabmix.prefs.getBoolPref("tabcontext.openNonRemoteWindow") &&
+        !Tabmix.singleWindowMode && gMultiProcessBrowser);
     }
 
     if (Tabmix.isVersion(430)) {
@@ -460,9 +462,9 @@ var TabmixContext = {
       Tabmix.showItem(tabViewMenu, Tabmix.prefs.getBoolPref("moveToGroup") && !aTab.pinned);
     });
     Tabmix.showItem("tm-mergeWindowsTab",
-                    Tabmix.prefs.getBoolPref("showMergeWindow") &&
-                    (!Tabmix.singleWindowMode ||
-                    (Tabmix.singleWindowMode && !isOneWindow)));
+      Tabmix.prefs.getBoolPref("showMergeWindow") &&
+      (!Tabmix.singleWindowMode ||
+      (Tabmix.singleWindowMode && !isOneWindow)));
     var showRenameTabMenu = Tabmix.prefs.getBoolPref("renameTabMenu");
     Tabmix.showItem("tm-renameTab", showRenameTabMenu);
     Tabmix.showItem("tm-copyTabUrl", Tabmix.prefs.getBoolPref("copyTabUrlMenu"));
@@ -539,7 +541,7 @@ var TabmixContext = {
     Tabmix.setItem("context_closeOtherTabs", "disabled", noTabsToClose);
     Tabmix.setItem("context_closeTabsToTheEnd", "disabled", cIndex == tabsCount - 1 || noTabsToClose);
     Tabmix.setItem("tm-closeLeftTabs", "disabled",
-                   cIndex === 0 || aTab.pinned || Tabmix.visibleTabs.previous(aTab).pinned);
+      cIndex === 0 || aTab.pinned || Tabmix.visibleTabs.previous(aTab).pinned);
 
     var closeTabsEmpty = TMP_ClosedTabs.count < 1;
     Tabmix.setItem("context_undoCloseTab", "disabled", closeTabsEmpty);
@@ -629,11 +631,11 @@ var TabmixContext = {
       var protectedTab = tab.hasAttribute("protected");
       var lockedTab = tab.hasAttribute("locked");
 
-     /**
-      * from Firefox 4.0 2009-09-11 there is gContextMenu.openLinkInCurrent
-      * Firefox only show this menu when the selection text is url see Bug 454518
-      * we check if gContextMenu.linkURL contain URL
-      */
+      /**
+       * from Firefox 4.0 2009-09-11 there is gContextMenu.openLinkInCurrent
+       * Firefox only show this menu when the selection text is url see Bug 454518
+       * we check if gContextMenu.linkURL contain URL
+       */
       var onLink = gContextMenu.onLink || gContextMenu.linkURL;
       Tabmix.showItem("context-openlinkincurrent", Tabmix.prefs.getBoolPref("openLinkHere") && onLink);
       var inverseLink = document.getElementById("tm-openinverselink");
@@ -656,7 +658,7 @@ var TabmixContext = {
           onLink && (Tabmix.prefs.getBoolPref("openLinkHere") ||
                      Tabmix.prefs.getBoolPref("openInverseLink") ||
                      Tabmix.prefs.getBoolPref("linkWithHistory"))) {
-        gContextMenu.tabmixLinkURL = tablib.getValidUrl();
+        gContextMenu.tabmixLinkURL = Tabmix.tablib.getValidUrl();
       }
 
       var freezeTabMenu = document.getElementById("tm-content-freezeTab");
@@ -704,11 +706,11 @@ var TabmixContext = {
       Tabmix.showItem(mergeMenu, !contentClick && !isOneWindow && Tabmix.prefs.getBoolPref("mergeWindowContent"));
 
       this._showAutoReloadMenu("tm-autoreload_menu", "autoReloadContent",
-                               !contentClick && !gContextMenu.isTextSelected);
+        !contentClick && !gContextMenu.isTextSelected);
 
       Tabmix.showItem("tm-openAllLinks",
-                      Tabmix.prefs.getBoolPref("openAllLinks") &&
-                      !TabmixContext.openMultipleLinks(true));
+        Tabmix.prefs.getBoolPref("openAllLinks") &&
+        !TabmixContext.openMultipleLinks(true));
 
       // show/hide menuseparator
       var undoCloseSep = document.getElementById("tm-content-undoCloseSep");
@@ -742,8 +744,8 @@ var TabmixContext = {
   },
 
   openMultipleLinks: function TMP_openMultipleLinks(check) {
-    let urls = Tabmix.isVersion(420) ? gContextMenu.tabmixLinks :
-        Tabmix.ContextMenu.getSelectedLinks(window.content, check);
+    let urls = Tabmix.isVersion(420) ? gContextMenu.tabmixLinks || [] :
+      Tabmix.ContextMenu.getSelectedLinks(window.content, check);
 
     if (!check && urls.length) {
       Tabmix.loadTabs(urls, false);
@@ -802,12 +804,14 @@ var TabmixAllTabs = {
   checkForCtrlClick: function TMP_checkForCtrlClick(aEvent) {
     var aButton = aEvent.target;
     if (!aButton.disabled && aEvent.button === 0 && (aEvent.ctrlKey || aEvent.metaKey)) {
-      if (aButton.id == "btn_undoclose")
+      if (aButton.id == "btn_undoclose") {
         TMP_ClosedTabs.undoCloseTab();
-      else
+        aButton.setAttribute("afterctrlclick", true);
+      } else if (aButton.id == "btn_tabslist" ||
+          aButton.parentNode && aButton.parentNode.id == "btn_tabslist_menu") {
         BrowserCloseTabOrWindow();
-
-      aButton.setAttribute("afterctrlclick", true);
+        aButton.setAttribute("afterctrlclick", true);
+      }
     }
   },
 
@@ -867,7 +871,7 @@ var TabmixAllTabs = {
   // show sort/unsort tabs list popup after click on sorted tab menu
   showTabsListPopup: function TMP_showTabsListPopup(event) {
     event.stopPropagation();
-    setTimeout(function(popup) {
+    setTimeout(popup => {
       popup.showPopup(popup.parentNode, -1, -1, "popup", "bottomleft", "topleft");
     }, 0, event.target.parentNode);
   },
@@ -890,7 +894,7 @@ var TabmixAllTabs = {
       popup.setAttribute("minwidth", popup.boxObject.width);
     }
 
-    gBrowser.tabContainer.mTabstrip.addEventListener("scroll", this, false);
+    gBrowser.tabContainer.mTabstrip.addEventListener("scroll", this);
     this._popup = popup;
     if (!this._popup._updateTabsVisibilityStatus)
       this._popup._updateTabsVisibilityStatus = this._updateTabsVisibilityStatus;
@@ -909,16 +913,17 @@ var TabmixAllTabs = {
       var menuItem = popup.firstChild;
       if (menuItem.id.indexOf("btn_tabslist") != -1)
         break;
-      menuItem.removeEventListener("click", TMP_ClosedTabs.checkForMiddleClick, false);
-      popup.removeChild(menuItem);
+      menuItem.removeEventListener("command", TMP_ClosedTabs);
+      menuItem.removeEventListener("click", TMP_ClosedTabs);
+      menuItem.remove();
     }
 
     if (!aCloseTabsPopup) {
-      gBrowser.tabContainer.addEventListener("TabAttrModified", this, false);
-      gBrowser.tabContainer.addEventListener("TabClose", this, false);
+      gBrowser.tabContainer.addEventListener("TabAttrModified", this);
+      gBrowser.tabContainer.addEventListener("TabClose", this);
     }
-    popup.addEventListener("DOMMenuItemActive", this, false);
-    popup.addEventListener("DOMMenuItemInactive", this, false);
+    popup.addEventListener("DOMMenuItemActive", this);
+    popup.addEventListener("DOMMenuItemInactive", this);
   },
 
   createCommonList: function TMP_createCommonList(popup, aType, side) {
@@ -974,12 +979,12 @@ var TabmixAllTabs = {
     }
 
     if (this._selectedItem)
-      popup.addEventListener("popupshown", this, false);
+      popup.addEventListener("popupshown", this);
   },
 
   _ensureElementIsVisible: function TMP__ensureElementIsVisible(event) {
     var popup = event.target;
-    popup.removeEventListener("popupshown", this, false);
+    popup.removeEventListener("popupshown", this);
     let scrollBox = document.getAnonymousElementByAttribute(popup, "class", "popup-internal-box");
     let items = Array.prototype.slice.call(popup.childNodes);
     let element = items.indexOf(this._selectedItem) < popup.childElementCount / 2 ? popup.firstChild : popup.lastChild;
@@ -1018,7 +1023,7 @@ var TabmixAllTabs = {
         let tabClr = TabmixSessionData.getTabValue(tab, "tabClr");
         if (tabClr)
           rule = "linear-gradient(rgba(255,255,255,.7),rgba(#1,.5),rgb(#1)),linear-gradient(rgb(#1),rgb(#1))"
-                 .replace(/#1/g, tabClr);
+              .replace(/#1/g, tabClr);
       }
       mi.style.setProperty('background-image', rule, 'important');
     }
@@ -1029,7 +1034,7 @@ var TabmixAllTabs = {
       return;
 
     aMenuitem.setAttribute("label", aMenuitem.getAttribute("count") + aTab.label);
-    aMenuitem.setAttribute("crop", aTab.getAttribute("crop"));
+    aMenuitem.setAttribute("crop", Tabmix.isVersion(530) ? "end" : aTab.getAttribute("crop"));
 
     if (aTab.hasAttribute("busy")) {
       aMenuitem.setAttribute("busy", aTab.getAttribute("busy"));
@@ -1052,8 +1057,9 @@ var TabmixAllTabs = {
 
   _tabOnTabClose: function TMP__tabOnTabClose(aEvent) {
     var menuItem = aEvent.target.mCorrespondingMenuitem;
-    if (menuItem && menuItem.parentNode)
-      menuItem.parentNode.removeChild(menuItem);
+    if (menuItem) {
+      menuItem.remove();
+    }
   },
 
   _tabsListOncommand: function TMP__tabsListOncommand(aEvent) {
@@ -1078,7 +1084,7 @@ var TabmixAllTabs = {
       if ("tab" in menuItem) {
         menuItem.tab.mCorrespondingMenuitem = null;
       }
-      popup.removeChild(menuItem);
+      menuItem.remove();
     }
 
     var item = popup.parentNode;
@@ -1090,11 +1096,11 @@ var TabmixAllTabs = {
       popup.removeAttribute("minwidth");
     }
 
-    gBrowser.tabContainer.removeEventListener("TabAttrModified", this, false);
-    gBrowser.tabContainer.mTabstrip.removeEventListener("scroll", this, false);
-    gBrowser.tabContainer.removeEventListener("TabClose", this, false);
-    popup.removeEventListener("DOMMenuItemActive", this, false);
-    popup.removeEventListener("DOMMenuItemInactive", this, false);
+    gBrowser.tabContainer.removeEventListener("TabAttrModified", this);
+    gBrowser.tabContainer.mTabstrip.removeEventListener("scroll", this);
+    gBrowser.tabContainer.removeEventListener("TabClose", this);
+    popup.removeEventListener("DOMMenuItemActive", this);
+    popup.removeEventListener("DOMMenuItemInactive", this);
 
     this.backupLabel = "";
     this._selectedItem = null;
