@@ -37,7 +37,7 @@ var TMP_extensionsCompatibility = {
     }
 
     // sessionManager extension is restartless since version 0.8
-    Tabmix.extensions.__defineGetter__("sessionManager", function() {
+    Tabmix.extensions.__defineGetter__("sessionManager", () => {
       return TabmixSvc.sessionManagerAddonInstalled ||
         "com" in window && com.morac &&
         typeof com.morac.SessionManagerAddon == "object";
@@ -63,7 +63,7 @@ var TMP_extensionsCompatibility = {
       document.getElementById("main-window").setAttribute("gscltTMPinstalled", true);
       let func = ["_setupForOtherExtensions", "enableCustomDragDropMode"];
       let GlaxChrome = window.GlaxChrome.CLT.DragDropManager;
-      func.forEach(function(aFn) {
+      func.forEach(aFn => {
         if (aFn in GlaxChrome) {
           Tabmix.changeCode(GlaxChrome, "GlaxChrome.CLT.DragDropManager." + aFn)._replace(
             '{', '{var TabDNDObserver = TMP_tabDNDObserver;',
@@ -91,7 +91,7 @@ var TMP_extensionsCompatibility = {
     if ("SecondSearchBrowser" in window && SecondSearchBrowser.prototype) {
       let func = ["canOpenNewTab", "loadForSearch", "checkToDoSearch"];
       let SSB = SecondSearchBrowser.prototype;
-      func.forEach(function(aFn) {
+      func.forEach(aFn => {
         if (aFn in SSB && SSB[aFn].toString().indexOf("TM_init") != -1) {
           Tabmix.changeCode(SSB, "SecondSearchBrowser.prototype." + aFn)._replace(
             'TM_init', 'Tabmix.startup'
@@ -221,6 +221,14 @@ var TMP_extensionsCompatibility = {
   },
 
   onWindowOpen: function TMP_EC_onWindowOpen() {
+    // https://addons.mozilla.org/firefox/addon/tabgroups-manager-revived
+    // TabGroupsManager.OverrideMethod set TabmixSessionManager.loadOneWindow to string by error
+    if (Tabmix.extensions.tabGroupManager &&
+        typeof TabmixSessionManager.loadOneWindow == "string") {
+      Tabmix._makeCode("TabmixSessionManager.loadOneWindow", TabmixSessionManager.loadOneWindow);
+      Tabmix.log("typeof TabmixSessionManager.loadOneWindow " + typeof TabmixSessionManager.loadOneWindow);
+    }
+
     this.setVerticalTabs();
 
     // Look for RSS/Atom News Reader
@@ -469,7 +477,7 @@ var TMP_extensionsCompatibility = {
     }
   },
 
-  setVerticalTabs: function() {
+  setVerticalTabs() {
     // https://addons.mozilla.org/EN-US/firefox/addon/vertical-tabs/
     // https://addons.mozilla.org/en-US/firefox/addon/vertical-tabs-reloaded/
     // https://addons.mozilla.org/EN-US/firefox/addon/side-tabs/
@@ -487,7 +495,7 @@ var TMP_extensionsCompatibility = {
 };
 
 TMP_extensionsCompatibility.RSSTICKER = {
-  init: function() {
+  init() {
     Tabmix.changeCode(RSSTICKER, "RSSTICKER.writeFeed")._replace(
       'tbb.setAttribute("onclick"',
       'tbb.setAttribute("onclick", "this.onClick(event);");\
@@ -500,7 +508,7 @@ TMP_extensionsCompatibility.RSSTICKER = {
     ).toCode();
   },
 
-  onClick: function(event) {
+  onClick(event) {
     if (event.ctrlKey) {
       this.markAsRead(true);
     } else if ((this.parent.alwaysOpenInNewTab && (event.which == 1)) || (event.which == 2)) {
@@ -510,7 +518,7 @@ TMP_extensionsCompatibility.RSSTICKER = {
     }
   },
 
-  onContextOpen: function(target) {
+  onContextOpen(target) {
     if (!target) {
       if (Tabmix.whereToOpen(null).lock)
         this.parent.browser.openInNewTab(this.href);
@@ -532,23 +540,23 @@ TMP_extensionsCompatibility.RSSTICKER = {
 // prevent Wizz RSS from load pages in locked tabs
 TMP_extensionsCompatibility.wizzrss = {
   started: null,
-  init: function() {
+  init() {
     if (this.started)
       return;
     this.started = true;
     var codeToReplace = /getContentBrowser\(\).loadURI|contentBrowser.loadURI/g;
     const newCode = "TMP_extensionsCompatibility.wizzrss.openURI";
     var _functions = ["addFeedbase", "validate", "gohome", "tryagain", "promptStuff",
-                      "doSearch", "viewLog", "renderItem", "playEnc", "renderAllEnc", "playAllEnc",
-                      "gotoLink", "itemLinkClick", "itemListClick"];
+      "doSearch", "viewLog", "renderItem", "playEnc", "renderAllEnc", "playAllEnc",
+      "gotoLink", "itemLinkClick", "itemListClick"];
 
-    _functions.forEach(function(_function) {
+    _functions.forEach(_function => {
       if (_function in window)
         Tabmix.changeCode(window, _function)._replace(codeToReplace, newCode).toCode();
     });
   },
 
-  openURI: function(uri) {
+  openURI(uri) {
     var w = Tabmix.getTopWin();
     var tabBrowser = w.gBrowser;
 
@@ -564,7 +572,7 @@ TMP_extensionsCompatibility.wizzrss = {
 
 // prevent Newsfox from load pages in locked tabs
 TMP_extensionsCompatibility.newsfox = {
-  init: function() {
+  init() {
     Tabmix.changeCode(window, "openNewsfox")._replace(
       /if \(newTab\) {/,
       'newTab = newTab || Tabmix.whereToOpen(null).lock; \
@@ -583,12 +591,12 @@ TMP_extensionsCompatibility.treeStyleTab = {
   installed: false,
   errorMsg: "Error in Tabmix when trying to load compatible functions with TreeStyleTab extension",
 
-  preInit: function() {
+  preInit() {
     let tstHelper = TreeStyleTabWindowHelper;
     if (typeof tstHelper.overrideExtensionsPreInit == "function") {
-      // overrideExtensionsPreInit look for 'gBrowser.restoreTab' in tablib.init
-      tablib._init = tablib.init;
-      tablib.init = function() {
+      // overrideExtensionsPreInit look for 'gBrowser.restoreTab' in Tabmix.tablib.init
+      Tabmix.tablib._init = Tabmix.tablib.init;
+      Tabmix.tablib.init = function() {
         this._init();
         /*
           var newTab = null
@@ -618,7 +626,7 @@ TMP_extensionsCompatibility.treeStyleTab = {
     }
   },
 
-  onContentLoaded: function() {
+  onContentLoaded() {
     // workaround, with version 0.15.2015061300a003855
     // gBrowser.treeStyleTab.initTabContentsOrder throw on Firefox 41+
     Tabmix.TST_initTabContentsOrder = function() {
@@ -660,14 +668,14 @@ TMP_extensionsCompatibility.treeStyleTab = {
       };
       gBrowser.tabContainer.addEventListener("TabSelect", ontabselect, true);
       window.addEventListener("unload", function onunload() {
-        window.removeEventListener("unload", onunload, false);
+        window.removeEventListener("unload", onunload);
         gBrowser.tabContainer.removeEventListener("TabSelect", ontabselect, true);
-      }, false);
+      });
     }
 
     // we removed TMP_howToOpen function 2011-11-15
     if ("TreeStyleTabWindowHelper" in window && TreeStyleTabWindowHelper.overrideExtensionsAfterBrowserInit) {
-      Tabmix.changeCode(TreeStyleTabWindowHelper, "TreeStyleTabWindowHelper.overrideExtensionsAfterBrowserInit")._replace(
+      Tabmix.changeCode(TreeStyleTabWindowHelper, "TreeStyleTabWindowHelper.overrideExtensionsAfterBrowserInit", {silent: true})._replace(
         /eval\(["|']window\.TMP_howToOpen/,
         'if (false) $&'
       ).toCode();
@@ -675,7 +683,7 @@ TMP_extensionsCompatibility.treeStyleTab = {
 
     // we removed TMP_openTabNext function 2011-11-15
     if ("TreeStyleTabWindowHelper" in window && TreeStyleTabWindowHelper.overrideExtensionsDelayed) {
-      Tabmix.changeCode(TreeStyleTabWindowHelper, "TreeStyleTabWindowHelper.overrideExtensionsDelayed")._replace(
+      Tabmix.changeCode(TreeStyleTabWindowHelper, "TreeStyleTabWindowHelper.overrideExtensionsDelayed", {silent: true})._replace(
         'var newTab',
         'gContextMenu.linkURL = url;'
       )._replace(
@@ -688,7 +696,7 @@ TMP_extensionsCompatibility.treeStyleTab = {
     }
   },
 
-  onWindowLoaded: function() {
+  onWindowLoaded() {
     // we don't need this hack since treestyletab version 0.16.2016021602
     if (typeof PlacesUIUtils.__treestyletab__openTabset != "function") {
       /**
@@ -741,13 +749,13 @@ TMP_extensionsCompatibility.treeStyleTab = {
       ).toCode();
       // Added 2011-11-09, i'm not sure we really need it, Tabmix.loadTabs call gBrowser.loadTabs
       Tabmix.changeCode(TabmixContext, "TabmixContext.openMultipleLinks")._replace(
-        /(Tabmix.loadTabs\([^\)]+\);)/g,
+        /(Tabmix.loadTabs\([^)]+\);)/g,
         'TreeStyleTabService.readyToOpenChildTab(gBrowser, true); $1 TreeStyleTabService.stopToOpenChildTab(gBrowser);'
       ).toCode();
     }
   },
 
-  onBeforeNewTabCommand: function(tab, openTabNext) {
+  onBeforeNewTabCommand(tab, openTabNext) {
     if (!this.installed) {
       return;
     }
@@ -759,7 +767,7 @@ TMP_extensionsCompatibility.treeStyleTab = {
   },
 
   // Don't call openNewTabNext if treeStyleTab already set readiedToAttachNewTab
-  checkToOpenTabNext: function(tab, check) {
+  checkToOpenTabNext(tab, check) {
     if (this.installed && check &&
         !gBrowser.treeStyleTab.checkToOpenChildTab(tab)) {
       this.openNewTabNext(tab, true);
@@ -768,7 +776,7 @@ TMP_extensionsCompatibility.treeStyleTab = {
 
   // instruct treeStyleTab to use 'kNEWTAB_OPEN_AS_NEXT_SIBLING' when our preference
   // is to open the tab next
-  openNewTabNext: function(tab, openTabNext, clean) {
+  openNewTabNext(tab, openTabNext, clean) {
     if (!this.installed || !openTabNext) {
       return;
     }
@@ -797,7 +805,10 @@ TMP_extensionsCompatibility.treeStyleTab = {
         return false;
       let nextTab = tst.getNextSiblingTab(baseTab);
       if (parentTab) {
-        return tst.readyToOpenChildTab(parentTab, false, nextTab);
+        return tst.readyToOpenChildTab(parentTab, false, {
+          insertBefore: nextTab,
+          insertAfter: baseTab
+        });
       } else if (nextTab) {
         ownerBrowser.treeStyleTab.readiedToAttachNewTab = true;
         ownerBrowser.treeStyleTab.parentTab = null;
@@ -817,4 +828,17 @@ TMP_extensionsCompatibility.treeStyleTab = {
       }, 0);
     }
   },
+
+  getProperties(tab) {
+    if (!this.installed) {
+      return "";
+    }
+
+    let props = "TreeStyleTabWindowHelper" in window &&
+        TreeStyleTabWindowHelper.extraProperties || [];
+    props = props.filter(prop => tab.hasAttribute(prop))
+        .map(prop => `${prop}=${encodeURI(tab.getAttribute(prop))}`)
+        .join(" ");
+    return props.length ? ` ${props}` : "";
+  },
 };
