@@ -24,17 +24,17 @@ Tabmix.linkHandling_init = function TMP_TBP_init() {
     if (autoComplete) {
       // https://addons.mozilla.org/en-US/firefox/addon/quieturl/
       let fn = typeof autoComplete._QuietUrlPopupClickOld == "function" ?
-               "_QuietUrlPopupClickOld" : "PopupAutoCompleteRichResult.onPopupClick";
+        "_QuietUrlPopupClickOld" : "PopupAutoCompleteRichResult.onPopupClick";
       let n = '\n            ';
       this.changeCode(autoComplete, fn)._replace(
-          /openUILink\(url, aEvent.*\);/,
-          'var tabmixOptions = typeof options == "object" ? options : {};' + n +
-          'var isBlankTab = gBrowser.isBlankNotBusyTab(gBrowser.mCurrentTab);' + n +
-          'var where = isBlankTab ? "current" : whereToOpenLink(aEvent);' + n +
-          'var pref = "extensions.tabmix.loadUrlInBackground";' + n +
-          'tabmixOptions.inBackground = Services.prefs.getBoolPref(pref);' + n +
-          'tabmixOptions.initiatingDoc = aEvent ? aEvent.target.ownerDocument : null;' + n +
-          'openUILinkIn(url, where, tabmixOptions);'
+        /openUILink\(url, aEvent.*\);/,
+        'var tabmixOptions = typeof options == "object" ? options : {};' + n +
+        'var isBlankTab = gBrowser.isBlankNotBusyTab(gBrowser.mCurrentTab);' + n +
+        'var where = isBlankTab ? "current" : whereToOpenLink(aEvent);' + n +
+        'var pref = "extensions.tabmix.loadUrlInBackground";' + n +
+        'tabmixOptions.inBackground = Services.prefs.getBoolPref(pref);' + n +
+        'tabmixOptions.initiatingDoc = aEvent ? aEvent.target.ownerDocument : null;' + n +
+        'openUILinkIn(url, where, tabmixOptions);'
       ).toCode();
     }
   }
@@ -84,7 +84,7 @@ Tabmix.beforeBrowserInitOnLoad = function() {
         get isStopped() {
           return TabmixSvc.SessionStore._loadState === 0; // STATE_STOPPED
         },
-        setRunning: function() {
+        setRunning() {
           TabmixSvc.SessionStore._loadState = 1; // STATE_RUNNING
         }
       };
@@ -101,7 +101,7 @@ Tabmix.beforeBrowserInitOnLoad = function() {
       let hasFirstArgument = window.arguments && window.arguments[0];
       if (hasFirstArgument) {
         let defaultArgs = Cc["@mozilla.org/browser/clh;1"]
-                            .getService(Ci.nsIBrowserHandler).defaultArgs;
+            .getService(Ci.nsIBrowserHandler).defaultArgs;
         if (window.arguments[0] == defaultArgs) {
           SM.overrideHomepage = window.arguments[0];
           window.arguments[0] = null;
@@ -195,12 +195,18 @@ Tabmix.beforeStartup = function TMP_beforeStartup(tabBrowser, aTabContainer) {
 
   tabBrowser.isBlankBrowser = function TMP_isBlankBrowser(aBrowser, aboutBlank) {
     try {
+      if (Tabmix.isVersion(550)) {
+        let tab = this.getTabForBrowser(aBrowser);
+        if (tab.hasAttribute("pending")) {
+          return TMP_SessionStore.isBlankPendingTab(tab);
+        }
+      }
       if (!aBrowser || !aBrowser.currentURI)
         return true;
       if (aBrowser.canGoForward || aBrowser.canGoBack)
         return false;
       return aboutBlank ? aBrowser.currentURI.spec == TabmixSvc.aboutBlank :
-      Tabmix.isNewTabUrls(aBrowser.currentURI.spec);
+        Tabmix.isNewTabUrls(aBrowser.currentURI.spec);
     } catch (ex) {
       Tabmix.assert(ex);
       return true;
@@ -312,9 +318,11 @@ Tabmix.adjustTabstrip = function tabContainer_adjustTabstrip(skipUpdateScrollSta
       break;
     case 5:
       this.removeAttribute("closebuttons-hover");
-      if (tabsCount < 3)
+      if (Tabmix.isVersion(550)) {
+        this.tabmix_adjustTabstrip();
+      } else if (tabsCount < 3) {
         this.setAttribute("closebuttons", "alltabs");
-      else {
+      } else {
         // make sure not to check collapsed, hidden or pinned tabs for width
         let tab = TMP_TabView.checkTabs(tabs);
         if (tab && tab.getBoundingClientRect().width > this.mTabClipWidth)
@@ -325,10 +333,10 @@ Tabmix.adjustTabstrip = function tabContainer_adjustTabstrip(skipUpdateScrollSta
       break;
   }
 
- /**
-  *  Don't use return in this function
-  *  TreeStyleTab add some code at the end
-  */
+  /**
+   *  Don't use return in this function
+   *  TreeStyleTab add some code at the end
+   */
   let transitionend = Tabmix.callerTrace("onxbltransitionend");
   if (tabsCount == 1) {
     let tab = this.selectedItem;
