@@ -28,7 +28,7 @@ Tabmix.openOptionsDialog = function TMP_openDialog(panel) {
     (appearanceWin || filetypeWin || promptWin || tabmixOptionsWin).focus();
   } else {
     window.openDialog("chrome://tabmixplus/content/preferences/preferences.xul", "Tab Mix Plus",
-        "chrome,titlebar,toolbar,close,dialog=no,centerscreen", panel || null);
+      "chrome,titlebar,toolbar,close,dialog=no,centerscreen", panel || null);
   }
 };
 
@@ -36,31 +36,31 @@ Tabmix.openOptionsDialog = function TMP_openDialog(panel) {
 // Speed-Dial, Fast-Dial, TabGroupManager
 function TMP_BrowserOpenTab(aEvent, aTab, replaceLastTab) {
   var newTabContent = replaceLastTab ? Tabmix.prefs.getIntPref("replaceLastTabWith.type") :
-                                       Tabmix.prefs.getIntPref("loadOnNewTab.type");
+    Tabmix.prefs.getIntPref("loadOnNewTab.type");
   var url;
   var newTabUrl = BROWSER_NEW_TAB_URL;
   var selectedTab = gBrowser.selectedTab;
   switch (newTabContent) {
-    case 0 : // blank tab, by default
+    case 0: // blank tab, by default
       url = "about:blank";
       break;
-    case 1 : // home page
+    case 1: // home page
       url = gHomeButton.getHomePage().split("|")[0];
       break;
-    case 2 : // current URI
+    case 2: // current URI
       var currentURI = gBrowser.currentURI;
       url = currentURI ? currentURI.spec : newTabUrl;
       break;
-    case 3 : { // duplicate tab
+    case 3: { // duplicate tab
       let currentUrl = gBrowser.currentURI.spec;
       let dupTab = gBrowser.duplicateTab(selectedTab, null, null, null, true);
       Tabmix.clearUrlBar(dupTab, currentUrl, true);
       return dupTab;
     }
-    case 4 : {// user url
+    case 4: {// user url
       if (replaceLastTab || !Tabmix.isVersion(410) || TabmixSvc.isCyberfox) {
         let prefName = replaceLastTab ? "extensions.tabmix.replaceLastTabWith.newtab.url" :
-                       TabmixSvc.newtabUrl;
+          TabmixSvc.newtabUrl;
         try {
           url = Services.prefs.getComplexValue(prefName, Ci.nsISupportsString).data;
           if (newTabUrl == "about:privatebrowsing" && url == TabmixSvc.aboutNewtab)
@@ -99,7 +99,7 @@ function TMP_BrowserOpenTab(aEvent, aTab, replaceLastTab) {
 
   // always select new tab when replacing last tab
   var loadInBackground = replaceLastTab ? false :
-                         Tabmix.prefs.getBoolPref("loadNewInBackground");
+    Tabmix.prefs.getBoolPref("loadNewInBackground");
   let baseTab = aTab && aTab.localName == "tab" ? aTab : null;
   let openTabNext = baseTab || !replaceLastTab && Tabmix.prefs.getBoolPref("openNewTabNext");
   // Let accel-click and middle-click on the new tab button toggle openTabNext preference
@@ -324,6 +324,18 @@ Tabmix.restoreTabState = function TMP_restoreTabState(aTab) {
   aTab.removeAttribute("maxwidth");
 };
 
+Tabmix.isPendingTab = function(aTab) {
+  const pending = aTab.hasAttribute("pending") || aTab.hasAttribute("tabmix_pending");
+  if (pending) {
+    if (TMP_Places.restoringTabs.indexOf(aTab) > -1) {
+      // don't show tab as pending when bookmarks restore_on_demand is false
+      return TMP_Places.bookmarksOnDemand;
+    }
+    return true;
+  }
+  return false;
+};
+
 Tabmix.setTabStyle = function(aTab, boldChanged) {
   if (!aTab)
     return;
@@ -331,8 +343,8 @@ Tabmix.setTabStyle = function(aTab, boldChanged) {
   let isSelected = aTab.getAttribute(TabmixSvc.selectedAtt) == "true";
   // if pending tab is blank we don't style it as unload or unread
   if (!isSelected && Tabmix.prefs.getBoolPref("unloadedTab") &&
-      (aTab.hasAttribute("pending") || aTab.hasAttribute("tabmix_pending"))) {
-    style = aTab.hasAttribute("visited") ||
+      this.isPendingTab(aTab)) {
+    style = aTab.pinned || aTab.hasAttribute("visited") ||
       TMP_SessionStore.isBlankPendingTab(aTab) ? "other" : "unloaded";
   } else if (!isSelected && Tabmix.prefs.getBoolPref("unreadTab") &&
       !aTab.hasAttribute("visited") && !isTabEmpty(aTab)) {
