@@ -29,6 +29,17 @@ var TMP_tabDNDObserver = {
     }
 
     tabBar.moveTabOnDragging = Tabmix.prefs.getBoolPref("moveTabOnDragging");
+
+    // https://addons.mozilla.org/en-US/firefox/addon/multiple-tab-handler/
+    const tabsDragUtils = "piro.sakura.ne.jp" in window &&
+      "tabsDragUtils" in window["piro.sakura.ne.jp"];
+    Tabmix.handleAnimateTabMove = function(dragContext) {
+      if (gBrowser.tabContainer.orient != "horizontal") {
+        return false;
+      }
+      return !dragContext || !dragContext.draggedTabs ||
+          dragContext.draggedTabs.length == 1;
+    };
     // Determine what tab we're dragging over.
     // * In tabmix tabs can have different width
     // * Point of reference is the start of the dragged tab when
@@ -36,7 +47,11 @@ var TMP_tabDNDObserver = {
     //   is before (for dragging left) or after (for dragging right)
     //   the middle of a background tab, the dragged tab would take that
     //   tab's position when dropped.
-    Tabmix.changeCode(tabBar, "gBrowser.tabContainer._animateTabMove")._replace(
+    let newCode = Tabmix.changeCode(tabBar, "gBrowser.tabContainer._animateTabMove")._replace(
+      'if (this.getAttribute("movingtab")',
+      `let tabmixHandleMove = Tabmix.handleAnimateTabMove(typeof TDUContext == "object" ? TDUContext : null);
+          $&`
+    )._replace(
       'this.selectedItem = draggedTab;',
       'if (Tabmix.prefs.getBoolPref("selectTabOnMouseDown"))\n\
             $&\n\
@@ -44,21 +59,54 @@ var TMP_tabDNDObserver = {
             this.setAttribute("movingBackgroundTab", true);\n\
             draggedTab.setAttribute("dragged", true);\n\
           }'
+    )._replace(
+      'draggedTab._dragData.animLastScreenX = screenX;',
+      'let draggingRight = screenX > draggedTab._dragData.animLastScreenX;\n          ' +
+      '$&', {check: Tabmix.isVersion(520) && !tabsDragUtils}
     )._replace(
       'let tabCenter = tabScreenX + translateX + tabWidth / 2;',
-      'let tabCenter = tabScreenX + translateX + draggingRight * tabWidth;'
+      'let tabCenter = tabScreenX + translateX + (tabmixHandleMove ? draggingRight * tabWidth : tabWidth / 2);'
     )._replace(
-      /let screenX = boxObject.*;/,
+      tabsDragUtils ? /screenX = boxObject\[TDUContext.*;/ :
+        /screenX = boxObject.*;/,
       '$&\n            ' +
-      'let halfWidth = boxObject.width / 2;\n            ' +
-      'screenX += draggingRight * halfWidth;'
+      `let halfWidth;
+            if (tabmixHandleMove) {
+              halfWidth = boxObject.width / 2;
+              screenX += draggingRight * halfWidth;
+            }`
     )._replace(
-      /screenX \+ boxObject.* < tabCenter/,
-      'screenX + halfWidth < tabCenter'
+      tabsDragUtils ? /screenX \+ boxObject\[TDUContext.* < tabCenter/ :
+        /screenX \+ boxObject.* < tabCenter/,
+      'tabmixHandleMove ? screenX + halfWidth < tabCenter : $&'
+    )._replace(
+      'screenX > TDUContext.lastTabCenter',
+      'tabmixHandleMove ? screenX > tabCenter : $&',
+      {check: tabsDragUtils}
     )._replace(
       'newIndex >= oldIndex',
-      'rtl ? $& : draggingRight && newIndex > -1'
-    ).toCode();
+      'rtl || !tabmixHandleMove ? $& : draggingRight && newIndex > -1'
+    );
+    if (tabsDragUtils) {
+      const topic = "browser-delayed-startup-finished";
+      const observer = function(subject) {
+        if (subject == window) {
+          Services.obs.removeObserver(observer, topic);
+          // update for multiple-tab-handler version 0.8.2017061501
+          if (!newCode.value.includes("draggingRight = screenX")) {
+            newCode.value = newCode.value.replace(
+              'draggedTab._dragData.animLastScreenX = screenX;',
+              'let draggingRight = screenX > draggedTab._dragData.animLastScreenX;\n          ' +
+              '$&'
+            );
+          }
+          newCode.toCode();
+        }
+      };
+      Services.obs.addObserver(observer, topic, false);
+    } else {
+      newCode.toCode();
+    }
 
     Tabmix.changeCode(tabBar, "gBrowser.tabContainer._finishAnimateTabMove")._replace(
       /(})(\)?)$/,
@@ -97,10 +145,14 @@ var TMP_tabDNDObserver = {
     tabBar._tabDropIndicator.style.MozTransform = "translate(0px, 0px)";
 
     if (Tabmix.isVersion(280)) {
-      let t = document.getElementById("TabsToolbar").parentNode.getBoundingClientRect();
-      let r = gBrowser.tabContainer.getBoundingClientRect();
-      let c = document.getElementById("content-deck").getBoundingClientRect();
-      this.onLastToolbar = Math.abs(t.bottom - r.bottom) < 2 && Math.abs(r.bottom - c.top) < 2;
+      window.requestAnimationFrame(() => {
+        window.requestAnimationFrame(() => {
+          let t = Tabmix.getBoundsWithoutFlushing(document.getElementById("TabsToolbar").parentNode);
+          let r = Tabmix.getBoundsWithoutFlushing(gBrowser.tabContainer);
+          let c = Tabmix.getBoundsWithoutFlushing(document.getElementById("content-deck"));
+          this.onLastToolbar = Math.abs(t.bottom - r.bottom) < 2 && Math.abs(r.bottom - c.top) < 2;
+        });
+      });
     }
   },
 
@@ -108,10 +160,11 @@ var TMP_tabDNDObserver = {
     return Tabmix.isVersion(280) && gBrowser.tabContainer._isCustomizing;
   },
 
-  onDragStart: function(event, tabmixDragstart) {
+  onDragStart(event, tabmixDragstart) {
     // we get here on capturing phase before "tabbrowser-close-tab-button"
     // binding stop the event propagation
-    if (event.originalTarget.getAttribute("anonid") == "tmp-close-button") {
+    if (event.originalTarget && typeof event.originalTarget.getAttribute == "function" &&
+        event.originalTarget.getAttribute("anonid") == "tmp-close-button") {
       event.stopPropagation();
       return;
     }
@@ -138,52 +191,71 @@ var TMP_tabDNDObserver = {
     // Set the cursor to an arrow during tab drags.
     dt.mozCursor = "default";
 
+    if (Tabmix.isVersion(550)) {
+      // Set the tab as the source of the drag, which ensures we have a stable
+      // node to deliver the `dragend` event.  See bug 1345473.
+      dt.addElement(tab);
+    }
+
     // Create a canvas to which we capture the current tab.
     // Until canvas is HiDPI-aware (bug 780362), we need to scale the desired
     // canvas size (in CSS pixels) to the window's backing resolution in order
     // to get a full-resolution drag image for use on HiDPI displays.
     let windowUtils = window.getInterface(Ci.nsIDOMWindowUtils);
     let scale = windowUtils.screenPixelsPerCSSPixel / windowUtils.fullZoom;
-    let canvas = tabBar._dndCanvas ? tabBar._dndCanvas :
-                 document.createElementNS("http://www.w3.org/1999/xhtml", "canvas");
-    canvas.mozOpaque = true;
+    let canvas = tabBar._dndCanvas;
+    if (!canvas) {
+      tabBar._dndCanvas = canvas =
+          document.createElementNS("http://www.w3.org/1999/xhtml", "canvas");
+      canvas.style.width = "100%";
+      canvas.style.height = "100%";
+      canvas.mozOpaque = true;
+    }
+
     canvas.width = 160 * scale;
     canvas.height = 90 * scale;
-    let toDrag;
+    let toDrag = canvas;
     let dragImageOffsetX = -16;
     let dragImageOffsetY = TabmixTabbar.visibleRows == 1 ? -16 : -30;
     if (gMultiProcessBrowser) {
       let context = canvas.getContext('2d');
       context.fillStyle = "white";
       context.fillRect(0, 0, canvas.width, canvas.height);
-      // Create a panel to use it in setDragImage
-      // which will tell xul to render a panel that follows
-      // the pointer while a dnd session is on.
-      if (!tabBar._dndPanel) {
-        tabBar._dndCanvas = canvas;
-        tabBar._dndPanel = document.createElement("panel");
-        tabBar._dndPanel.setAttribute("type", "drag");
-        tabBar._dndPanel.className = "dragfeedback-tab";
-        let wrapper = document.createElementNS("http://www.w3.org/1999/xhtml", "div");
-        wrapper.style.width = "160px";
-        wrapper.style.height = "90px";
-        wrapper.appendChild(canvas);
-        canvas.style.width = "100%";
-        canvas.style.height = "100%";
-        tabBar._dndPanel.appendChild(wrapper);
-        document.documentElement.appendChild(tabBar._dndPanel);
+
+      let captureListener;
+      let platform = AppConstants.platform;
+      // On Windows and Mac we can update the drag image during a drag
+      // using updateDragImage. On Linux, we can use a panel.
+      if (Tabmix.isVersion(530) && (platform == "win" || platform == "macosx")) {
+        captureListener = function() {
+          dt.updateDragImage(canvas, dragImageOffsetX, dragImageOffsetY);
+        };
+      } else {
+        // Create a panel to use it in setDragImage
+        // which will tell xul to render a panel that follows
+        // the pointer while a dnd session is on.
+        if (!tabBar._dndPanel) {
+          tabBar._dndCanvas = canvas;
+          tabBar._dndPanel = document.createElement("panel");
+          tabBar._dndPanel.className = "dragfeedback-tab";
+          tabBar._dndPanel.setAttribute("type", "drag");
+          let wrapper = document.createElementNS("http://www.w3.org/1999/xhtml", "div");
+          wrapper.style.width = "160px";
+          wrapper.style.height = "90px";
+          wrapper.appendChild(canvas);
+          tabBar._dndPanel.appendChild(wrapper);
+          document.documentElement.appendChild(tabBar._dndPanel);
+        }
+        toDrag = tabBar._dndPanel;
       }
       // PageThumb is async with e10s but that's fine
-      // since we can update the panel during the dnd.
-      PageThumbs.captureToCanvas(browser, canvas);
-      toDrag = tabBar._dndPanel;
+      // since we can update the image during the dnd.
+      PageThumbs.captureToCanvas(browser, canvas, captureListener);
     } else {
       // For the non e10s case we can just use PageThumbs
-      // sync. No need for xul magic, the native dnd will
-      // be fine, so let's use the canvas for setDragImage.
+      // sync, so let's use the canvas for setDragImage.
       let elm = Tabmix.isVersion(360) ? browser : browser.contentWindow;
       PageThumbs.captureToCanvas(elm, canvas);
-      toDrag = canvas;
       dragImageOffsetX *= scale;
       dragImageOffsetY *= scale;
     }
@@ -217,14 +289,14 @@ var TMP_tabDNDObserver = {
     var newIndex = this._getDNDIndex(event);
     var oldIndex = dragType != this.DRAG_LINK ? sourceNode._tPos : -1;
     var left_right; // 1:right, 0: left, -1: drop link on tab to replace tab
-///XXX check if we need here visibleTabs insteadof gBrowser.tabs
-///    check with groups with or without pinned tabs
+    ///XXX check if we need here visibleTabs insteadof gBrowser.tabs
+    ///    check with groups with or without pinned tabs
     if (newIndex < gBrowser.tabs.length)
       left_right = this.getLeft_Right(event, newIndex, oldIndex, dragType);
     else {
       newIndex = dragType != this.DRAG_TAB_IN_SAME_WINDOW &&
                  Tabmix.getOpenTabNextPref(dragType == this.DRAG_LINK) ?
-                     tabBar.selectedIndex : gBrowser.tabs.length - 1;
+        tabBar.selectedIndex : gBrowser.tabs.length - 1;
       left_right = 1;
     }
 
@@ -241,16 +313,17 @@ var TMP_tabDNDObserver = {
     if (replaceTab && !isCopy) {
       let disAllowDrop, targetTab = gBrowser.tabs[newIndex];
       if (targetTab.getAttribute("locked") && !gBrowser.isBlankNotBusyTab(targetTab)) {
+        // Pass true to disallow dropping javascript: or data: urls
+        let links;
         try {
-          var url = browserDragAndDrop.drop(event, {});
-          if (!url || !url.length || url.indexOf(" ", 0) != -1 ||
-              /^\s*(javascript|data):/.test(url))
-            url = null;
-
+          if (Tabmix.isVersion(520)) {
+            links = browserDragAndDrop.dropLinks(event, true);
+          } else {
+            links = [{url: browserDragAndDrop.drop(event, {}, true)}];
+          }
+          const url = links && links.length ? links[0].url : null;
           disAllowDrop = url ? !Tabmix.ContentClick.isUrlForDownload(url) : true;
-        } catch (ex) {
-          Tabmix.assert(ex);
-        }
+        } catch (ex) {}
 
         if (disAllowDrop)
           dt.effectAllowed = "none";
@@ -281,7 +354,7 @@ var TMP_tabDNDObserver = {
     // show Drag & Drop message
     if (dragType == this.DRAG_LINK) {
       this.gMsg = event.originalTarget.getAttribute("command") == "cmd_newNavigatorTab" ?
-                              this.gBackupLabel : this.draglink;
+        this.gBackupLabel : this.draglink;
       if (event.target.localName != "tab" && event.target.localName != "tabs")
         this.gMsg = this.gBackupLabel;
       var statusTextFld = document.getElementById("statusbar-display");
@@ -324,8 +397,9 @@ var TMP_tabDNDObserver = {
       }
       if (_scroll) {
         let scrollIncrement = TabmixTabbar.isMultiRow ?
-            Math.round(tabStrip._singleRowHeight / 6) : tabStrip.scrollIncrement;
-        tabStrip.scrollByPixels((ltr ? _scroll : -_scroll) * scrollIncrement, true);
+          Math.round(tabStrip._singleRowHeight / 6) : tabStrip.scrollIncrement;
+        const instantScroll = !Tabmix.isVersion(570);
+        tabStrip.scrollByPixels((ltr ? _scroll : -_scroll) * scrollIncrement, instantScroll);
         hideIndicator = true;
       }
     }
@@ -348,6 +422,78 @@ var TMP_tabDNDObserver = {
     this.setDragmark(newIndex, left_right);
   },
 
+  // built-in drop method doesn't take into account different tab width
+  drop(event) {
+    if (!Tabmix.isVersion(550)) {
+      return;
+    }
+
+    var tabBar = gBrowser.tabContainer;
+    var dt = event.dataTransfer;
+    var dropEffect = dt.dropEffect;
+    var draggedTab;
+    if (dt.mozTypesAt(0)[0] == TAB_DROP_TYPE) { // tab copy or move
+      draggedTab = dt.mozGetDataAt(TAB_DROP_TYPE, 0);
+      // not our drop then
+      if (!draggedTab)
+        return;
+    }
+
+    // fall back to build-in drop method
+    if (!draggedTab || dropEffect == "copy" || draggedTab.parentNode != tabBar) {
+      return;
+    }
+
+    tabBar._tabDropIndicator.collapsed = true;
+    event.stopPropagation();
+    let oldTranslateX = draggedTab._dragData && draggedTab._dragData.translateX;
+    let dropIndex = "animDropIndex" in draggedTab._dragData &&
+        draggedTab._dragData.animDropIndex;
+    if (dropIndex && dropIndex > draggedTab._tPos)
+      dropIndex--;
+
+    let newTranslateX = 0;
+    if (oldTranslateX && dropIndex !== false) {
+      let tabIndex = draggedTab._tPos;
+      if (dropIndex > tabIndex) {
+        for (let i = tabIndex + 1; i <= dropIndex; i++) {
+          newTranslateX += Tabmix.getBoundsWithoutFlushing(gBrowser.tabs[i]).width;
+        }
+      } else if (dropIndex < tabIndex) {
+        for (let i = dropIndex; i < tabIndex; i++) {
+          newTranslateX -= Tabmix.getBoundsWithoutFlushing(gBrowser.tabs[i]).width;
+        }
+      }
+    }
+
+    if (oldTranslateX && oldTranslateX != newTranslateX) {
+      draggedTab.setAttribute("tabdrop-samewindow", "true");
+      draggedTab.style.transform = "translateX(" + newTranslateX + "px)";
+      let onTransitionEnd = transitionendEvent => {
+        if (transitionendEvent.propertyName != "transform" ||
+            transitionendEvent.originalTarget != draggedTab) {
+          return;
+        }
+        draggedTab.removeEventListener("transitionend", onTransitionEnd);
+
+        draggedTab.removeAttribute("tabdrop-samewindow");
+
+        tabBar._finishAnimateTabMove();
+        if (dropIndex !== false)
+          gBrowser.moveTabTo(draggedTab, dropIndex);
+      };
+      draggedTab.addEventListener("transitionend", onTransitionEnd);
+    } else {
+      tabBar._finishAnimateTabMove();
+      if (dropIndex !== false)
+        gBrowser.moveTabTo(draggedTab, dropIndex);
+    }
+
+    if (draggedTab) {
+      delete draggedTab._dragData;
+    }
+  },
+
   onDrop: function minit_onDrop(event) {
     this.clearDragmark();
     this.updateStatusField();
@@ -379,7 +525,7 @@ var TMP_tabDNDObserver = {
     else {
       newIndex = dragType != this.DRAG_TAB_IN_SAME_WINDOW &&
                  Tabmix.getOpenTabNextPref(dragType == this.DRAG_LINK) ?
-                     gBrowser.tabContainer.selectedIndex : gBrowser.tabs.length - 1;
+        gBrowser.tabContainer.selectedIndex : gBrowser.tabs.length - 1;
       left_right = 1;
     }
 
@@ -433,7 +579,7 @@ var TMP_tabDNDObserver = {
         // ensure that the remoteness of the newly created browser is
         // appropriate for the URL of the tab being dragged in.
         gBrowser.updateBrowserRemotenessByURL(newBrowser,
-                                              draggedBrowserURL);
+          draggedBrowserURL);
       }
 
       // Stop the about:blank load
@@ -454,20 +600,48 @@ var TMP_tabDNDObserver = {
       gBrowser.updateCurrentBrowser(true);
     } else {
       // Pass true to disallow dropping javascript: or data: urls
-      let url;
+      let links;
       try {
-        url = browserDragAndDrop.drop(event, {}, true);
+        if (Tabmix.isVersion(520)) {
+          links = browserDragAndDrop.dropLinks(event, true);
+        } else {
+          links = [{url: browserDragAndDrop.drop(event, {}, true)}];
+        }
       } catch (ex) {}
 
-      if (!url)
+      if (!links || links.length === 0) {
         return;
+      }
 
       let bgLoad = Services.prefs.getBoolPref("browser.tabs.loadInBackground");
 
       if (event.shiftKey)
         bgLoad = !bgLoad; // shift Key reverse the pref
 
-      if (left_right > -1 && !Tabmix.ContentClick.isUrlForDownload(url)) {
+      const url = links[0].url;
+      const replaceCurrentTab = left_right == -1 || Tabmix.ContentClick.isUrlForDownload(url);
+      let tab;
+      if (replaceCurrentTab) {
+        tab = event.target.localName == "tab" ? event.target : gBrowser.tabs[newIndex];
+        // allow to load in locked tab
+        tab.linkedBrowser.tabmix_allowLoad = true;
+      }
+      if (Tabmix.isVersion(520)) {
+        let urls = links.map(link => link.url);
+        let params = {
+          inBackground: bgLoad,
+          replace: replaceCurrentTab,
+          allowThirdPartyFixup: true,
+          targetTab: tab,
+          newIndex: newIndex + left_right,
+          userContextId: gBrowser.tabContainer.selectedItem.getAttribute("usercontextid"),
+        };
+        if (Tabmix.isVersion(550)) {
+          params.triggeringPrincipal = dt.mozSourceNode ?
+            dt.mozSourceNode.nodePrincipal : Services.scriptSecurityManager.getSystemPrincipal();
+        }
+        gBrowser.tabContainer.tabbrowser.loadTabs(urls, params);
+      } else if (!replaceCurrentTab) {
         // We're adding a new tab.
         let newTab = gBrowser.loadOneTab(url, {
           inBackground: bgLoad,
@@ -477,11 +651,8 @@ var TMP_tabDNDObserver = {
         gBrowser.moveTabTo(newTab, newIndex + left_right);
       } else {
         // Load in an existing tab.
-        let tab = event.target.localName == "tab" ? event.target : gBrowser.tabs[newIndex];
         try {
           let browser = tab.linkedBrowser;
-          // allow to load in locked tab
-          browser.tabmix_allowLoad = true;
           let webNav = Ci.nsIWebNavigation;
           let flags = webNav.LOAD_FLAGS_ALLOW_THIRD_PARTY_FIXUP;
           if (Tabmix.isVersion(290))
@@ -503,8 +674,20 @@ var TMP_tabDNDObserver = {
 
   onDragEnd: function minit_onDragEnd(aEvent) {
     var tabBar = gBrowser.tabContainer;
-    if (!tabBar.useTabmixDnD(aEvent))
+
+    var dt = aEvent.dataTransfer;
+    var draggedTab = dt.mozGetDataAt(TAB_DROP_TYPE, 0);
+
+    if (!tabBar.useTabmixDnD(aEvent)) {
+      // Prevent this code from running if a tabdrop animation is
+      // running since calling _finishAnimateTabMove would clear
+      // any CSS transition that is running.
+      if (draggedTab.hasAttribute("tabdrop-samewindow")) {
+        return;
+      }
+
       tabBar._finishAnimateTabMove();
+    }
 
     if (this.draggedTab) {
       delete this.draggedTab.__tabmixDragStart;
@@ -512,8 +695,6 @@ var TMP_tabDNDObserver = {
       this.draggedTab = null;
     }
     // see comment in gBrowser.tabContainer.dragEnd
-    var dt = aEvent.dataTransfer;
-    var draggedTab = dt.mozGetDataAt(TAB_DROP_TYPE, 0);
     if (dt.mozUserCancelled || dt.dropEffect != "none" || this._isCustomizing) {
       delete draggedTab._dragData;
       return;
@@ -562,8 +743,8 @@ var TMP_tabDNDObserver = {
     // screen.availLeft et. al. only check the screen that this window is on,
     // but we want to look at the screen the tab is being dropped onto.
     var screen = Cc["@mozilla.org/gfx/screenmanager;1"]
-                   .getService(Ci.nsIScreenManager)
-                   .screenForRect(eX, eY, 1, 1);
+        .getService(Ci.nsIScreenManager)
+        .screenForRect(eX, eY, 1, 1);
     var fullX = {}, fullY = {}, fullWidth = {}, fullHeight = {};
     var availX = {}, availY = {}, availWidth = {}, availHeight = {};
     // get full screen rect and available rect, both in desktop pix
@@ -586,9 +767,9 @@ var TMP_tabDNDObserver = {
     var winWidth = Math.min(window.outerWidth, availWidth.value);
     var winHeight = Math.min(window.outerHeight, availHeight.value);
     var left = Math.min(Math.max(eX - draggedTab._dragData.offsetX, availX.value),
-        availX.value + availWidth.value - winWidth);
+      availX.value + availWidth.value - winWidth);
     var top = Math.min(Math.max(eY - draggedTab._dragData.offsetY, availY.value),
-        availY.value + availHeight.value - winHeight);
+      availY.value + availHeight.value - winHeight);
 
     delete draggedTab._dragData;
 
@@ -629,7 +810,7 @@ var TMP_tabDNDObserver = {
     this.updateStatusField();
   },
 
-  updateStatusField: function() {
+  updateStatusField() {
     var statusTextFld = document.getElementById("statusbar-display");
     if (statusTextFld && this.statusFieldChanged) {
       statusTextFld.label = "";
@@ -641,7 +822,7 @@ var TMP_tabDNDObserver = {
   },
 
   // get _tPos from group index
-  _getDNDIndex: function(aEvent) {
+  _getDNDIndex(aEvent) {
     var indexInGroup = this.getNewIndex(aEvent);
     var tabs = gBrowser.visibleTabs;
     var lastIndex = tabs.length - 1;
@@ -650,7 +831,7 @@ var TMP_tabDNDObserver = {
     return tabs[indexInGroup]._tPos;
   },
 
-  getNewIndex: function(event) {
+  getNewIndex(event) {
     let getTabRowNumber = (tab, top) => (tab.pinned ? 1 : Tabmix.tabsUtils.getTabRowNumber(tab, top));
     // if mX is less then the first tab return 0
     // check if mY is below the tab.... if yes go to next row
@@ -662,7 +843,7 @@ var TMP_tabDNDObserver = {
     var numTabs = tabs.length;
     if (!tabBar.hasAttribute("multibar")) {
       let i = event.target.localName == "tab" ?
-          Tabmix.visibleTabs.indexOf(event.target) : 0;
+        Tabmix.visibleTabs.indexOf(event.target) : 0;
       for (; i < numTabs; i++) {
         let tab = tabs[i];
         if (Tabmix.compare(mX, Tabmix.itemEnd(tab, Tabmix.ltr), Tabmix.ltr))
@@ -686,7 +867,7 @@ var TMP_tabDNDObserver = {
     return numTabs;
   },
 
-  getLeft_Right: function(event, newIndex, oldIndex, dragType) {
+  getLeft_Right(event, newIndex, oldIndex, dragType) {
     var mX = event.screenX;
     var left_right;
     var tab = gBrowser.tabs[newIndex];
@@ -718,7 +899,7 @@ var TMP_tabDNDObserver = {
     if (aSourceNode && aSourceNode instanceof XULElement && aSourceNode.localName == "tab") {
       if (aSourceNode.parentNode == gBrowser.tabContainer)
         return this.DRAG_TAB_IN_SAME_WINDOW; // 2
-      if (aSourceNode.ownerDocument.defaultView instanceof ChromeWindow &&
+      if (aSourceNode.ownerGlobal instanceof ChromeWindow &&
            aSourceNode.ownerDocument.documentElement.getAttribute("windowtype") == "navigator:browser")
         return this.DRAG_TAB_TO_NEW_WINDOW; // 1
     }
@@ -796,7 +977,7 @@ var TMP_tabDNDObserver = {
       ind.style.MozMarginStart = (-ind.clientWidth) + "px";
     }
 
-    this.dragmarkindex = {newIndex: newIndex, index: index};
+    this.dragmarkindex = {newIndex, index};
   },
 
   clearDragmark: function minit_clearDragmark() {
@@ -816,22 +997,22 @@ var TMP_tabDNDObserver = {
     this.dragmarkindex = null;
   },
 
-  setFirefoxDropIndicator: function(val) {
+  setFirefoxDropIndicator(val) {
     gBrowser.tabContainer._tabDropIndicator.collapsed = !val;
   },
 
-  removeDragmarkAttribute: function(tab) {
+  removeDragmarkAttribute(tab) {
     tab.removeAttribute("dragmark");
   },
 
-  setDragmarkAttribute: function(tab, markSide) {
+  setDragmarkAttribute(tab, markSide) {
     tab.setAttribute("dragmark", markSide);
   },
 
   /*
    *  helper functions
    */
-  _getDropEffectForTabDrag: function(event) {
+  _getDropEffectForTabDrag(event) {
     let tabBar = gBrowser.tabContainer;
     if (Tabmix.isVersion(440)) {
       return tabBar._getDropEffectForTabDrag(event);
@@ -846,7 +1027,7 @@ var TMP_tabDNDObserver = {
     return null;
   },
 
-  isCopyDropEffect: function(dt, event, type) {
+  isCopyDropEffect(dt, event, type) {
     let isCopy = dt.dropEffect == "copy";
     if (isCopy && type == this.DRAG_LINK) {
       // Dragging bookmark or livemark from the Bookmarks toolbar, or dragging
@@ -882,7 +1063,7 @@ var TMP_tabDNDObserver = {
 }; // TMP_tabDNDObserver end
 
 var TMP_undocloseTabButtonObserver = {
-  onDragOver: function(aEvent) {
+  onDragOver(aEvent) {
     var dt = aEvent.dataTransfer;
     var sourceNode = TMP_tabDNDObserver.getSourceNode(dt) || this.NEW_getSourceNode(dt);
     if (!sourceNode || sourceNode.localName != "tab") {
@@ -907,7 +1088,7 @@ var TMP_undocloseTabButtonObserver = {
     return true;
   },
 
-  onDragExit: function(aEvent) {
+  onDragExit(aEvent) {
     if (aEvent.target.hasAttribute("dragover")) {
       var statusTextFld = document.getElementById("statusbar-display");
       if (statusTextFld)
@@ -919,7 +1100,7 @@ var TMP_undocloseTabButtonObserver = {
     }
   },
 
-  onDrop: function(aEvent) {
+  onDrop(aEvent) {
     var dt = aEvent.dataTransfer;
     var sourceNode = TMP_tabDNDObserver.getSourceNode(dt) || this.NEW_getSourceNode(dt);
     if (sourceNode && sourceNode.localName == "tab")
@@ -976,7 +1157,7 @@ Tabmix.whereToOpen = function TMP_whereToOpen(pref, altKey) {
 
 Tabmix.getStyle = function TMP_getStyle(aObj, aStyle) {
   try {
-    return parseInt(window.getComputedStyle(aObj, null)[aStyle]) || 0;
+    return parseInt(window.getComputedStyle(aObj)[aStyle]) || 0;
   } catch (ex) {
     this.assert(ex);
   }
@@ -997,7 +1178,7 @@ Tabmix.hidePopup = function TMP_hidePopup(aPopupMenu) {
 
 var TMP_TabView = {
   subScriptLoaded: false,
-  init: function() {
+  init() {
     try {
       if (this.installed) {
         this._patchBrowserTabview();
@@ -1015,11 +1196,11 @@ var TMP_TabView = {
     return installed;
   },
 
-  exist: function(id) {
+  exist(id) {
     return this.installed && typeof TabView[id] == "function";
   },
 
-  checkTabs: function(tabs) {
+  checkTabs(tabs) {
     var firstTab;
     for (var i = 0; i < tabs.length; i++) {
       let tab = tabs[i];
@@ -1032,12 +1213,12 @@ var TMP_TabView = {
   },
 
   // including _removingTabs
-  currentGroup: function() {
+  currentGroup() {
     return Array.prototype.filter.call(gBrowser.tabs, tab => !tab.hidden);
   },
 
   // visibleTabs don't include  _removingTabs
-  getTabPosInCurrentGroup: function(aTab) {
+  getTabPosInCurrentGroup(aTab) {
     if (aTab) {
       let tabs = Array.prototype.filter.call(gBrowser.tabs, tab => !tab.hidden);
       return tabs.indexOf(aTab);
@@ -1045,7 +1226,7 @@ var TMP_TabView = {
     return -1;
   },
 
-  getIndexInVisibleTabsFromTab: function(aTab) {
+  getIndexInVisibleTabsFromTab(aTab) {
     if (aTab)
       return gBrowser.visibleTabs.indexOf(aTab);
     return -1;
@@ -1060,8 +1241,8 @@ Tabmix.navToolbox = {
 
   init: function TMP_navToolbox_init() {
     this.updateToolboxItems();
-    gNavToolbox.addEventListener("beforecustomization", this, false);
-    gNavToolbox.addEventListener("aftercustomization", this, false);
+    gNavToolbox.addEventListener("beforecustomization", this);
+    gNavToolbox.addEventListener("aftercustomization", this);
 
     if (!Tabmix.isVersion(290))
       return;
@@ -1086,8 +1267,8 @@ Tabmix.navToolbox = {
   },
 
   deinit: function TMP_navToolbox_deinit() {
-    gNavToolbox.removeEventListener("beforecustomization", this, false);
-    gNavToolbox.removeEventListener("aftercustomization", this, false);
+    gNavToolbox.removeEventListener("beforecustomization", this);
+    gNavToolbox.removeEventListener("aftercustomization", this);
 
     // fix bug 1034394 - tab mix plus's tabmixScrollBox is not cleaned up after
     // uninstalling tab mix plus
@@ -1111,11 +1292,11 @@ Tabmix.navToolbox = {
 
     let alltabsPopup = document.getElementById("alltabs-popup");
     if (alltabsPopup && alltabsPopup._tabmix_inited) {
-      alltabsPopup.removeEventListener("popupshown", alltabsPopup.__ensureElementIsVisible, false);
+      alltabsPopup.removeEventListener("popupshown", alltabsPopup.__ensureElementIsVisible);
     }
   },
 
-  cleanCurrentset: function() {
+  cleanCurrentset() {
     let tabsToolbar = document.getElementById("TabsToolbar");
     let cSet = tabsToolbar.getAttribute("currentset");
     if (cSet.indexOf("tabmixScrollBox") > -1) {
@@ -1131,7 +1312,7 @@ Tabmix.navToolbox = {
         this.customizeStart();
         break;
       case "customizationchange":
-        gNavToolbox.removeEventListener("customizationchange", this, false);
+        gNavToolbox.removeEventListener("customizationchange", this);
         this.toolboxChanged = true;
         break;
       case "aftercustomization":
@@ -1141,13 +1322,13 @@ Tabmix.navToolbox = {
   },
 
   customizeStart: function TMP_navToolbox_customizeStart() {
-    gNavToolbox.addEventListener("customizationchange", this, false);
+    gNavToolbox.addEventListener("customizationchange", this);
     this.toolboxChanged = false;
     this.customizeStarted = true;
   },
 
   customizeDone: function TMP_navToolbox_customizeDone(aToolboxChanged) {
-    gNavToolbox.removeEventListener("customizationchange", this, false);
+    gNavToolbox.removeEventListener("customizationchange", this);
     this.customizeStarted = false;
 
     if (aToolboxChanged)
@@ -1202,10 +1383,11 @@ Tabmix.navToolbox = {
     }
   },
 
-  handleCommand: function(event, openUILinkWhere, openUILinkParams) {
+  handleCommand(event, openUILinkWhere, openUILinkParams = {}) {
     let prevTab, prevTabPos;
-    let action = this._parseActionUrl(this.value);
-    if (action && action.type == "switchtab" && this.hasAttribute("actiontype")) {
+    let action = this._parseActionUrl(this.value) || {};
+    if (Tabmix.prefs.getBoolPref("moveSwitchToTabNext") &&
+        action.type == "switchtab" && this.hasAttribute("actiontype")) {
       prevTab = gBrowser.selectedTab;
       prevTabPos = prevTab._tPos;
     }
@@ -1215,8 +1397,9 @@ Tabmix.navToolbox = {
       let altEnter = !isMouseEvent && event &&
           event.altKey && !isTabEmpty(gBrowser.selectedTab);
       let where = "current";
+      let url = action.params ? action.params.url : this.value;
       let loadNewTab = Tabmix.whereToOpen("extensions.tabmix.opentabfor.urlbar",
-              altEnter).inNew && !(/^ *javascript:/.test(this.value));
+        altEnter).inNew && !(/^ *javascript:/.test(url));
       if (isMouseEvent || altEnter || loadNewTab) {
         // Use the standard UI link behaviors for clicks or Alt+Enter
         where = "tab";
@@ -1230,17 +1413,18 @@ Tabmix.navToolbox = {
       openUILinkWhere = where;
     }
 
+    openUILinkParams.inBackground = Tabmix.prefs.getBoolPref("loadUrlInBackground");
     Tabmix.originalFunctions.gURLBar_handleCommand.call(gURLBar, event, openUILinkWhere, openUILinkParams);
 
     // move the tab that was switched to after the previously selected tab
-    if (prevTabPos) {
+    if (typeof prevTabPos == "number") {
       let pos = prevTabPos + Number(gBrowser.selectedTab._tPos > prevTabPos) -
           Number(!prevTab || !prevTab.parentNode);
       gBrowser.moveTabTo(gBrowser.selectedTab, pos);
     }
   },
 
-  handleCommand_beforeV50: function() {
+  handleCommand_beforeV50() {
     let obj = gURLBar, fn;
     // Fix incompatibility with Omnibar (O is not defined)
     // URL Dot 0.4.x extension
@@ -1318,7 +1502,7 @@ Tabmix.navToolbox = {
     }
   },
 
-  whereToOpenSearch: function(aWhere) {
+  whereToOpenSearch(aWhere) {
     var tab = gBrowser.selectedTab;
     var isBlankTab = gBrowser.isBlankNotBusyTab(tab);
     var isLockTab = !isBlankTab && tab.hasAttribute("locked");
@@ -1343,12 +1527,12 @@ Tabmix.navToolbox = {
     })(window.esteban_torres || window);
 
     let _handleSearchCommand = searchLoadExt ? searchLoadExt.MOZhandleSearch.toString() :
-                                               searchbar.handleSearchCommand.toString();
+      searchbar.handleSearchCommand.toString();
     // we check browser.search.openintab also for search button click
     if (_handleSearchCommand.indexOf("whereToOpenLink") > -1 &&
           _handleSearchCommand.indexOf("forceNewTab") == -1) {
       [obj, fn] = searchLoadExt ? [searchLoadExt, "MOZhandleSearch"] :
-                                      [searchbar, "handleSearchCommand"];
+        [searchbar, "handleSearchCommand"];
       $LF = '\n            ';
       Tabmix.changeCode(obj, "searchbar." + fn)._replace(
         'where = whereToOpenLink(aEvent, false, true);',
@@ -1362,7 +1546,7 @@ Tabmix.navToolbox = {
 
     let organizeSE = "organizeSE" in window && "doSearch" in window.organizeSE;
     [obj, fn] = searchLoadExt ? [searchLoadExt, "MOZdoSearch"] :
-                                    [organizeSE ? window.organizeSE : searchbar, "doSearch"];
+      [organizeSE ? window.organizeSE : searchbar, "doSearch"];
     if ("__treestyletab__original_doSearch" in searchbar)
       [obj, fn] = [searchbar, "__treestyletab__original_doSearch"];
     let fnString = obj[fn].toString();
@@ -1420,15 +1604,15 @@ Tabmix.navToolbox = {
         let scrollBox = document.getAnonymousElementByAttribute(this, "class", "popup-internal-box");
         scrollBox.ensureElementIsVisible(gBrowser.mCurrentTab.mCorrespondingMenuitem);
       };
-      alltabsPopup.addEventListener("popupshown", alltabsPopup.__ensureElementIsVisible, false);
+      alltabsPopup.addEventListener("popupshown", alltabsPopup.__ensureElementIsVisible);
 
       // alltabs-popup fix visibility for multi-row
       Tabmix.setNewFunction(alltabsPopup, "_updateTabsVisibilityStatus",
-          TabmixAllTabs._updateTabsVisibilityStatus);
+        TabmixAllTabs._updateTabsVisibilityStatus);
     }
   },
 
-  tabStripAreaChanged: function() {
+  tabStripAreaChanged() {
     /**
      * we need to position three elements in TabsToolbar :
      * tabmixScrollBox, new-tab-button, and tabmix-tabs-closebutton.
@@ -1448,7 +1632,7 @@ Tabmix.navToolbox = {
       TMP_eventListener.updateMultiRow(true);
   },
 
-  setScrollButtons: function(reset, onlyPosition) {
+  setScrollButtons(reset, onlyPosition) {
     let box = document.getElementById("tabmixScrollBox");
     if (!box)
       return;
@@ -1476,7 +1660,7 @@ Tabmix.navToolbox = {
   },
 
   _closeButtonInitialized: false,
-  setCloseButtonPosition: function() {
+  setCloseButtonPosition() {
     if (this._closeButtonInitialized)
       return;
 
