@@ -1,10 +1,11 @@
 "use strict";
 
-if (!window.tablib || window.tablib.version != "tabmixplus")
-var tablib = { // eslint-disable-line
+Tabmix.backwardCompatibilityGetter(window, "tablib", "Tabmix.tablib");
+
+Tabmix.tablib = {
   version: "tabmixplus",
   _inited: false,
-  init: function TMP_tablib_init() {
+  init: function tabmix_tablib_init() {
     if (this._inited)
       return;
     this._inited = true;
@@ -15,7 +16,7 @@ var tablib = { // eslint-disable-line
   },
 
   _loadURIWithFlagsInitialized: false,
-  setLoadURIWithFlags: function tablib_setLoadURIWithFlags(aBrowser) {
+  setLoadURIWithFlags: function tabmix_tablib_setLoadURIWithFlags(aBrowser) {
     // set init value according to lockallTabs state
     // we update this value in TabmixProgressListener.listener.onStateChange
     aBrowser.tabmix_allowLoad = !TabmixTabbar.lockallTabs;
@@ -27,31 +28,38 @@ var tablib = { // eslint-disable-line
       if (this._loadURIWithFlagsInitialized)
         return;
       this._loadURIWithFlagsInitialized = true;
-      [obj, name] = [window, "window._loadURIWithFlags"];
+      [obj, name] = [window, "_loadURIWithFlags"];
     } else {
-      [obj, name] = [aBrowser, "browser.loadURIWithFlags"];
+      [obj, name] = [aBrowser, "loadURIWithFlags"];
     }
-    Tabmix.changeCode(obj, name)._replace(
-      '{',
-      '$&\n' +
-      '  let args = Array.prototype.slice.call(arguments);\n' +
-      '  if (!Tabmix.isVersion(360))\n' +
-      '    args.unshift(this);\n' +
-      '  var tabmixResult = tablib._loadURIWithFlags.apply(null, args);\n' +
-      '  if (tabmixResult)\n' +
-      '    return tabmixResult;\n'
-    )._replace(
-      /(})(\)?)$/,
-      '  return null;\n' +
-      '$1$2'
-    )._replace(
-      'this.webNavigation.LOAD_FLAGS_FROM_EXTERNAL',
-      'Ci.nsIWebNavigation.LOAD_FLAGS_FROM_EXTERNAL',
-      {check: !Tabmix.isVersion(360) && ("loadTabsProgressively" in window)}
-    ).toCode();
+
+    Tabmix.originalFunctions._loadURIWithFlags = obj[name];
+    obj[name] = function(...args) {
+      try {
+        const arg0 = [];
+        if (!Tabmix.isVersion(360)) {
+          arg0.push(this);
+        }
+
+        // if we redirected the load request to a new tab return it
+        const tabmixResult = Tabmix.tablib._loadURIWithFlags.apply(null, arg0.concat(args));
+        if (tabmixResult) {
+          return tabmixResult;
+        }
+
+        Tabmix.originalFunctions._loadURIWithFlags.apply(this, args);
+      } catch (ex) {
+        Tabmix.reportError(ex);
+      }
+      return null;
+    };
   },
 
-  _loadURIWithFlags: function(browser, uri, params) {
+  _loadURIWithFlags(browser, uri, params) {
+    if (Tabmix.tablib.allowLoad(browser, uri)) {
+      return null;
+    }
+    // redirect load request to a new tab
     let flags;
     if (!Tabmix.isVersion(380)) {
       flags = params;
@@ -64,12 +72,21 @@ var tablib = { // eslint-disable-line
       flags = params.flags;
     }
 
+    let isFlagged = flag => Boolean(flags & Ci.nsIWebNavigation[flag]);
+    params.inBackground = false;
+    params.allowThirdPartyFixup = isFlagged("LOAD_FLAGS_ALLOW_THIRD_PARTY_FIXUP");
+    params.fromExternal = isFlagged("LOAD_FLAGS_FROM_EXTERNAL");
+    params.allowMixedContent = isFlagged("LOAD_FLAGS_ALLOW_MIXED_CONTENT");
+    return gBrowser.loadOneTab(uri, params);
+  },
+
+  allowLoad(browser, uri) {
     var tab = gBrowser.getTabForBrowser(browser);
     if (!tab) {
       browser.tabmix_allowLoad = true;
-      return null;
+      return true;
     }
-    var allowLoad = tablib.isException(browser.tabmix_allowLoad !== false ||
+    var allowLoad = this.isException(browser.tabmix_allowLoad !== false ||
                                        uri.match(/^javascript:/));
 
     let allowedUrls = [
@@ -101,16 +118,11 @@ var tablib = { // eslint-disable-line
       return gBrowser.isBlankBrowser(browser);
     }());
     var isLockedTab = tab.hasAttribute("locked");
-    if (!allowLoad && !isBlankTab && isLockedTab) {
-      let isFlagged = flag => Boolean(flags & Ci.nsIWebNavigation[flag]);
-      params.inBackground = false;
-      params.allowThirdPartyFixup = isFlagged("LOAD_FLAGS_ALLOW_THIRD_PARTY_FIXUP");
-      params.fromExternal = isFlagged("LOAD_FLAGS_FROM_EXTERNAL");
-      params.allowMixedContent = isFlagged("LOAD_FLAGS_ALLOW_MIXED_CONTENT");
-      return gBrowser.loadOneTab(uri, params);
+    if (allowLoad || isBlankTab || !isLockedTab) {
+      browser.tabmix_allowLoad = uri == TabmixSvc.aboutBlank || !isLockedTab;
+      return true;
     }
-    browser.tabmix_allowLoad = uri == TabmixSvc.aboutBlank || !isLockedTab;
-    return null;
+    return false;
   },
 
   /**
@@ -119,7 +131,7 @@ var tablib = { // eslint-disable-line
    *
    * current code only check if the caller is in the exception list
    */
-  isException: function(loadInCurrent) {
+  isException(loadInCurrent) {
     if (loadInCurrent)
       return loadInCurrent;
 
@@ -132,7 +144,7 @@ var tablib = { // eslint-disable-line
       tabGroups: "paneSession.clearData",
     };
     let keys = Object.keys(exceptionList);
-    let isInstalled = keys.some(function(item) {
+    let isInstalled = keys.some(item => {
       return typeof window[item] == "object";
     });
     if (!isInstalled)
@@ -161,7 +173,7 @@ var tablib = { // eslint-disable-line
           (typeof params != "object" || params instanceof Ci.nsIURI)) {
         referrerURI = params;
         params = {
-          referrerURI: referrerURI,
+          referrerURI,
           charset: args[2],
           postData: args[3],
           ownerTab: args[4],
@@ -248,7 +260,7 @@ var tablib = { // eslint-disable-line
       let aboutBlank = 'this.addTab("about:blank", {skipAnimation: true})';
       let aboutNewtab = 'this.addTab(BROWSER_NEW_TAB_URL, {skipAnimation: true})';
       let code = gBrowser._beginRemoveTab.toString().indexOf(aboutNewtab) > -1 ?
-                 aboutNewtab : aboutBlank;
+        aboutNewtab : aboutBlank;
       Tabmix.changeCode(gBrowser, "gBrowser._beginRemoveTab")._replace(
         code, 'TMP_BrowserOpenTab(null, null, true)'
       ).toCode();
@@ -256,8 +268,8 @@ var tablib = { // eslint-disable-line
 
     Tabmix.changeCode(gBrowser, "gBrowser._endRemoveTab")._replace(
       'this._blurTab(aTab);',
-      'tablib.onRemoveTab(aTab); \
-       if (Services.prefs.getBoolPref("browser.tabs.animate")) { \
+      'Tabmix.tablib.onRemoveTab(aTab); \
+       if (TabmixSvc.tabAnimationsEnabled) { \
          TMP_eventListener.onTabClose_updateTabBar(aTab);\
        } \
        $&'
@@ -291,8 +303,8 @@ var tablib = { // eslint-disable-line
 
     Tabmix.changeCode(gBrowser, "gBrowser.getWindowTitleForBrowser")._replace(
       'if (!docTitle)',
-      'let tab = this.getTabForBrowser(aBrowser);\
-       if (tab.hasAttribute("tabmix_changed_label"))\
+      (Tabmix.isVersion(550) ? '' : 'let tab = this.getTabForBrowser(aBrowser);\n') +
+      'if (tab.hasAttribute("tabmix_changed_label"))\
          docTitle = tab.getAttribute("tabmix_changed_label");\
        else\
          docTitle = TMP_Places.getTabTitle(tab, aBrowser.currentURI.spec, docTitle);\
@@ -317,37 +329,61 @@ var tablib = { // eslint-disable-line
       [obj, fnName] = [gBrowser, "setTabTitle"];
     }
     Tabmix.changeCode(obj, "gBrowser." + fnName)._replace(
-      'var title = browser.contentTitle;',
+      Tabmix.isVersion(550) ? 'let isContentTitle = false;' : 'var title = browser.contentTitle;',
       '$&\
-       var urlTitle, title = tablib.getTabTitle(aTab, browser.currentURI.spec, title);'
+       var urlTitle, title = Tabmix.tablib.getTabTitle(aTab, browser.currentURI.spec, title);'
     )._replace(
       'title = textToSubURI.unEscapeNonAsciiURI(characterSet, title);',
       '$&\
       urlTitle = title;'
     )._replace(
-      'if (aTab.label == title',
+      Tabmix.isVersion(550) ? 'return this._setTabLabel' : 'if (aTab.label == title',
       'if (aTab.hasAttribute("mergeselected"))\
          title = "(*) " + title;\
-       var noChange = aTab.label == title && aTab.crop == crop;\
+       const noChange = aTab.label == title && (Tabmix.isVersion(530) || aTab.crop == crop);\
        if (aTab.hasAttribute("tabmix_changed_label")) {\
          aTab.removeAttribute("tabmix_changed_label");\
          if (noChange)\
-           tablib.onTabTitleChanged(aTab, browser, title == urlTitle);\
+           Tabmix.tablib.onTabTitleChanged(aTab, browser, title == urlTitle);\
        }\
        else if (noChange)\
          TMP_Places.currentTab = null;\
        $&'
     )._replace(
-      'aTab.crop = crop;',
-      '$&\
-       tablib.onTabTitleChanged(aTab, browser, title == urlTitle);'
+      'this._tabAttrModified',
+      `Tabmix.tablib.onTabTitleChanged(aTab, browser, title == urlTitle);
+            $&`, {check: !Tabmix.isVersion(550)}
+    )._replace(
+      '{ isContentTitle }',
+      '{ isContentTitle, urlTitle }',
+      {check: Tabmix.isVersion(550)}
     ).toCode();
 
-    // after bug 347930 - change Tab strip to be a toolbar
-    Tabmix.changeCode(gBrowser, "gBrowser.setStripVisibilityTo")._replace(
-      'this.tabContainer.visible = aShow;',
-      'if (!aShow || TabmixTabbar.hideMode != 2) $&'
-    ).toCode();
+    if (Tabmix.isVersion(550)) {
+      Tabmix.originalFunctions.gBrowser_setInitialTabTitle = gBrowser.setInitialTabTitle;
+      gBrowser.setInitialTabTitle = function(aTab) {
+        if (aTab._labelIsInitialTitle &&
+            aTab.hasAttribute("tabmix_changed_label")) {
+          return;
+        }
+        Tabmix.originalFunctions.gBrowser_setInitialTabTitle.apply(this, arguments);
+      };
+
+      Tabmix.changeCode(gBrowser, "gBrowser._setTabLabel")._replace(
+        'this._tabAttrModified',
+        `let urlTitle = aOptions && aOptions.urlTitle;
+              Tabmix.tablib.onTabTitleChanged(aTab, aTab.linkedBrowser, aLabel == urlTitle);
+              $&`
+      ).toCode();
+    }
+
+    if (!Tabmix.isVersion(570)) {
+      // after bug 347930 - change Tab strip to be a toolbar
+      Tabmix.changeCode(gBrowser, "gBrowser.setStripVisibilityTo")._replace(
+        'this.tabContainer.visible = aShow;',
+        'if (!aShow || TabmixTabbar.hideMode != 2) $&'
+      ).toCode();
+    }
 
     if (Tabmix.isVersion(390) && gMultiProcessBrowser) {
       /*
@@ -384,6 +420,8 @@ var tablib = { // eslint-disable-line
          $&'
       ).toCode();
 
+      let marginInlineStart = Tabmix.isVersion(570) ? "marginInlineStart" : "MozMarginStart";
+      let paddingInlineStart = Tabmix.isVersion(570) ? "paddingInlineStart" : "MozPaddingStart";
       let $LF = '\n          ';
       Tabmix.changeCode(tabBar, "gBrowser.tabContainer._positionPinnedTabs")._replace(
         'this.removeAttribute("positionpinnedtabs");',
@@ -391,7 +429,7 @@ var tablib = { // eslint-disable-line
            this.mTabstrip.resetFirstTabInRow();\
          $&'
       )._replace(
-        /this.mTabstrip._scrollButtonDown.(scrollWidth|getBoundingClientRect\(\).width)/,
+        /(tabstrip|this.mTabstrip)\._scrollButtonDown\.getBoundingClientRect\(\)\.width/,
         'TabmixTabbar.scrollButtonsMode != TabmixTabbar.SCROLL_BUTTONS_LEFT_RIGHT ? 0 : $&'
       )._replace(
         'if (doPosition)',
@@ -399,21 +437,28 @@ var tablib = { // eslint-disable-line
         '    Tabmix.prefs.getBoolPref("pinnedTabScroll")) {' + $LF +
         '  doPosition = false;' + $LF +
         '}' + $LF +
-        '  if (doPosition && TabmixTabbar.isMultiRow) {' +
-        '    this.setAttribute("positionpinnedtabs", "true");' +
-        '    let width = TabmixSvc.australis ? 0 : this.mTabstrip.scrollboxPaddingStart;' +
+        'if (doPosition && TabmixTabbar.isMultiRow) {' + $LF +
+        '  this.setAttribute("positionpinnedtabs", "true");' + $LF +
+        '  let layoutData = this._pinnedTabsLayoutCache;' + $LF +
+        '  if (!layoutData) {' + $LF +
+        '    layoutData = {pinnedTabWidth: this.childNodes[0].getBoundingClientRect().width};' + $LF +
+        '    if (Tabmix.isVersion(550)) {' + $LF +
+        '      this._pinnedTabsLayoutCache = layoutData;' + $LF +
+        '    }' + $LF +
+        '  }' + $LF +
+        '    let width = TabmixSvc.australis ? 0 : this.mTabstrip.scrollboxPaddingStart || 0;' + $LF +
         '    for (let i = 0; i < numPinned; i++) {' +
         '      let tab = this.childNodes[i];' +
-        '      tab.style.MozMarginStart = width + "px";' +
-        '      width += tab.getBoundingClientRect().width;' +
+        `      tab.style.${marginInlineStart} = width + "px";` + $LF +
+        '      width += layoutData.pinnedTabWidth;' +
         '    }' +
         '    if (width != this.mTabstrip.firstTabInRowMargin) {' +
         '      this.mTabstrip.firstTabInRowMargin = width;' +
         '      this.mTabstrip.firstVisible =  {tab: null, x: 0, y: 0};' +
         '      gTMPprefObserver.dynamicRules["tabmix-firstTabInRow"]' +
-        '        .style.setProperty("-moz-margin-start", width + "px", null);' +
+        `        .style.${marginInlineStart} =  width + "px";` + $LF +
         '    }' +
-        '    this.style.MozPaddingStart = "";' +
+        `    this.style.${paddingInlineStart} = "";` + $LF +
         '    TMP_tabDNDObserver.paddingLeft = Tabmix.getStyle(this, "paddingLeft");' +
         '    this.mTabstrip.setFirstTabInRow();' +
         '  }' +
@@ -421,7 +466,7 @@ var tablib = { // eslint-disable-line
       )._replace(
         /(})(\)?)$/,
         'if (TabmixTabbar.scrollButtonsMode != TabmixTabbar.SCROLL_BUTTONS_MULTIROW) {' +
-        '  TMP_tabDNDObserver.paddingLeft = parseInt(this.style.MozPaddingStart || 0);' +
+        `  TMP_tabDNDObserver.paddingLeft = parseInt(this.style.${paddingInlineStart} || 0);` +
         '}' +
         '$1$2'
       ).toCode();
@@ -559,10 +604,20 @@ var tablib = { // eslint-disable-line
 
     Tabmix.changeCode(window, "handleDroppedLink")._replace(
       'loadURI(uri, null, postData.value, false);',
-      'tablib.contentAreaOnDrop(event, url, postData.value);', {check: TabmixSvc.isPaleMoon}
+      'Tabmix.tablib.contentAreaOnDrop(event, url, postData.value);', {check: TabmixSvc.isPaleMoon}
     )._replace(
       'loadURI(data.url, null, data.postData, false);',
-      'tablib.contentAreaOnDrop(event, data.url, data.postData);', {check: !TabmixSvc.isPaleMoon}
+      'Tabmix.tablib.contentAreaOnDrop(event, data.url, data.postData);',
+      {check: !Tabmix.isVersion(520) && !TabmixSvc.isPaleMoon}
+    )._replace(
+      'let lastLocationChange = gBrowser.selectedBrowser.lastLocationChange;',
+      'let tabmixContentDrop = event ? event.tabmixContentDrop : links[0].tabmixContentDrop;\n  ' +
+      '$&',
+      {check: Tabmix.isVersion(520) && !TabmixSvc.isPaleMoon}
+    )._replace(
+      'replace: true',
+      'replace: (tabmixContentDrop || Tabmix.tablib.whereToOpenDrop(event, urls[0])) != "tab"',
+      {check: Tabmix.isVersion(520) && !TabmixSvc.isPaleMoon}
     ).toCode();
     // update current browser
     gBrowser.selectedBrowser.droppedLinkHandler = handleDroppedLink;
@@ -585,7 +640,7 @@ var tablib = { // eslint-disable-line
       }
 
       let pref = Tabmix.callerTrace("gotoHistoryIndex", "BrowserForward", "BrowserBack") ?
-          "openTabNext" : "openDuplicateNext";
+        "openTabNext" : "openDuplicateNext";
       let openTabNext = Tabmix.prefs.getBoolPref(pref);
       TMP_extensionsCompatibility.treeStyleTab.openNewTabNext(aTab, openTabNext, true);
 
@@ -607,10 +662,10 @@ var tablib = { // eslint-disable-line
 
     Tabmix.changeCode(window, "BrowserCloseTabOrWindow")._replace(
       'closeWindow(true);', // Mac
-      'tablib.closeLastTab();', {check: TabmixSvc.isMac, flags: "g"}
+      'Tabmix.tablib.closeLastTab();', {check: TabmixSvc.isMac, flags: "g"}
     )._replace(
       'gBrowser.removeCurrentTab({animate: true})',
-      'tablib.closeLastTab();'
+      'Tabmix.tablib.closeLastTab();'
     ).toCode();
 
     // hide open link in window in single window mode
@@ -647,17 +702,19 @@ var tablib = { // eslint-disable-line
     let TSTopenURI = Tabmix.extensions.treeStyleTab &&
         typeof fnObj.__treestyletab__openURI == "function" ? "__treestyletab__openURI" : "";
     [fnName, arg] = Tabmix.isVersion(260) ? ["_openURIInNewTab", "aIsExternal"] :
-                                            [TSTopenURI || "openURI", "isExternal"];
+      [TSTopenURI || "openURI", "isExternal"];
     var _openURI = Tabmix.changeCode(fnObj, "nsBrowserAccess.prototype." + fnName);
 
     var loadURIWithFlags = Tabmix.isVersion(380) ?
-        '      gBrowser.loadURIWithFlags(aURI.spec, {\n' +
-        '        flags: loadflags,\n' +
-        '        referrerURI: aReferrer,\n' +
-        '        referrerPolicy: aReferrerPolicy,\n' +
-        '      });' :
-        '      browser.loadURIWithFlags(aURI.spec, loadflags, referrer, null, null);'
-        .replace("referrer", (Tabmix.isVersion(360) ? "aReferrer" : "referrer"));
+      '      gBrowser.loadURIWithFlags(aURI.spec, {\n' +
+      '        triggeringPrincipal: typeof aTriggeringPrincipal == "undefined" ? 0 : aTriggeringPrincipal,\n' +
+      '        flags: loadflags,\n' +
+      '        referrerURI: aReferrer,\n' +
+      '        referrerPolicy: aReferrerPolicy,\n' +
+      '        userContextId: typeof aUserContextId == "undefined" ? 0 : aUserContextId,\n' +
+      '      });' :
+      '      browser.loadURIWithFlags(aURI.spec, loadflags, referrer, null, null);'
+          .replace("referrer", (Tabmix.isVersion(360) ? "aReferrer" : "referrer"));
 
     _openURI = _openURI._replace(
       'if (#1 && (!aURI || aURI.spec == "'.replace("#1", arg) + TabmixSvc.aboutBlank + '")) {',
@@ -665,7 +722,7 @@ var tablib = { // eslint-disable-line
        $&'
     )._replace(
       'win.BrowserOpenTab()',
-      'if (currentIsBlank) tablib.setURLBarFocus(); \
+      'if (currentIsBlank) Tabmix.tablib.setURLBarFocus(); \
       else $&'
     )._replace(
       '"browser.tabs.loadDivertedInBackground"',
@@ -688,7 +745,7 @@ var tablib = { // eslint-disable-line
 
     if (Tabmix.isVersion(260)) {
       _openURI.toCode();
-      fnName = "nsBrowserAccess.prototype." + (TSTopenURI || "openURI");
+      fnName = "nsBrowserAccess.prototype." + (TSTopenURI || (Tabmix.isVersion(570) ? "getContentWindowOrOpenURI" : "openURI"));
       _openURI = Tabmix.changeCode(fnObj, fnName);
     }
 
@@ -760,8 +817,8 @@ var tablib = { // eslint-disable-line
         '$&' +
         'if (where == "current" && Tabmix.whereToOpen(false).inNew) where = "tab";'
       )._replace(
-       'loadOneOrMoreURIs(homePage);',
-       '$& \
+        /loadOneOrMoreURIs\([^;]+;/,
+        '$& \
         gBrowser.ensureTabIsVisible(gBrowser.selectedTab);'
       ).toCode();
     }
@@ -776,17 +833,19 @@ var tablib = { // eslint-disable-line
        }'
     ).toCode();
 
-    Tabmix.changeCode(newWindowButtonObserver, "newWindowButtonObserver.onDrop")._replace(
-      '{',
-      '{if (Tabmix.singleWindowMode) return;'
-    ).toCode();
+    Tabmix.originalFunctions.newWindowButtonObserver_onDrop = newWindowButtonObserver.onDrop;
+    newWindowButtonObserver.onDrop = function onDrop(...args) {
+      if (!Tabmix.singleWindowMode) {
+        Tabmix.originalFunctions.newWindowButtonObserver_onDrop.apply(this, args);
+      }
+    };
 
     Tabmix.changeCode(window, "warnAboutClosingWindow")._replace(
       'gBrowser.warnAboutClosingTabs(gBrowser.closingTabsEnum.ALL)',
-      'tablib.closeWindow(true)', {flags: "g"}
+      'Tabmix.tablib.closeWindow(true)', {flags: "g"}
     )._replace(
-      'os.notifyObservers(null, "browser-lastwindow-close-granted", null);',
-      'if (!TabmixSvc.isMac && !tablib.closeWindow(true)) return false;\
+      /os\.notifyObservers\(null, "browser-lastwindow-close-granted"(?:, null)?\);/,
+      'if (!TabmixSvc.isMac && !Tabmix.tablib.closeWindow(true)) return false;\
        $&'
     ).toCode();
 
@@ -797,7 +856,7 @@ var tablib = { // eslint-disable-line
       'if (!closeWindow(false, warnAboutClosingWindow))',
       'var reallyClose = closeWindow(false, warnAboutClosingWindow);\
        if (reallyClose && !Tabmix._warnedBeforeClosing)\
-         reallyClose = tablib.closeWindow();\
+         reallyClose = Tabmix.tablib.closeWindow();\
        if (!reallyClose)'
     ).toCode();
 
@@ -813,7 +872,7 @@ var tablib = { // eslint-disable-line
     // make sure that undoCloseWindow will open the closed window in the most recent non-private window
     Tabmix.changeCode(window, "undoCloseWindow")._replace(
       'window = #1.undoCloseWindow(aIndex || 0);'
-        .replace("#1", Tabmix.isVersion(260) ? "SessionStore" : "ss"),
+          .replace("#1", Tabmix.isVersion(260) ? "SessionStore" : "ss"),
       '{if (Tabmix.singleWindowMode) {\
           window = Tabmix.RecentWindow.getMostRecentBrowserWindow({private: false});\
        }\
@@ -850,7 +909,7 @@ var tablib = { // eslint-disable-line
       '$& \
        let SM = TabmixSessionManager;\
        Tabmix.setItem("Browser:RestoreLastSession", "disabled", !SM.canRestoreLastSession || SM.isPrivateWindow);',
-       {check: Tabmix.prefs.getBoolPref("sessions.manager")}
+      {check: Tabmix.prefs.getBoolPref("sessions.manager")}
     ).toCode();
 
     Tabmix.changeCode(HistoryMenu.prototype, "HistoryMenu.prototype.populateUndoWindowSubmenu")._replace(
@@ -862,7 +921,7 @@ var tablib = { // eslint-disable-line
       '                                   document.getElementById(arguments[0]);'
     )._replace(
       /(})(\)?)$/,
-      '  tablib.populateUndoWindowSubmenu(undoPopup);\n' +
+      '  Tabmix.tablib.populateUndoWindowSubmenu(undoPopup);\n' +
       '$1$2'
     ).toCode();
 
@@ -899,7 +958,7 @@ var tablib = { // eslint-disable-line
     Tabmix.setNewFunction(window, "URLBarSetURI", _URLBarSetURI);
   },
 
-  populateUndoWindowSubmenu: function(undoPopup) {
+  populateUndoWindowSubmenu(undoPopup) {
     /* eslint-disable tabmix/balanced-listeners */
     if (!undoPopup.hasAttribute("context"))
       undoPopup.setAttribute("context", "tm_undocloseWindowContextMenu");
@@ -916,14 +975,14 @@ var tablib = { // eslint-disable-line
       if (undoItem && m.hasAttribute("targetURI")) {
         let otherTabsCount = undoItem.tabs.length - 1;
         let label = (otherTabsCount === 0) ?
-            menuLabelStringSingleTab : PluralForm.get(otherTabsCount, menuLabelString);
+          menuLabelStringSingleTab : PluralForm.get(otherTabsCount, menuLabelString);
         TMP_SessionStore.getTitleForClosedWindow(undoItem);
         let menuLabel = label.replace("#1", undoItem.title)
-                             .replace("#2", otherTabsCount);
+            .replace("#2", otherTabsCount);
         m.setAttribute("label", menuLabel);
         m.setAttribute("value", i);
         m.fileName = "closedwindow";
-        m.addEventListener("click", checkForMiddleClick, false);
+        m.addEventListener("click", checkForMiddleClick);
       }
     }
     let restoreAllWindows = undoPopup.lastChild;
@@ -932,7 +991,7 @@ var tablib = { // eslint-disable-line
     clearList.id = "menu_clearClosedWindowsList";
     clearList.setAttribute("label", TabmixSvc.getString("undoClosedWindows.clear.label"));
     clearList.setAttribute("value", -1);
-    clearList.addEventListener("command", function() {
+    clearList.addEventListener("command", () => {
       TabmixSessionManager.forgetClosedWindow(-1);
     });
     undoPopup.insertBefore(clearList, restoreAllWindows);
@@ -945,7 +1004,7 @@ var tablib = { // eslint-disable-line
         aTab = this.mCurrentTab;
 
       var newTab = null;
-      let copyToNewWindow = window != aTab.ownerDocument.defaultView;
+      let copyToNewWindow = window != aTab.ownerGlobal;
       let openDuplicateNext = !disallowSelect && !copyToNewWindow && Tabmix.prefs.getBoolPref("openDuplicateNext");
       TMP_extensionsCompatibility.treeStyleTab.openNewTabNext(aTab, openDuplicateNext);
 
@@ -990,6 +1049,10 @@ var tablib = { // eslint-disable-line
           var activeIndex = (tabState.index || tabState.entries.length) - 1;
           var entriesToRemove = 0;
           var newEntry = {url: aHref}; // we don't know the page title at this moment
+          let triggeringPrincipal = TabmixSvc.SERIALIZED_SYSTEMPRINCIPAL;
+          if (triggeringPrincipal) {
+            newEntry.triggeringPrincipal_base64 = triggeringPrincipal;
+          }
           tabState.entries.splice(activeIndex + 1, entriesToRemove, newEntry);
           tabState.index++;
         } catch (ex) {
@@ -1071,13 +1134,13 @@ var tablib = { // eslint-disable-line
         };
 
         Services.obs.addObserver(delayedStartupFinished,
-                                 "browser-delayed-startup-finished",
-                                 false);
+          "browser-delayed-startup-finished",
+          false);
       }
     };
 
     gBrowser.openLinkWithHistory = function() {
-      var url = tablib.getValidUrl();
+      var url = Tabmix.tablib.getValidUrl();
       if (!url)
         return;
 
@@ -1097,7 +1160,7 @@ var tablib = { // eslint-disable-line
     };
 
     gBrowser.openInverseLink = function(event) {
-      var url = tablib.getValidUrl();
+      var url = Tabmix.tablib.getValidUrl();
       if (!url)
         return;
 
@@ -1107,8 +1170,8 @@ var tablib = { // eslint-disable-line
       Tabmix.originalFunctions.openInverseLink.call(gContextMenu, event);
     };
 
-    tablib.openLinkInCurrent = function() {
-      var url = tablib.getValidUrl();
+    Tabmix.tablib.openLinkInCurrent = function() {
+      var url = Tabmix.tablib.getValidUrl();
       if (!url)
         return;
 
@@ -1117,7 +1180,7 @@ var tablib = { // eslint-disable-line
       gContextMenu.openLinkInCurrent();
     };
 
-    tablib.getValidUrl = function() {
+    Tabmix.tablib.getValidUrl = function() {
       if (!gContextMenu) {
         return null;
       }
@@ -1147,7 +1210,7 @@ var tablib = { // eslint-disable-line
         // we only get here when it is safe to use contentWindowAsCPOW
         // see TabmixContext.updateMainContextMenu
         let result = Tabmix.ContentClick.getParamsForLink(json,
-              target, linkURL, browser, gBrowser.selectedBrowser._contentWindow);
+          target, linkURL, browser, gBrowser.selectedBrowser._contentWindow);
         return result._href && isValid(result._href) ? result._href : null;
       }
       return linkURL;
@@ -1280,7 +1343,7 @@ var tablib = { // eslint-disable-line
       if (aTab._tPos > this.mCurrentTab._tPos)
         this.selectedTab = aTab;
       let tabPos = childNodes.indexOf(aTab);
-      tablib.reloadTabs(childNodes.slice(0, tabPos).reverse());
+      Tabmix.tablib.reloadTabs(childNodes.slice(0, tabPos).reverse());
     };
 
     gBrowser.reloadRightTabs = function(aTab) {
@@ -1290,7 +1353,7 @@ var tablib = { // eslint-disable-line
       if (aTab._tPos < this.mCurrentTab._tPos)
         this.selectedTab = aTab;
       let tabPos = childNodes.indexOf(aTab);
-      tablib.reloadTabs(childNodes.slice(tabPos + 1));
+      Tabmix.tablib.reloadTabs(childNodes.slice(tabPos + 1));
     };
 
     gBrowser.reloadAllTabsBut = function(aTab) {
@@ -1298,7 +1361,7 @@ var tablib = { // eslint-disable-line
         aTab = this.mCurrentTab;
       else
         this.selectedTab = aTab;
-      tablib.reloadTabs(this.visibleTabs, aTab);
+      Tabmix.tablib.reloadTabs(this.visibleTabs, aTab);
     };
 
     gBrowser.lockTab = function(aTab) {
@@ -1377,12 +1440,12 @@ var tablib = { // eslint-disable-line
       if (aTab.localName != "tab")
         aTab = this.mCurrentTab;
       var clipboard = Components.classes["@mozilla.org/widget/clipboardhelper;1"]
-                   .getService(Components.interfaces.nsIClipboardHelper);
+          .getService(Components.interfaces.nsIClipboardHelper);
 
       clipboard.copyString(this.getBrowserForTab(aTab).currentURI.spec);
     };
 
-  /** XXX need to fix this functions:
+    /** XXX need to fix this functions:
     previousTabIndex
     previousTab
     selectIndexAfterRemove
@@ -1394,7 +1457,7 @@ var tablib = { // eslint-disable-line
       var temp_id, tempIndex = -1, max_id = 0;
       var tabs = aTabs || this.visibleTabs;
       var items = Array.prototype.filter.call(this.tabContainer.getElementsByAttribute("tabmix_selectedID", "*"),
-          tab => !tab.hidden && !tab.closing);
+        tab => !tab.hidden && !tab.closing);
       for (var i = 0; i < items.length; ++i) {
         if (aTab && items[i] != aTab) {
           temp_id = parseInt(items[i].getAttribute("tabmix_selectedID") || 0);
@@ -1417,7 +1480,7 @@ var tablib = { // eslint-disable-line
       // if no tabmix_selectedID go to previous tab, from first tab go to the next tab
       if (tempIndex == -1)
         this.selectedTab = aTab == tabs[0] ? Tabmix.visibleTabs.next(aTab) :
-                                             Tabmix.visibleTabs.previous(aTab);
+          Tabmix.visibleTabs.previous(aTab);
       else
         this.selectedTab = tabs[tempIndex];
 
@@ -1442,7 +1505,7 @@ var tablib = { // eslint-disable-line
           return currentIndex == l - 1 ? currentIndex - 1 : l - 1;
         case 6: {// last opened
           let lastTabIndex, maxID = -1;
-          tabs.forEach(function(tab, index) {
+          tabs.forEach((tab, index) => {
             if (tab == oldTab)
               return;
             let linkedPanel = tab.linkedPanel.replace('panel', '');
@@ -1462,7 +1525,7 @@ var tablib = { // eslint-disable-line
           if (tempIndex > -1)
             return tempIndex;
         }
-          /* falls through */
+        /* falls through */
         case 2: // opener / right  (default )
         case 5: // right tab
           /* falls through */
@@ -1496,7 +1559,7 @@ var tablib = { // eslint-disable-line
       {value: {ALL: 0, OTHER: 1, TO_END: 2, ALL_ONEXIT: 3, TO_START: 4, GROUP: 5}, writable: false});
 
     let warnAboutClosingTabs = function(whatToClose, aTab, aDomain) {
-      // see tablib.closeWindow comment
+      // see Tabmix.tablib.closeWindow comment
       if (Tabmix.callerTrace("BG__onQuitRequest")) {
         return true;
       }
@@ -1531,10 +1594,11 @@ var tablib = { // eslint-disable-line
       var numProtected = protectedTabs.length;
       var shouldPrompt = 0;
       var prefs = ["extensions.tabmix.tabs.warnOnClose",
-                  "extensions.tabmix.protectedtabs.warnOnClose",
-                  "browser.tabs.warnOnClose"];
+        "extensions.tabmix.protectedtabs.warnOnClose",
+        "browser.tabs.warnOnClose"];
       if (onExit) {
-        if (numTabs > 1 && Services.prefs.getBoolPref(prefs[2]))
+        let openTabs = numTabs - this._removingTabs.length;
+        if (openTabs > 1 && Services.prefs.getBoolPref(prefs[2]))
           shouldPrompt = 3;
         else if (numProtected > 0 && Services.prefs.getBoolPref(prefs[1]))
           shouldPrompt = 2;
@@ -1631,33 +1695,33 @@ var tablib = { // eslint-disable-line
       if (shouldPrompt == 1 || numProtected === 0) {
         if (Tabmix.isVersion(290))
           message = PluralForm.get(tabsToClose, bundle.getString("tabs.closeWarningMultiple"))
-                      .replace("#1", tabsToClose);
+              .replace("#1", tabsToClose);
         else
           message = bundle.getFormattedString("tabs.closeWarningMultipleTabs", [tabsToClose]);
         chkBoxLabel = shouldPrompt == 1 ? bundle.getString("tabs.closeWarningPromptMe") :
-                                          TabmixSvc.getString("window.closeWarning.1");
+          TabmixSvc.getString("window.closeWarning.2");
       } else {
         let messageKey = "protectedtabs.closeWarning.";
         messageKey += (numProtected < tabsToClose) ? "3" : (numProtected == 1) ? "1" : "2";
         message = TabmixSvc.getFormattedString(messageKey, [tabsToClose, numProtected]);
-        var chkBoxKey = shouldPrompt == 3 ? "window.closeWarning.1" : "protectedtabs.closeWarning.4";
+        var chkBoxKey = shouldPrompt == 3 ? "window.closeWarning.2" : "protectedtabs.closeWarning.5";
         chkBoxLabel = TabmixSvc.getString(chkBoxKey);
       }
 
       var buttonLabel = shouldPrompt == 1 ? bundle.getString("tabs.closeButtonMultiple") :
-                                            TabmixSvc.getString("closeWindow.label");
+        TabmixSvc.getString("closeWindow.label");
 
       window.focus();
       var promptService = Services.prompt;
       var buttonPressed = promptService.confirmEx(window,
-                                                  bundle.getString("tabs.closeWarningTitle"),
-                                                  message,
-                                                  (promptService.BUTTON_TITLE_IS_STRING * promptService.BUTTON_POS_0) +
-                                                  (promptService.BUTTON_TITLE_CANCEL * promptService.BUTTON_POS_1),
-                                                  buttonLabel,
-                                                  null, null,
-                                                  chkBoxLabel,
-                                                  warnOnClose);
+        bundle.getString("tabs.closeWarningTitle"),
+        message,
+        (promptService.BUTTON_TITLE_IS_STRING * promptService.BUTTON_POS_0) +
+        (promptService.BUTTON_TITLE_CANCEL * promptService.BUTTON_POS_1),
+        buttonLabel,
+        null, null,
+        chkBoxLabel,
+        warnOnClose);
       var reallyClose = (buttonPressed === 0);
       // don't set the pref unless they press OK and it's false
       if (reallyClose && !warnOnClose.value) {
@@ -1670,7 +1734,7 @@ var tablib = { // eslint-disable-line
 
     gBrowser.TMP_selectNewForegroundTab = function(aTab, aLoadInBackground, aUrl, addOwner) {
       var bgLoad = typeof aLoadInBackground == "boolean" ? aLoadInBackground :
-      Services.prefs.getBoolPref("browser.tabs.loadInBackground");
+        Services.prefs.getBoolPref("browser.tabs.loadInBackground");
       if (!bgLoad) {
         // set new tab owner
         addOwner = typeof addOwner == "boolean" ? addOwner : true;
@@ -1678,7 +1742,7 @@ var tablib = { // eslint-disable-line
           aTab.owner = this.selectedTab;
         this.selectedTab = aTab;
         if (aUrl && Tabmix.isNewTabUrls(aUrl))
-          tablib.setURLBarFocus();
+          Tabmix.tablib.setURLBarFocus();
       }
     };
 
@@ -1687,7 +1751,7 @@ var tablib = { // eslint-disable-line
       // Do not allow transferring a private tab to a non-private window
       // and vice versa.
       if (PrivateBrowsingUtils.isWindowPrivate(window) !=
-          PrivateBrowsingUtils.isWindowPrivate(aOtherTab.ownerDocument.defaultView)) {
+          PrivateBrowsingUtils.isWindowPrivate(aOtherTab.ownerGlobal)) {
         return;
       }
 
@@ -1709,8 +1773,10 @@ var tablib = { // eslint-disable-line
     // Bug 752376 - Avoid calling scrollbox.ensureElementIsVisible()
     // if the tab strip doesn't overflow to prevent layout flushes
     gBrowser.ensureTabIsVisible = function tabmix_ensureTabIsVisible(aTab, aSmoothScroll) {
-      if (Tabmix.tabsUtils.overflow)
-        this.tabContainer.mTabstrip.ensureElementIsVisible(aTab, aSmoothScroll);
+      if (Tabmix.tabsUtils.overflow) {
+        const instantScroll = Tabmix.isVersion(570) ? !aSmoothScroll : aSmoothScroll;
+        this.tabContainer.mTabstrip.ensureElementIsVisible(aTab, instantScroll);
+      }
     };
 
     /** DEPRECATED **/
@@ -1738,9 +1804,11 @@ var tablib = { // eslint-disable-line
   get labels() {
     delete this.labels;
     this.labels = [
-      gBrowser.mStringBundle.getString("tabs.connecting"),
       gBrowser.mStringBundle.getString("tabs.emptyTabTitle"),
     ];
+    if (!Tabmix.isVersion(550)) {
+      this.labels.push(gBrowser.mStringBundle.getString("tabs.connecting"));
+    }
     return this.labels;
   },
 
@@ -1775,7 +1843,7 @@ var tablib = { // eslint-disable-line
     // Not in use since Firefox 27, see comment in TabmixTabClickOptions
     if (Tabmix.prefs.getBoolPref("tabbar.click_dragwindow") &&
         TabmixTabClickOptions._blockDblClick) {
-      setTimeout(function() {
+      setTimeout(() => {
         TabmixTabClickOptions._blockDblClick = false;
         if (!Tabmix.isVersion(270))
           gBrowser.tabContainer._blockDblClick = false;
@@ -1789,7 +1857,9 @@ var tablib = { // eslint-disable-line
     }
 
     try {
-      TabmixSessionManager.tabScrolled(tab);
+      if (!Tabmix.isVersion(320)) {
+        TabmixSessionManager.tabScrolled(tab);
+      }
     } catch (ex) {
       Tabmix.assert(ex, "ERROR in TabmixSessionManager.tabScrolled");
     }
@@ -1899,14 +1969,17 @@ var tablib = { // eslint-disable-line
     return canClose;
   },
 
-  whereToOpenDrop: function(aEvent, aUri) {
+  whereToOpenDrop(aEvent, aUri) {
+    if (!aEvent) {
+      return "current";
+    }
     let browser = gBrowser.selectedBrowser;
     let where = "current";
     if (aUri != browser.currentURI.spec) {
       let tab = gBrowser.mCurrentTab;
       let isCopy = "dataTransfer" in aEvent ?
-          TMP_tabDNDObserver.isCopyDropEffect(aEvent.dataTransfer, aEvent, 0) :
-          (aEvent.ctrlKey || aEvent.metaKey);
+        TMP_tabDNDObserver.isCopyDropEffect(aEvent.dataTransfer, aEvent, 0) :
+        (aEvent.ctrlKey || aEvent.metaKey);
       if (!isCopy && tab.getAttribute("locked") &&
           !gBrowser.isBlankNotBusyTab(tab) &&
           !Tabmix.ContentClick.isUrlForDownload(aUri)) {
@@ -1932,7 +2005,7 @@ var tablib = { // eslint-disable-line
       gURLBar.focus();
   },
 
-  reloadTabs: function(tabs, skipTab) {
+  reloadTabs(tabs, skipTab) {
     let l = tabs.length;
     for (let i = 0; i < l; i++) {
       let tab = tabs[i];
@@ -1944,7 +2017,7 @@ var tablib = { // eslint-disable-line
     }
   }
 
-}; // end tablib
+}; // end Tabmix.tablib
 
 Tabmix.isNewTabUrls = function Tabmix_isNewTabUrls(aUrl) {
   return this.newTabUrls.indexOf(aUrl) > -1;
