@@ -6,21 +6,24 @@ var TMP_Places = {
   prefBookmark: "extensions.tabmix.opentabfor.bookmarks",
 
   addEvent: function TMP_PC_addEvent() {
-    window.addEventListener("load", this, false);
-    window.addEventListener("unload", this, false);
+    window.addEventListener("load", this);
+    window.addEventListener("unload", this);
   },
 
   handleEvent: function TMP_PC_handleEvent(aEvent) {
     switch (aEvent.type) {
       case "load":
-        window.removeEventListener("load", this, false);
+        window.removeEventListener("load", this);
         this.init();
         Tabmix.onContentLoaded.change_utilityOverlay();
         break;
       case "unload":
-        window.removeEventListener("unload", this, false);
+        window.removeEventListener("unload", this);
         this.deinit();
         break;
+      case "SSTabRestored":
+        this.updateRestoringTabsList(aEvent.target);
+        break;
     }
   },
 
@@ -33,6 +36,7 @@ var TMP_Places = {
     // use tab label for bookmark name when user renamed the tab
     // PlacesCommandHook exist on browser window
     if ("PlacesCommandHook" in window) {
+      gBrowser.tabContainer.addEventListener("SSTabRestored", this);
       if (Tabmix.isVersion(400)) {
         if (!Tabmix.originalFunctions.placesBookmarkPage) {
           Tabmix.originalFunctions.placesBookmarkPage = PlacesCommandHook.bookmarkPage;
@@ -87,6 +91,10 @@ var TMP_Places = {
   },
 
   deinit: function TMP_PC_deinit() {
+    if ("gBrowser" in window) {
+      gBrowser.tabContainer.removeEventListener("SSTabRestored", this);
+      this.restoringTabs = [];
+    }
     this.stopObserver();
   },
 
@@ -108,10 +116,10 @@ var TMP_Places = {
     }
   },
 
-  openMenuItem: function(aUri, aEvent, aParams, aPref) {
+  openMenuItem(aUri, aEvent, aParams, aPref) {
     let pref = "extensions.tabmix.opentabfor." + aPref;
     let where = this.isBookmarklet(aUri) ? "current" :
-                this.fixWhereToOpen(aEvent, whereToOpenLink(aEvent, false, true), pref);
+      this.fixWhereToOpen(aEvent, whereToOpenLink(aEvent, false, true), pref);
     if (where == "current")
       Tabmix.getTopWin().gBrowser.selectedBrowser.tabmix_allowLoad = true;
     aParams.inBackground = Services.prefs.getBoolPref("browser.tabs.loadBookmarksInBackground");
@@ -125,7 +133,7 @@ var TMP_Places = {
     BMB_bookmarksPopup: "bookmarks",
   },
 
-  openUILink: function(url, event, where, params) {
+  openUILink(url, event, where, params) {
     // divert all the calls from places UI to use our preferences
     //   HistoryMenu.prototype._onCommand
     //   BookmarkingUI._updateRecentBookmarks/onItemCommand
@@ -167,12 +175,12 @@ var TMP_Places = {
     return where;
   },
 
-  isBookmarklet: function(url) {
+  isBookmarklet(url) {
     var jsURL = /^ *javascript:/;
     return jsURL.test(url);
   },
 
-  fixWhereToOpen: function(aEvent, aWhere, aPref) {
+  fixWhereToOpen(aEvent, aWhere, aPref) {
     var w = Tabmix.getTopWin();
     if (!w)
       return aWhere;
@@ -198,7 +206,7 @@ var TMP_Places = {
     return aWhere;
   },
 
-  getPrefByDocumentURI: function(aWindow) {
+  getPrefByDocumentURI(aWindow) {
     switch (aWindow.document.documentURI) {
       case "chrome://browser/content/places/places.xul": {
         let history = PlacesUIUtils.getString("OrganizerQueryHistory");
@@ -209,7 +217,7 @@ var TMP_Places = {
         if (!historySelected)
           return this.prefBookmark;
       }
-        /* falls through */
+      /* falls through */
       case "chrome://browser/content/history/history-panel.xul":
         return this.prefHistory;
       case "chrome://browser/content/browser.xul":
@@ -226,23 +234,21 @@ var TMP_Places = {
   //        locked and protected tabs open bookmark after those tabs
   // fixed: focus the first tab if "extensions.tabmix.openTabNext" is true
   // fixed: remove "selected" and "tabmix_selectedID" from reuse tab
-  //
-  // TODO - try to use sessionStore to add many tabs
   openGroup: function TMP_PC_openGroup(bmGroup, bmIds, aWhere) {
-    var tabs = gBrowser.visibleTabs;
+    var openTabs = gBrowser.visibleTabs;
 
     var doReplace = (/^tab/).test(aWhere) ? false :
-        Tabmix.prefs.getBoolPref("loadBookmarksAndReplace");
+      Tabmix.prefs.getBoolPref("loadBookmarksAndReplace");
     var loadInBackground = bmGroup.length > 1 ?
-        Tabmix.prefs.getBoolPref("loadBookmarksGroupInBackground") :
-        Services.prefs.getBoolPref("browser.tabs.loadBookmarksInBackground");
+      Tabmix.prefs.getBoolPref("loadBookmarksGroupInBackground") :
+      Services.prefs.getBoolPref("browser.tabs.loadBookmarksInBackground");
     var openTabNext = Tabmix.getOpenTabNextPref();
 
     // catch tab for reuse
     var aTab, reuseTabs = [], removeTabs = [], i;
     var tabIsBlank, canReplace;
-    for (i = 0; i < tabs.length; i++) {
-      aTab = tabs[i];
+    for (i = 0; i < openTabs.length; i++) {
+      aTab = openTabs[i];
       tabIsBlank = gBrowser.isBlankNotBusyTab(aTab);
       // don't reuse collapsed tab if width fitTitle is set
       canReplace = (doReplace && !aTab.hasAttribute("locked") &&
@@ -257,33 +263,78 @@ var TMP_Places = {
       }
     }
 
+    // load tabs progressively:
+    // we use two preferences to control this feature:
+    // load_tabs_progressively - type int:
+    // when the number of tabs to load is more than the number in the preference
+    // we use SessionStore to restore each tab otherwise we use browser.loadURI.
+    //
+    // restore_on_demand - type int:
+    // when the number of tabs to load exceed the number in the preference we
+    // instruct SessionStore to use restore on demand for the current set of tabs.
+    const tabCount = this.restoringTabs.length + bmGroup.length;
+    const [loadProgressively, restoreOnDemand] = this.getPreferences(tabCount);
+
     var tabToSelect = null;
-    var prevTab = (!doReplace && openTabNext && gBrowser.mCurrentTab._tPos < tabs.length - 1) ?
-                   gBrowser.mCurrentTab : Tabmix.visibleTabs.last;
+    var prevTab = (!doReplace && openTabNext && gBrowser.mCurrentTab._tPos < openTabs.length - 1) ?
+      gBrowser.mCurrentTab : Tabmix.visibleTabs.last;
     var tabPos, index;
     var multiple = bmGroup.length > 1;
+    let tabs = [], tabsData = [];
+    let savePrincipal = TabmixSvc.SERIALIZED_SYSTEMPRINCIPAL;
     for (i = 0; i < bmGroup.length; i++) {
       let url = bmGroup[i];
-      try { // bug 300911
-        if (i < reuseTabs.length) {
-          aTab = reuseTabs[i];
-          let browser = gBrowser.getBrowserForTab(aTab);
-          browser.userTypedValue = url;
-          browser.loadURI(url);
-          // setTabTitle will call TabmixTabbar.updateScrollStatus for us
-          aTab.collapsed = false;
-          // reset visited & tabmix_selectedID attribute
-          if (!aTab.selected) {
-            aTab.removeAttribute("visited");
-            aTab.removeAttribute("tabmix_selectedID");
-          } else
-            aTab.setAttribute("reloadcurrent", true);
-        } else {
-          aTab = gBrowser.addTab(url, {skipAnimation: multiple, dontMove: true});
+      if (i < reuseTabs.length) {
+        aTab = reuseTabs[i];
+        if (!loadProgressively) {
+          const browser = aTab.linkedBrowser;
+          try {
+            browser.userTypedValue = url;
+            if (Tabmix.isVersion(550)) {
+              browser.loadURIWithFlags(url, {
+                flags: Ci.nsIWebNavigation.LOAD_FLAGS_NONE,
+                triggeringPrincipal: Services.scriptSecurityManager.getSystemPrincipal(),
+              });
+            } else {
+              browser.loadURI(url);
+            }
+          } catch (ex) { }
         }
-
-        this.setTabTitle(aTab, url, bmIds[i]);
-      } catch (er) { }
+        this.resetRestoreState(aTab);
+        aTab.collapsed = false;
+        // reset visited & tabmix_selectedID attribute
+        if (!aTab.selected) {
+          aTab.removeAttribute("visited");
+          aTab.removeAttribute("tabmix_selectedID");
+        } else
+          aTab.setAttribute("reloadcurrent", true);
+      } else {
+        let params = {
+          skipAnimation: multiple,
+          noInitialLabel: true,
+          dontMove: true,
+          forceNotRemote: loadProgressively,
+        };
+        if (Tabmix.isVersion(550)) {
+          params.triggeringPrincipal = Services.scriptSecurityManager.getSystemPrincipal();
+        }
+        aTab = gBrowser.addTab(loadProgressively ? "about:blank" : url, params);
+      }
+      this.setTabTitle(aTab, url, bmIds[i]);
+      if (Tabmix.isVersion(550)) {
+        aTab._labelIsInitialTitle = true;
+      }
+      if (loadProgressively) {
+        tabs.push(aTab);
+        let entry = {url, title: aTab.label};
+        if (savePrincipal) {
+          entry.triggeringPrincipal_base64 = TabmixSvc.SERIALIZED_SYSTEMPRINCIPAL;
+        }
+        tabsData.push({entries: [entry], index: 0});
+        if (!url.startsWith("file:") && url != "about:blank") {
+          aTab.setAttribute("_tabmix_load_bypass_cache", true);
+        }
+      }
 
       if (!tabToSelect)
         tabToSelect = aTab;
@@ -319,6 +370,71 @@ var TMP_Places = {
     while (removeTabs.length > 0) {
       gBrowser.removeTab(removeTabs.pop());
     }
+
+    if (loadProgressively) {
+      this.restoreTabs(tabs, tabsData, this.bookmarksOnDemand || restoreOnDemand);
+    }
+  },
+
+  getPreferences(tabCount) {
+    // negative value indicate that the feature is disabled
+    const progressively = Tabmix.prefs.getIntPref("load_tabs_progressively");
+    if (progressively < 0) {
+      return [false, false];
+    }
+    let onDemand = Tabmix.prefs.getIntPref("restore_on_demand");
+    if (onDemand < 0) {
+      return [tabCount > progressively, false];
+    }
+    if (onDemand < progressively) {
+      Tabmix.prefs.setIntPref("restore_on_demand", progressively);
+      onDemand = progressively;
+    }
+    return [tabCount > progressively, tabCount > onDemand];
+  },
+
+  restoreTabs(tabs, tabsData, restoreOnDemand) {
+    this.restoringTabs.push(...tabs);
+    this.bookmarksOnDemand = restoreOnDemand;
+    let fnName = Tabmix.isVersion(280) ? "restoreTabs" :
+      "restoreHistoryPrecursor";
+    TabmixSvc.SessionStore[fnName](window, tabs, tabsData, 0);
+    // set icon on pending tabs
+    const pendingData = tabs.map(tab => ({tab, url: tabsData.shift().entries[0].url}))
+        .filter(({tab, url}) => tab.hasAttribute("pending") && url != "about:blank");
+    for (let data of pendingData) {
+      const {tab, url} = data;
+      const entryURI = BrowserUtils.makeURI(url, null, null);
+      PlacesUtils.favicons.getFaviconURLForPage(entryURI, uri => {
+        // skip tab that already restored
+        if (tab.hasAttribute("pending") && uri) {
+          gBrowser.setIcon(tab, uri);
+        }
+      });
+    }
+  },
+
+  bookmarksOnDemand: false,
+  restoringTabs: [],
+
+  resetRestoreState(tab) {
+    if (tab.linkedBrowser.__SS_restoreState) {
+      TabmixSvc.SessionStore._resetTabRestoringState(tab);
+    }
+    this.updateRestoringTabsList(tab);
+  },
+
+  updateRestoringTabsList(tab) {
+    if (!this.restoringTabs.length && !this.bookmarksOnDemand) {
+      return;
+    }
+    let index = this.restoringTabs.indexOf(tab);
+    if (index > -1) {
+      this.restoringTabs.splice(index, 1);
+    }
+    if (!this.restoringTabs.length) {
+      this.bookmarksOnDemand = false;
+    }
   },
 
   setTabTitle: function TMP_PC_setTabTitle(aTab, aUrl, aID) {
@@ -331,9 +447,13 @@ var TMP_Places = {
     let title = this.getTabTitle(aTab, aUrl, aTab.label);
     if (title != aTab.label) {
       aTab.label = title;
-      aTab.crop = title != aUrl || aUrl == TabmixSvc.aboutBlank ? "end" : "center";
       aTab.setAttribute("tabmix_changed_label", title);
-      gBrowser._tabAttrModified(aTab, ["label", "crop"]);
+      if (Tabmix.isVersion(530)) {
+        gBrowser._tabAttrModified(aTab, ["label"]);
+      } else {
+        aTab.crop = title != aUrl || aUrl == TabmixSvc.aboutBlank ? "end" : "center";
+        gBrowser._tabAttrModified(aTab, ["label", "crop"]);
+      }
       if (aTab.selected)
         gBrowser.updateTitlebar();
       if (!aTab.hasAttribute("faviconized"))
@@ -359,11 +479,11 @@ var TMP_Places = {
     return (this._titlefrombookmark = Tabmix.prefs.getBoolPref("titlefrombookmark"));
   },
 
-  getTitleFromBookmark: function(aUrl, aTitle, aItemId, aTab) {
+  getTitleFromBookmark(aUrl, aTitle, aItemId, aTab) {
     return this.PlacesUtils.getTitleFromBookmark(aUrl, aTitle, aItemId, aTab);
   },
 
-  isUserRenameTab: function(aTab, aUrl) {
+  isUserRenameTab(aTab, aUrl) {
     if (aTab.hasAttribute("label-uri")) {
       let label = aTab.getAttribute("label-uri");
       if (label == aUrl || label == "*")
@@ -372,7 +492,7 @@ var TMP_Places = {
     return false;
   },
 
-  afterTabTitleChanged: function(bookmarkChanged = true) {
+  afterTabTitleChanged(bookmarkChanged = true) {
     if (bookmarkChanged && !this.inUpdateBatch) {
       this.AboutNewTab.updateAllBrowsers(window);
     }
@@ -390,8 +510,13 @@ var TMP_Places = {
       let tabstrip = gBrowser.tabContainer.mTabstrip;
       if (!TabmixTabbar.isMultiRow) {
         let scrollPosition = tabstrip.scrollPosition;
-        if (scrollPosition < 100)
-          tabstrip.scrollPosition = 0;
+        if (scrollPosition < 100) {
+          if (tabstrip.orient == "vertical") {
+            tabstrip._scrollbox.scrollTop = 0;
+          } else {
+            tabstrip._scrollbox.scrollLeft = 0;
+          }
+        }
       }
       gBrowser.ensureTabIsVisible(this.currentTab, false);
       this.currentTab = null;
@@ -402,7 +527,11 @@ var TMP_Places = {
     // Start observing bookmarks if needed.
     if (!this._hasBookmarksObserver) {
       try {
-        PlacesUtils.addLazyBookmarkObserver(this);
+        if (Tabmix.isVersion(560)) {
+          PlacesUtils.bookmarks.addObserver(this);
+        } else {
+          PlacesUtils.addLazyBookmarkObserver(this);
+        }
         this._hasBookmarksObserver = true;
       } catch (ex) {
         Tabmix.reportError(ex, "Failed to add bookmarks observer:");
@@ -412,12 +541,16 @@ var TMP_Places = {
 
   stopObserver: function TMP_PC_stopObserver() {
     if (this._hasBookmarksObserver) {
-      PlacesUtils.removeLazyBookmarkObserver(this);
+      if (Tabmix.isVersion(560)) {
+        PlacesUtils.bookmarks.removeObserver(this);
+      } else {
+        PlacesUtils.removeLazyBookmarkObserver(this);
+      }
       this._hasBookmarksObserver = false;
     }
   },
 
-  onDelayedStartup: function() {
+  onDelayedStartup() {
     if (!this._titlefrombookmark || !gBrowser.tabs)
       return;
 
@@ -425,7 +558,7 @@ var TMP_Places = {
   },
 
   // extensions.tabmix.titlefrombookmark changed
-  onPreferenceChanged: function(aPrefValue) {
+  onPreferenceChanged(aPrefValue) {
     this._titlefrombookmark = aPrefValue;
 
     if (aPrefValue) {
@@ -554,18 +687,18 @@ var TMP_Places = {
     this.currentTab = null;
   },
 
-  onItemVisited: function() {},
-  onItemMoved: function() {}
+  onItemVisited() {},
+  onItemMoved() {}
 };
 
 TMP_Places.contextMenu = {
-  toggleEventListener: function(enable) {
+  toggleEventListener(enable) {
     var eventListener = enable ? "addEventListener" : "removeEventListener";
     window[eventListener]("unload", this, false);
     document.getElementById("placesContext")[eventListener]("popupshowing", this, false);
   },
 
-  handleEvent: function(aEvent) {
+  handleEvent(aEvent) {
     switch (aEvent.type) {
       case "popupshowing":
         this.buildContextMenu();
@@ -588,7 +721,7 @@ TMP_Places.contextMenu = {
   // update context menu for bookmarks manager and sidebar
   // for bookmarks/places, history, sage and more.....
   update: function TMP_contextMenu_update(open, openInWindow, openInPrivateWindow,
-                                           openInTab, pref) {
+                                          openInTab, pref) {
     // if all 4 is hidden... probably "Open all in Tabs" is visible
     if (open.hidden && openInWindow.hidden && openInPrivateWindow.hidden &&
         openInTab.hidden) {
@@ -622,12 +755,12 @@ TMP_Places.contextMenu = {
 };
 
 Tabmix.onContentLoaded = {
-  changeCode: function() {
+  changeCode() {
     this.change_miscellaneous();
     this.change_utilityOverlay();
   },
 
-  change_miscellaneous: function() {
+  change_miscellaneous() {
     Tabmix.changeCode(nsContextMenu.prototype, "nsContextMenu.prototype.openLinkInTab")._replace(
       /allowMixedContent:|charset:/,
       'inBackground: !Services.prefs.getBoolPref("browser.tabs.loadInBackground"),\n' +
@@ -658,11 +791,18 @@ Tabmix.onContentLoaded = {
       )._replace(
         /(})(\)?)$/,
         // when we get in and out of tabsintitlebar mode call updateScrollStatus
-        '  if (TabmixTabbar._enablePositionCheck &&\n  ' +
-        '      TabmixTabbar.getTabsPosition() != TabmixTabbar._tabsPosition) {\n  ' +
-        '    TabmixTabbar.updateScrollStatus();\n  ' +
-        '  }\n' +
+        // force another update when rows number changed by Tabmix to update
+        // titlebar and titlebarContent height
+        '  TabmixTabbar.updateTabsInTitlebarAppearance();\n  ' +
         '$1$2'
+      )._replace(
+        'titlebarContentHeight = Math.max(titlebarContentHeight, fullTabsHeight)',
+        'titlebarContentHeight = Math.max(titlebarContentHeight, TabmixTabbar.singleRowHeight + verticalMargins(tabsStyles))',
+        {check: !Tabmix.isVersion(570) && Tabmix.isVersion(550)}
+      )._replace(
+        'titlebarContentHeight = fullTabsHeight + 1',
+        'titlebarContentHeight = TabmixTabbar.singleRowHeight + verticalMargins(tabsStyles) + 1',
+        {check: Tabmix.isVersion(570)}
       ).toCode();
     }
 
@@ -684,7 +824,7 @@ Tabmix.onContentLoaded = {
     ).toCode();
   },
 
-  change_utilityOverlay: function() {
+  change_utilityOverlay() {
     // fix small bug when the event is not mouse event
     // inverse focus of middle/ctrl/meta clicked bookmarks/history
     // don't inverse focus when called from onPopupClick and One-Click Search
@@ -704,7 +844,8 @@ Tabmix.onContentLoaded = {
       'return shift ? "tabshifted" : "tab";',
       '{' + $LF +
       'let callerTrace = Tabmix.callerTrace();' + $LF +
-      'let list = ["openUILink", "handleLinkClick", "TMP_tabshifted", "TMP_contentLinkClick", "BG_observe"];' + $LF +
+      'let list = ["openUILink", "handleLinkClick", "BG_observe", "contentAreaClick",' + $LF +
+      '            "TMP_tabshifted", "TMP_whereToOpenLink", "TMP_contentLinkClick"];' + $LF +
       'let pref = callerTrace.contain(list) ?' + $LF +
       '    "extensions.tabmix.inversefocusLinks" : "extensions.tabmix.inversefocusOther";' + $LF +
       'let notOneClickSearch = !getBoolPref("browser.search.showOneOffButtons", false) ||' + $LF +
@@ -736,26 +877,40 @@ Tabmix.onContentLoaded = {
       '$&\n' +
       '  var bookMarkId            = params.bookMarkId;'
     )._replace(
-      'where == "current" && w.gBrowser.selectedTab.pinned',
-      '$& && !params.suppressTabsOnFileDownload'
+      'where == "current" && #1.pinned'
+          .replace("#1", Tabmix.isVersion(520) ? "tab" : "w.gBrowser.selectedTab"),
+      '$& && !params.suppressTabsOnFileDownload',
+      {check: !Tabmix.isVersion(530)}
     )._replace(
-      'var w = getTopWin();',
-      '$&\n' +
-      '  if (w && where == "window" &&\n' +
+      '(targetBrowser).pinned',
+      '$& && !params.suppressTabsOnFileDownload',
+      {check: Tabmix.isVersion(530)}
+    )._replace(
+      'if ((where == "tab" ||',
+      'if (w && where == "window" &&\n' +
       '      !Tabmix.isNewWindowAllow(aIsPrivate)) {\n' +
       '    where = "tab";\n' +
-      '  }'
+      '  }\n' +
+      '  $&'
     )._replace(
       /Services.ww.openWindow[^;]*;/,
       'let newWin = $&\n' +
       '    if (newWin && bookMarkId) {\n' +
       '        newWin.bookMarkIds = bookMarkId;\n' +
-      '    }'
+      '    }',
+      {check: !Tabmix.isVersion(540)}
+    )._replace(
+      /win = Services.ww.openWindow[^;]*;/,
+      '$&\n' +
+      '    if (win && bookMarkId) {\n' +
+      '        win.bookMarkIds = bookMarkId;\n' +
+      '    }',
+      {check: Tabmix.isVersion(540)}
     )._replace(
       /(})(\)?)$/,
-      '  var tab = where == "current" ?\n' +
+      '  const targetTab = where == "current" ?\n' +
       '      w.gBrowser.selectedTab : w.gBrowser.getTabForLastPanel();\n' +
-      '  w.TMP_Places.setTabTitle(tab, url, bookMarkId);\n' +
+      '  w.TMP_Places.setTabTitle(targetTab, url, bookMarkId);\n' +
       '  if (where == "current") {\n' +
       '    w.gBrowser.ensureTabIsVisible(w.gBrowser.selectedTab);\n' +
       '  }\n' +
@@ -765,10 +920,10 @@ Tabmix.onContentLoaded = {
 
   // update compatibility with X-notifier(aka WebMail Notifier) 2.9.13+
   // object name wmn replace with xnotifier for version 3.0+
-  getXnotifierFunction: function(aName) {
+  getXnotifierFunction(aName) {
     let com = window.com;
     if (typeof com == "object" && typeof com.tobwithu == "object") {
-      let fn = ["wmn", "xnotifier"].filter(function(f) {
+      let fn = ["wmn", "xnotifier"].filter(f => {
         return typeof com.tobwithu[f] == "object" &&
           typeof com.tobwithu[f][aName] == "function";
       });
@@ -783,6 +938,6 @@ Tabmix.onContentLoaded = {
 
 /** DEPRECATED **/
 TMP_Places.getTabFixedTitle = function(aBrowser, aUri) {
-  let win = aBrowser.ownerDocument.defaultView;
+  let win = aBrowser.ownerGlobal;
   return win.TMP_Places.getTabTitle(win.gBrowser.getTabForBrowser(aBrowser), aUri.spec);
 };
