@@ -2,7 +2,7 @@
 "use strict";
 
 var gAppearancePane = {
-  init: function() {
+  init() {
     var browserWindow = Tabmix.getTopWin();
     // disable options for position the tabbar and scroll mode if TreeStyleTab extension installed
     if (browserWindow.Tabmix.extensions.verticalTabBar) {
@@ -38,7 +38,7 @@ var gAppearancePane = {
     }
 
     // rtl update position
-    var direction = window.getComputedStyle($("appearance"), null).direction;
+    var direction = window.getComputedStyle($("appearance")).direction;
     if (direction == "rtl") {
       let right = $("newTabButton.position.right");
       // let left = $("newTabButton.position.left");
@@ -68,19 +68,24 @@ var gAppearancePane = {
       hbox.setAttribute("align", "center");
     }
 
+    // Firefox 57+ uses both smooth scroll and scrollDelay (see tabstrip._startScroll)
+    if (Tabmix.isVersion(570)) {
+      gPrefWindow.removeChild("obs_smoothScroll");
+    }
+
     gPrefWindow.initPane("paneAppearance");
     // call this function after initPane
     // we update some broadcaster that initPane may reset
     this.toolbarButtons(browserWindow);
   },
 
-  tabCloseButtonChanged: function() {
+  tabCloseButtonChanged() {
     var tabCbValue = $("pref_tabCloseButton").value;
     Tabmix.setItem("tabDelayCheck", "hidden", tabCbValue != 2 && tabCbValue != 4);
     Tabmix.setItem("tabWidthBox", "hidden", tabCbValue != 5);
   },
 
-  setTabCloseButtonUI: function() {
+  setTabCloseButtonUI() {
     if ($("pref_flexTabs").value) {
       $("alltabsItem").disabled = true;
       let tabCbUI = $("tabCloseButton");
@@ -93,18 +98,24 @@ var gAppearancePane = {
     }
   },
 
-  tabsScrollChanged: function() {
+  tabsScrollChanged() {
     var multiRow = $("pref_tabsScroll").value == 2;
     $("multi-rows").hidden = !multiRow;
     $("one-row").hidden = multiRow;
   },
 
-  tabmixCustomizeToolbar: function() {
+  tabmixCustomizeToolbar() {
     this._tabmixCustomizeToolbar = true;
-    Tabmix.getTopWin().BrowserCustomizeToolbar();
+    const win = Tabmix.getTopWin();
+    if (typeof win.gCustomizeMode == "object") {
+      // Firefox 57
+      win.gCustomizeMode.enter();
+    } else {
+      win.BrowserCustomizeToolbar();
+    }
   },
 
-  toolbarButtons: function(aWindow) {
+  toolbarButtons(aWindow) {
     // Display > Toolbar
     var buttons = ["btn_sessionmanager", "btn_undoclose", "btn_closedwindows", "btn_tabslist"];
     var onToolbar = $("onToolbar");
@@ -138,7 +149,7 @@ var gAppearancePane = {
 
   // block width change on instantApply
   // user is force to hit apply
-  userChangedWidth: function(item) {
+  userChangedWidth(item) {
     gPrefWindow.widthChanged = $("minWidth").value != $("pref_minWidth").valueFromPreferences ||
                         $("maxWidth").value != $("pref_maxWidth").valueFromPreferences;
     if (!gPrefWindow.instantApply)
@@ -148,7 +159,7 @@ var gAppearancePane = {
     return $(item.getAttribute("preference")).value;
   },
 
-  changeTabsWidth: function() {
+  changeTabsWidth() {
     if (!gPrefWindow.widthChanged)
       return;
     gPrefWindow.widthChanged = false;
@@ -158,15 +169,15 @@ var gAppearancePane = {
     [$("pref_minWidth").value, $("pref_maxWidth").value] = [minWidth, maxWidth];
   },
 
-  resetWidthChange: function() {
+  resetWidthChange() {
     gPrefWindow.widthChanged = false;
     $("minWidth").value = $("pref_minWidth").value;
     $("maxWidth").value = $("pref_maxWidth").value;
   },
 
-  openAdvanceAppearance: function() {
-    let url = "chrome://tabmixplus/content/preferences/subdialogs/pref-appearance.xul";
-    window.openDialog(url, "advanceAppearanceDialog", "modal,titlebar,toolbar,centerscreen");
+  openAdvanceAppearance() {
+    window.openDialog("chrome://tabmixplus/content/preferences/subdialogs/pref-appearance.xul",
+      "advanceAppearanceDialog", "modal,titlebar,toolbar,centerscreen");
   }
 
 };
