@@ -2,7 +2,7 @@
 "use strict";
 
 var gEventsPane = {
-  init: function() {
+  init() {
     // for locales with long labels
     var hbox = $("focusTab-box");
     var label = $("focusTab-label").boxObject.width;
@@ -13,6 +13,8 @@ var gEventsPane = {
       hbox.setAttribute("align", "center");
     }
 
+    $("keepMenuOpen").label = TabmixSvc.getString("undoclosetab.keepOpen.label");
+
     var browserWindow = Tabmix.getTopWin();
     let ctrlTab = browserWindow.document.getElementById("ctrlTab-panel") && "ctrlTab" in browserWindow;
     if (!ctrlTab) {
@@ -28,7 +30,7 @@ var gEventsPane = {
     this.disableReplaceLastTabWith();
     this.disableShowTabList();
 
-    var direction = window.getComputedStyle($("paneEvents"), null).direction;
+    var direction = window.getComputedStyle($("paneEvents")).direction;
     if (direction == "rtl") {
       let focusTab = $("focusTab").firstChild.childNodes;
       let [rightLabel, leftLabel] = [focusTab[2].label, focusTab[1].label];
@@ -37,19 +39,54 @@ var gEventsPane = {
       focusTab[5].label = focusTab[5].getAttribute("rtlLabel");
     }
 
-    // align Tab opening group boxes
-    var vbox1 = $("tabopening1");
-    var vbox2 = $("tabopening2");
-    var vbox3 = $("tabopening3");
-    var max = Math.max(vbox1.boxObject.width, vbox2.boxObject.width, vbox3.boxObject.width);
-    vbox1.style.setProperty("width", max + "px", "important");
-    vbox2.style.setProperty("width", max + "px", "important");
-    vbox3.style.setProperty("width", max + "px", "important");
+    // bug 1210586 - Create a Synced tabs sidebar
+    if (Tabmix.isVersion(470)) {
+      const {label: syncedTabs} = browserWindow.document.getElementById("menu_tabsSidebar");
+      $("syncedTabs").label = syncedTabs;
+      $("selectSyncedTabs").label = syncedTabs;
+    } else {
+      $("syncedTabs").hidden = true;
+      $("selectSyncedTabs").hidden = true;
+    }
+
+    // Bug 1352069 - Introduce a pref that allows for disabling cosmetic animations
+    // remove the UI for the old 'browser.tabs.animate'
+    if (Tabmix.isVersion(550)) {
+      gPrefWindow.removeChild("pref_disableTabsAnimate");
+      gPrefWindow.removeChild("disableTabsAnimate");
+    }
+
+    this.alignTabOpeningBoxes();
 
     gPrefWindow.initPane("paneEvents");
   },
 
-  disableShowTabList: function() {
+  // align Tab opening group boxes
+  // add setWidth attribute to columns that need to be aligned
+  alignTabOpeningBoxes() {
+    const widths = {};
+    const rows = $("tabopening").querySelectorAll("hbox");
+    function updateGrid(fn) {
+      for (let row of rows) {
+        let id = 0;
+        const cols = row.querySelectorAll("vbox");
+        for (let col of cols) {
+          if (++id && col.hasAttribute("setWidth")) {
+            fn(col, id);
+          }
+        }
+      }
+    }
+    updateGrid((col, id) => {
+      widths[id] = Math.max(widths[id] || 0, col.boxObject.width);
+    });
+
+    updateGrid((col, id) => {
+      col.style.setProperty("width", widths[id] + "px", "important");
+    });
+  },
+
+  disableShowTabList() {
     var ctrlTabPv = $("pref_ctrltab.tabPreviews");
     var disableShowTabList = $("pref_ctrltab").value &&
                              ctrlTabPv && ctrlTabPv.value;
@@ -58,7 +95,7 @@ var gEventsPane = {
       gPrefWindow.setDisabled("respondToMouse", disableShowTabList);
   },
 
-  disableReplaceLastTabWith: function() {
+  disableReplaceLastTabWith() {
     // we disable replaceLastTabWith if one of this test is true
     // browser.tabs.closeWindowWithLastTab = true OR
     // extensions.tabmix.keepLastTab = true
@@ -67,7 +104,7 @@ var gEventsPane = {
     this.newTabUrl($("pref_replaceLastTabWith"), disable, !disable);
   },
 
-  newTabUrl: function(preference, disable, setFocus) {
+  newTabUrl(preference, disable, setFocus) {
     var showTabUrlBox = preference.value == 4;
     var item = $(preference.id.replace("pref_", ""));
     var idnum = item.getAttribute("idnum") || "";
@@ -77,7 +114,7 @@ var gEventsPane = {
       $("newTabUrl" + idnum).focus();
   },
 
-  syncFromNewTabUrlPref: function(item) {
+  syncFromNewTabUrlPref(item) {
     var preference = $(item.getAttribute("preference"));
     // If the pref is set to the default, set the value to ""
     // to show the placeholder text
@@ -87,7 +124,7 @@ var gEventsPane = {
     return this.syncToNewTabUrlPref(value, TabmixSvc.aboutBlank);
   },
 
-  syncToNewTabUrlPref: function(value, def = TabmixSvc.aboutNewtab) {
+  syncToNewTabUrlPref(value, def = TabmixSvc.aboutNewtab) {
     // If the value is "", use about:newtab or about:blank.
     if (value === "") {
       return def;
@@ -97,18 +134,65 @@ var gEventsPane = {
     return undefined;
   },
 
-  onNewTabKeyDown: function(event) {
+  onNewTabKeyDown(event) {
     // block spaces from the user to go to about:newtab preference
     if (event.keyCode == 32) {
       event.preventDefault();
     }
   },
 
-  editSlideShowKey: function() {
+  editSlideShowKey() {
     document.documentElement.showPane($("paneMenu"));
     if (typeof gMenuPane == "object")
       gMenuPane.editSlideShowKey();
     else
       $("paneMenu").setAttribute("editSlideShowKey", true);
-  }
+  },
+
+  loadProgressively: {
+    syncToCheckBox(item) {
+      let preference = $(item.getAttribute("preference"));
+      if (preference.value === 0) {
+        preference.value = 1;
+      }
+      if (preference.hasAttribute("notChecked")) {
+        preference.setAttribute("notChecked", -Math.abs(preference.value));
+      }
+      this.setOnDemandDisabledState();
+      return preference.value > -1;
+    },
+
+    syncFromCheckBox(item) {
+      let preference = $(item.getAttribute("preference"));
+      let control = $(item.getAttribute("control"));
+      control.disabled = !item.checked;
+      return -preference.value;
+    },
+
+    syncFromPref(item) {
+      const preference = $(item.getAttribute("preference"));
+      const prefValue = Math.abs(preference.value);
+      this.setOnDemandMinValue(item, prefValue);
+      return prefValue;
+    },
+
+    setOnDemandMinValue(item, prefValue) {
+      if (item.id != "loadProgressively") {
+        return;
+      }
+      const onDemand = $("restoreOnDemand");
+      onDemand.min = item.valueNumber;
+      onDemand._enableDisableButtons();
+      const restoreOnDemand = $("pref_restoreOnDemand");
+      if (prefValue > Math.abs(restoreOnDemand.value)) {
+        restoreOnDemand.value = $("chk_restoreOnDemand").checked ? prefValue : -prefValue;
+      }
+    },
+
+    setOnDemandDisabledState() {
+      const disabled = $("pref_loadProgressively").value < 0 ||
+                       $("pref_restoreOnDemand").value < 0;
+      gPrefWindow.setDisabled("restoreOnDemand", disabled);
+    },
+  },
 };
