@@ -5,12 +5,16 @@
 
 var {classes: Cc, interfaces: Ci, utils: Cu} = Components;
 
-Cu.import("resource://gre/modules/XPCOMUtils.jsm");
-Cu.import("resource://gre/modules/Services.jsm");
+Cu.import("resource://gre/modules/XPCOMUtils.jsm", this);
+Cu.import("resource://gre/modules/Services.jsm", this);
+
+XPCOMUtils.defineLazyModuleGetter(this, "AppConstants",
+  "resource://gre/modules/AppConstants.jsm");
 
 // DocShellCapabilities exist since Firefox 27
 XPCOMUtils.defineLazyModuleGetter(this, "DocShellCapabilities",
   "resource:///modules/sessionstore/DocShellCapabilities.jsm");
+
 XPCOMUtils.defineLazyModuleGetter(this, "BrowserUtils",
   "resource://gre/modules/BrowserUtils.jsm");
 
@@ -21,7 +25,7 @@ XPCOMUtils.defineLazyModuleGetter(this, "NetUtil",
   "resource://gre/modules/NetUtil.jsm");
 
 XPCOMUtils.defineLazyModuleGetter(this, "TabmixSvc",
-  "resource://tabmixplus/Services.jsm");
+  "resource://tabmixplus/TabmixSvc.jsm");
 
 XPCOMUtils.defineLazyModuleGetter(this, "LinkNodeUtils",
   "resource://tabmixplus/LinkNodeUtils.jsm");
@@ -53,7 +57,7 @@ var TabmixContentHandler = {
     "Tabmix:collectOpener",
   ],
 
-  init: function() {
+  init() {
     this.MESSAGES.forEach(m => addMessageListener(m, this));
 
     if (PROCESS_TYPE_CONTENT) {
@@ -61,7 +65,7 @@ var TabmixContentHandler = {
     }
   },
 
-  receiveMessage: function({name, data}) {
+  receiveMessage({name, data}) {
     // The docShell might be gone. Don't process messages,
     // that will just lead to errors anyway.
     if (!docShell) {
@@ -79,7 +83,7 @@ var TabmixContentHandler = {
       }
       case "Tabmix:collectPermissions": {
         let caps = DocShellCapabilities.collect(docShell).join(",");
-        sendSyncMessage("Tabmix:collectPermissionsComplete", {caps: caps});
+        sendSyncMessage("Tabmix:collectPermissionsComplete", {caps});
         break;
       }
       case "Tabmix:resetContentName": {
@@ -91,7 +95,7 @@ var TabmixContentHandler = {
         // workaround for bug 1081891
         let title = content.document.title;
         if (title)
-          sendAsyncMessage("DOMTitleChanged", {title: title});
+          sendAsyncMessage("DOMTitleChanged", {title});
         break;
       }
       case "Tabmix:updateHistoryTitle": {
@@ -104,7 +108,7 @@ var TabmixContentHandler = {
           scrollX: content.scrollX,
           scrollY: content.scrollY
         };
-        sendAsyncMessage("Tabmix:updateScrollPosition", {scroll: scroll});
+        sendAsyncMessage("Tabmix:updateScrollPosition", {scroll});
         break;
       }
       case "Tabmix:setScrollPosition": {
@@ -118,7 +122,7 @@ var TabmixContentHandler = {
           postData: null
         };
         let sh = docShell.QueryInterface(Ci.nsIWebNavigation).sessionHistory
-                         .QueryInterface(Ci.nsISHistoryInternal);
+            .QueryInterface(Ci.nsISHistoryInternal);
         if (sh) {
           let entry = sh.getEntryAtIndex(sh.index, false);
           let postData = entry.postData;
@@ -136,7 +140,7 @@ var TabmixContentHandler = {
       }
       case "Tabmix:isFrameInContent": {
         let result = LinkNodeUtils.isFrameInContent(content, data.href, data.name);
-        sendAsyncMessage("Tabmix:isFrameInContentResult", {result: result});
+        sendAsyncMessage("Tabmix:isFrameInContentResult", {result, epoch: data.epoch});
         break;
       }
       case "Tabmix:collectOpener": {
@@ -146,17 +150,27 @@ var TabmixContentHandler = {
     }
   },
 
-  getCapabilities: function() {
+  getCapabilities() {
     return DocShellCapabilities.collect(docShell).join(",") || "";
   },
 
-  onDrop: function(event) {
-    let uri, name = { };
-    let linkHandler = Cc["@mozilla.org/content/dropped-link-handler;1"]
-                        .getService(Ci.nsIDroppedLinkHandler);
+  onDrop(event) {
+    let links;
+    const linkName = { };
+    const linkHandler = Cc["@mozilla.org/content/dropped-link-handler;1"]
+        .getService(Ci.nsIDroppedLinkHandler);
     try {
       // Pass true to prevent the dropping of javascript:/data: URIs
-      uri = linkHandler.dropLink(event, name, true);
+      if (TabmixSvc.version(520)) {
+        links = linkHandler.dropLinks(event, true);
+        // we can not send a message with array of wrapped nsIDroppedLinkItem
+        links = links.map(link => {
+          const {url, name, type} = link;
+          return {url, name, type};
+        });
+      } else {
+        links = [{url: linkHandler.dropLink(event, linkName, true)}];
+      }
     } catch (ex) {
       return;
     }
@@ -168,8 +182,8 @@ var TabmixContentHandler = {
         shiftKey: event.shiftKey,
         altKey: event.altKey
       },
-      uri: uri,
-      name: name.value
+      links,
+      name: linkName.value
     };
     let result = sendSyncMessage("Tabmix:contentDrop", data);
     if (result[0]) {
@@ -182,43 +196,67 @@ var TabmixContentHandler = {
 TabmixClickEventHandler = {
   init: function init(global) {
     if (TabmixSvc.version(380)) {
-      global.addEventListener("click", this, true);
+      global.addEventListener("click", event => {
+        let linkData = this.getLinkData(event);
+        if (linkData) {
+          let [href, node] = linkData;
+          let currentHref = event.originalTarget.ownerDocument.documentURI;
+          const divertMiddleClick = event.button == 1 && TabmixSvc.prefBranch.getBoolPref("middlecurrent");
+          const ctrlKey = AppConstants.platform == "macosx" ? event.metaKey : event.ctrlKey;
+          if (divertMiddleClick || ctrlKey ||
+              LinkNodeUtils.isSpecialPage(href, node, currentHref)) {
+            this.contentAreaClick(event, linkData);
+          }
+        }
+      }, true);
+      Cc["@mozilla.org/eventlistenerservice;1"]
+          .getService(Ci.nsIEventListenerService)
+          .addSystemEventListener(global, "click", this, true);
     }
   },
 
-  handleEvent: function(event) {
+  handleEvent(event) {
     switch (event.type) {
       case "click":
-        this.contentAreaClick(event);
+        this.contentAreaClick(event, this.getLinkData(event));
         break;
     }
   },
 
-  contentAreaClick: function(event) {
+  getLinkData(event) {
     // tabmix_isMultiProcessBrowser is undefined for remote browser when
     // window.gMultiProcessBrowser is true
     if (!event.isTrusted || event.defaultPrevented || event.button == 2 ||
         event.tabmix_isMultiProcessBrowser === false) {
-      return;
+      return null;
     }
 
-    let originalTarget = event.originalTarget;
-    let ownerDoc = originalTarget.ownerDocument;
+    let ownerDoc = event.originalTarget.ownerDocument;
 
     // let Firefox code handle click events from about pages
     if (!ownerDoc || /^about:(certerror|blocked|neterror)$/.test(ownerDoc.documentURI)) {
+      return null;
+    }
+
+    return this._hrefAndLinkNodeForClickEvent(event);
+  },
+
+  contentAreaClick(event, linkData) {
+    if (!linkData) {
       return;
     }
 
-    let [href, node, principal] = this._hrefAndLinkNodeForClickEvent(event);
+    let [href, node, principal] = linkData;
+    let ownerDoc = event.originalTarget.ownerDocument;
 
-    // get referrer attribute from clicked link and parse it
-    // if per element referrer is enabled, the element referrer overrules
-    // the document wide referrer
+    // first get document wide referrer policy, then
+    // get referrer attribute from clicked link and parse it and
+    // allow per element referrer to overrule the document wide referrer if enabled
     let referrerPolicy = TabmixSvc.version(380) ? ownerDoc.referrerPolicy : null;
-    if (TabmixSvc.version(420) &&
-        Services.prefs.getBoolPref("network.http.enablePerElementReferrer") &&
-        node) {
+    let setReferrerPolicy = node && (TabmixSvc.version(550) ||
+      TabmixSvc.version(420) &&
+      Services.prefs.getBoolPref("network.http.enablePerElementReferrer"));
+    if (setReferrerPolicy) {
       let value = node.getAttribute(TabmixSvc.version(450) ? "referrerpolicy" : "referrer");
       let referrerAttrValue = Services.netUtils.parseAttributePolicyString(value);
       let policy = TabmixSvc.version(490) ? "REFERRER_POLICY_UNSET" : "REFERRER_POLICY_DEFAULT";
@@ -227,6 +265,13 @@ TabmixClickEventHandler = {
       }
     }
 
+    let frameOuterWindowID;
+    if (TabmixSvc.version(540)) {
+      frameOuterWindowID = ownerDoc.defaultView.QueryInterface(Ci.nsIInterfaceRequestor)
+          .getInterface(Ci.nsIDOMWindowUtils)
+          .outerWindowID;
+    }
+
     let json = {
       button: event.button,
       shiftKey: event.shiftKey,
@@ -236,7 +281,9 @@ TabmixClickEventHandler = {
       href: null,
       title: null,
       bookmark: false,
-      referrerPolicy: referrerPolicy,
+      referrerPolicy,
+      frameOuterWindowID,
+      triggeringPrincipal: principal,
       originAttributes: principal ? principal.originAttributes : {},
       isContentWindowPrivate: TabmixSvc.version(510) && PrivateBrowsingUtils.isContentWindowPrivate(ownerDoc.defaultView),
     };
@@ -248,12 +295,16 @@ TabmixClickEventHandler = {
     // for the case there is no href
     let linkNode = href ? node : LinkNodeUtils.getNodeWithOnClick(event.target);
     if (linkNode) {
+      if (!href) {
+        json.originPrincipal = ownerDoc.nodePrincipal;
+        json.triggeringPrincipal = ownerDoc.nodePrincipal;
+      }
       linkNode = LinkNodeUtils.wrap(linkNode, TabmixUtils.focusedWindow(content),
-                                    href && event.button === 0);
+        href && event.button === 0);
     }
 
     let result = sendSyncMessage("TabmixContent:Click",
-                    {json: json, href: href, node: linkNode});
+      {json, href, node: linkNode});
     let data = result[0];
     if (data.where == "default")
       return;
@@ -308,6 +359,8 @@ TabmixClickEventHandler = {
           }
         }
       }
+      json.originPrincipal = ownerDoc.nodePrincipal;
+      json.triggeringPrincipal = ownerDoc.nodePrincipal;
 
       sendAsyncMessage("Content:Click", json);
       return;
@@ -330,7 +383,7 @@ TabmixClickEventHandler = {
    *       element. This includes SVG links, because callers expect |node|
    *       to behave like an <a> element, which SVG links (XLink) don't.
    */
-  _hrefAndLinkNodeForClickEvent: function(event) {
+  _hrefAndLinkNodeForClickEvent(event) {
     function isHTMLLink(aNode) {
       // Be consistent with what nsContextMenu.js does.
       return ((aNode instanceof content.HTMLAnchorElement && aNode.href) ||
@@ -367,12 +420,12 @@ TabmixClickEventHandler = {
     // callers expect <a>-like elements.
     // Note: makeURI() will throw if aUri is not a valid URI.
     return [href ? BrowserUtils.makeURI(href, null, baseURI).spec : null, null,
-            node && node.ownerDocument.nodePrincipal];
+      node && node.ownerDocument.nodePrincipal];
   },
 };
 
 var AboutNewTabHandler = {
-  init: function(global) {
+  init(global) {
     if (!TabmixSvc.version(420)) {
       return;
     }
@@ -394,13 +447,13 @@ var AboutNewTabHandler = {
     });
   },
 
-  receiveMessage: function({name}) {
+  receiveMessage({name}) {
     if (name == "Tabmix:updateTitlefrombookmark") {
       this.updateTitles();
     }
   },
 
-  updateTitles: function() {
+  updateTitles() {
     if (content && content.gGrid) {
       TabmixAboutNewTab.updateTitles(content.gGrid.cells);
     }
@@ -408,13 +461,13 @@ var AboutNewTabHandler = {
 };
 
 var ContextMenuHandler = {
-  init: function(global) {
+  init(global) {
     Cc["@mozilla.org/eventlistenerservice;1"]
-      .getService(Ci.nsIEventListenerService)
-      .addSystemEventListener(global, "contextmenu", this.prepareContextMenu, true);
+        .getService(Ci.nsIEventListenerService)
+        .addSystemEventListener(global, "contextmenu", this.prepareContextMenu, true);
   },
 
-  prepareContextMenu: function(event) {
+  prepareContextMenu(event) {
     if (event.defaultPrevented) {
       return;
     }
@@ -424,7 +477,7 @@ var ContextMenuHandler = {
       links = ContextMenu.getSelectedLinks(content).join("\n");
     }
 
-    sendRpcMessage("Tabmix:contextmenu", {links: links});
+    sendRpcMessage("Tabmix:contextmenu", {links});
   }
 };
 
@@ -432,11 +485,11 @@ const AMO = new RegExp("https://addons.mozilla.org/.+/firefox/addon/tab-mix-plus
 const BITBUCKET = "https://bitbucket.org/onemen/tabmixplus/issues?status=new&status=open";
 
 var TabmixPageHandler = {
-  init: function(global) {
+  init(global) {
     global.addEventListener("DOMContentLoaded", this);
   },
 
-  handleEvent: function(event) {
+  handleEvent(event) {
     const doc = content.document;
     if (event.target != doc) {
       return;
@@ -456,7 +509,7 @@ var TabmixPageHandler = {
   },
 
   buttonID: "tabmixplus-bug-report",
-  createAMOButton: function() {
+  createAMOButton() {
     const doc = content.document;
     const email = doc.querySelector('ul>li>.email[href="mailto:tabmix.onemen@gmail.com"]');
     if (email && !doc.getElementById(this.buttonID)) {
@@ -473,7 +526,7 @@ var TabmixPageHandler = {
   },
 
   count: 0,
-  moveAMOButton: function(eventType) {
+  moveAMOButton(eventType) {
     const doc = content.document;
     // add-review is null on DOMContentLoaded
     const addReview = doc.getElementById("add-review");
@@ -499,7 +552,7 @@ var TabmixPageHandler = {
     }
   },
 
-  styleBitbucket: function() {
+  styleBitbucket() {
     let createIssue = content.document.getElementById("create-issue-contextual");
     if (createIssue) {
       createIssue.classList.remove("aui-button-subtle");
