@@ -1,3 +1,4 @@
+/* exported TMP_ClosedTabs */
 "use strict";
 
 /*
@@ -34,13 +35,17 @@ var TMP_SessionStore = {
     return aData.entries[activeIndex] || {};
   },
 
-  getTitleFromTabState: function(aTab) {
+  getTitleFromTabState(aTab) {
     let tabData = TabmixSvc.JSON.parse(TabmixSvc.ss.getTabState(aTab));
-    return this.getActiveEntryData(tabData).title || null;
+    let data = this.getActiveEntryData(tabData);
+    if (data.url == TabmixSvc.aboutBlank) {
+      return gBrowser.mStringBundle.getString("tabs.emptyTabTitle");
+    }
+    return data.title || null;
   },
 
   // check if pending tab has no history or is about:blank
-  isBlankPendingTab: function(aTab) {
+  isBlankPendingTab(aTab) {
     if (!aTab.hasAttribute("pending"))
       return false;
     let tabData = TabmixSvc.JSON.parse(TabmixSvc.ss.getTabState(aTab));
@@ -50,15 +55,15 @@ var TMP_SessionStore = {
     return !entries[0] || entries[0].url == "about:blank";
   },
 
- /**
-  * @brief       - Add attribute to nsSessionStore persistTabAttribute.
-  *
-  *   we call this after nsSessionStore.init
-  *   we add this also when we use TMP session manager.
-  *   we use Firefox SessionStore closed tab service and for restore after restart
-  *
-  * @returns        Nothing.
-  */
+  /**
+   * @brief       - Add attribute to nsSessionStore persistTabAttribute.
+   *
+   *   we call this after nsSessionStore.init
+   *   we add this also when we use TMP session manager.
+   *   we use Firefox SessionStore closed tab service and for restore after restart
+   *
+   * @returns        Nothing.
+   */
   persistTabAttribute: function TMP_ss_persistTabAttribute() {
     if (TabmixSvc.sm.persistTabAttributeSet)
       return;
@@ -74,7 +79,7 @@ var TMP_SessionStore = {
       if ("TreeStyleTabBrowser" in window)
         _xulAttributes = _xulAttributes.concat(TabmixSessionData.tabTSTProperties);
 
-      _xulAttributes.forEach(function(aAttr) {
+      _xulAttributes.forEach(aAttr => {
         TabmixSvc.ss.persistTabAttribute(aAttr);
       });
 
@@ -84,29 +89,29 @@ var TMP_SessionStore = {
     }
   },
 
- /**
-  * @brief         make sure that we don't enable both sessionStore and session manager
-  *
-  * @param msgNo   a Integer value - msg no. to show.
-  *                -1 when session manager extension enabled (see SessionManagerExtension.jsm)
-  *
-  * @param start   a Boolean value - true if we call this function before startup.
-  *
-  * @returns       Nothing.
-  */
+  /**
+   * @brief         make sure that we don't enable both sessionStore and session manager
+   *
+   * @param msgNo   a Integer value - msg no. to show.
+   *                -1 when session manager extension enabled (see SessionManagerExtension.jsm)
+   *
+   * @param start   a Boolean value - true if we call this function before startup.
+   *
+   * @returns       Nothing.
+   */
   setService: function TMP_ss_setSessionService(msgNo, start) {
     if (TabmixSvc.sm.settingPreference || Tabmix.prefs.prefHasUserValue("setDefault"))
       return;
-   /*
-    * From 2008-03-10 we don't set browser.sessionstore.enabled to false anymore
-    * we use nsISessionStore service in TMP.
-    * if we use TMP session manager we set all other sessionstore pref to false to disable SessionRestore
-    *
-    * Bug 449596 - remove the browser.sessionstore.enabled pref
-    * so here we don't set it to true, we just clear user pref to the default
-    * if the pref exist in firefox this set the pref to true
-    * if the pref don't exist this will remove the pref
-    */
+    /*
+     * From 2008-03-10 we don't set browser.sessionstore.enabled to false anymore
+     * we use nsISessionStore service in TMP.
+     * if we use TMP session manager we set all other sessionstore pref to false to disable SessionRestore
+     *
+     * Bug 449596 - remove the browser.sessionstore.enabled pref
+     * so here we don't set it to true, we just clear user pref to the default
+     * if the pref exist in firefox this set the pref to true
+     * if the pref don't exist this will remove the pref
+     */
     const TMP_SS_MANAGER = "extensions.tabmix.sessions.manager";
     const TMP_SS_CRASHRECOVERY = "extensions.tabmix.sessions.crashRecovery";
     var TMP_manager_enabled = Services.prefs.getBoolPref(TMP_SS_MANAGER);
@@ -175,7 +180,7 @@ var TMP_SessionStore = {
         TabmixSvc.sm.settingPreference = false;
       };
       let result = Tabmix.promptService([Tabmix.BUTTON_OK, Tabmix.HIDE_MENUANDTEXT, Tabmix.HIDE_CHECKBOX],
-                                        [title, msg, "", "", buttons], window, start ? callBack : null);
+        [title, msg, "", "", buttons], window, start ? callBack : null);
       if (!start)
         callBack(result);
     } else if (!Services.prefs.prefHasUserValue("browser.warnOnQuit ")) {
@@ -185,7 +190,7 @@ var TMP_SessionStore = {
     }
   },
 
-  isSessionStoreEnabled: function() {
+  isSessionStoreEnabled() {
     return Services.prefs.getIntPref("browser.startup.page") == 3 ||
       Services.prefs.getBoolPref("browser.sessionstore.resume_from_crash");
   },
@@ -194,7 +199,7 @@ var TMP_SessionStore = {
   // we call this only one time on window load
   // and store the value in Tabmix.isWindowAfterSessionRestore
   // we call this from onContentLoaded before nsSessionStore run its onLoad
-  setAfterSessionRestored: function() {
+  setAfterSessionRestored() {
     let afterSessionRestore;
     if (!Tabmix.isFirstWindow)
       afterSessionRestore = false;
@@ -217,13 +222,13 @@ var TMP_SessionStore = {
       // calling doRestore before sessionstartup finished to read
       // sessionStore.js file throw error since Firefox 28, and force
       // syncRead in Firefox 25-27
-      XPCOMUtils.defineLazyGetter(Tabmix, "isWindowAfterSessionRestore", function() {
+      XPCOMUtils.defineLazyGetter(Tabmix, "isWindowAfterSessionRestore", () => {
         let ss = Cc["@mozilla.org/browser/sessionstartup;1"]
-                   .getService(Ci.nsISessionStartup);
+            .getService(Ci.nsISessionStartup);
         // when TMP session manager is enabled ss.doRestore is true only after restart
-        ss.onceInitialized.then(function() {
+        ss.onceInitialized.then(() => {
           Tabmix.isWindowAfterSessionRestore = ss.doRestore();
-        }).then(null, Tabmix.reportError);
+        }).catch(Tabmix.reportError);
         // until sessionstartup initialized just return the pref value,
         // we only use isWindowAfterSessionRestore when our Session Manager enable
         return Services.prefs.getBoolPref("browser.sessionstore.resume_session_once");
@@ -231,7 +236,7 @@ var TMP_SessionStore = {
     }
   },
 
-  setSessionRestore: function(aEnable) {
+  setSessionRestore(aEnable) {
     Services.prefs.setBoolPref("browser.warnOnQuit", aEnable);
     Services.prefs.setBoolPref("browser.sessionstore.resume_from_crash", aEnable);
     if (aEnable)
@@ -240,17 +245,17 @@ var TMP_SessionStore = {
       Services.prefs.setIntPref("browser.startup.page", 1);
   },
 
- /**
-  * @brief           update tab title from user name or bookmark.
-  *
-  * @param aTabData  an object value - tabData from nsSessionStore
-  *
-  * @param aUri      string value - url address
-  *
-  * @param aTitle    string value - title
-  *
-  * @returns         tab title - string.
-  */
+  /**
+   * @brief           update tab title from user name or bookmark.
+   *
+   * @param aTabData  an object value - tabData from nsSessionStore
+   *
+   * @param aUri      string value - url address
+   *
+   * @param aTitle    string value - title
+   *
+   * @returns         tab title - string.
+   */
   _getTitle: function ct_getTitle(aData, aUri, aTitle) {
     var fixedLabelUri = this._getAttribute(aData, "label-uri");
     if (fixedLabelUri == aUri || fixedLabelUri == "*")
@@ -259,15 +264,15 @@ var TMP_SessionStore = {
     return TMP_Places.getTitleFromBookmark(aUri, aTitle, this._getAttribute(aData, "tabmix_bookmarkId"));
   },
 
- /**
-  * @brief           get attribute xultab data
-  *
-  * @param aTabData  an object value - tabData from nsSessionStore
-  *
-  * @param attrib    attribute name as string
-  *
-  * @returns         attribute value as string or empty string.
-  */
+  /**
+   * @brief           get attribute xultab data
+   *
+   * @param aTabData  an object value - tabData from nsSessionStore
+   *
+   * @param attrib    attribute name as string
+   *
+   * @returns         attribute value as string or empty string.
+   */
   _getAttribute: function TMP_ss__getAttribute(aTabData, attrib) {
     if (aTabData.attributes && attrib in aTabData.attributes)
       return aTabData.attributes[attrib];
@@ -294,7 +299,7 @@ var TMP_ClosedTabs = {
   },
 
   // make btn_undoclose single-functionality or dual-functionality
-  setButtonType: function(menuOnly) {
+  setButtonType(menuOnly) {
     var buttonType = menuOnly ? "menu" : "menu-button";
     if (this.buttonBroadcaster.getAttribute("type") != buttonType)
       this.buttonBroadcaster.setAttribute("type", buttonType);
@@ -306,16 +311,16 @@ var TMP_ClosedTabs = {
     Tabmix.setItem(this.buttonBroadcaster, "disabled", aState || null);
   },
 
- /**
-  * Get closed tabs count
-  */
+  /**
+   * Get closed tabs count
+   */
   get count() {
     return window.__SSi ? TabmixSvc.ss.getClosedTabCount(window) : 0;
   },
 
- /**
-  * Get closed tabs data
-  */
+  /**
+   * Get closed tabs data
+   */
   get getClosedTabData() {
     return window.__SSi ? TabmixSvc.JSON.parse(TabmixSvc.ss.getClosedTabData(window)) : {};
   },
@@ -330,11 +335,24 @@ var TMP_ClosedTabs = {
 
   /* .......... functions for closedtabs list menu and context menu .......... */
 
-  populateUndoSubmenu: function ct_populateUndoSubmenu(aPopup) {
-    /* eslint-disable tabmix/balanced-listeners */
+  get keepMenuOpen() {
+    return Tabmix.prefs.getBoolPref("undoClose.keepMenuOpen");
+  },
+
+  set keepMenuOpen(val) {
+    Tabmix.prefs.setBoolPref("undoClose.keepMenuOpen", Boolean(val));
+    return val;
+  },
+
+  populateUndoSubmenu: function ct_populateUndoSubmenu(aPopup, keepWidth) {
     if (TabmixAllTabs.isAfterCtrlClick(aPopup.parentNode))
       return false;
 
+    if (keepWidth && !aPopup.hasAttribute("width")) {
+      const width = Tabmix.getBoundsWithoutFlushing(aPopup).width;
+      aPopup.setAttribute("width", width);
+    }
+
     TabmixAllTabs.beforeCommonList(aPopup, true);
 
     // populate menu
@@ -349,11 +367,14 @@ var TMP_ClosedTabs = {
       var _uri = makeURI(url);
       if (_uri.scheme == "about" && title === "")
         url = title = "about:blank";
-      else try {
-        url = _uri.scheme == "about" ? _uri.spec :
-          _uri.scheme + "://" + _uri.hostPort + _uri.path;
-      } catch (e) {
-        url = title;
+      else {
+        try {
+          const pathProp = TabmixSvc.version(570) ? "pathQueryRef" : "path";
+          url = _uri.scheme == "about" ? _uri.spec :
+            _uri.scheme + "://" + _uri.hostPort + _uri[pathProp];
+        } catch (e) {
+          url = title;
+        }
       }
       var label = title ? title : url;
       let count = "";
@@ -376,65 +397,107 @@ var TMP_ClosedTabs = {
       }
       m.setAttribute("class", "menuitem-iconic bookmark-item menuitem-with-favicon");
       m.setAttribute("value", i);
-      m.addEventListener("command", TMP_ClosedTabs.restoreCommand, false);
-      m.addEventListener("click", TMP_ClosedTabs.checkForMiddleClick, false);
+      m.setAttribute("closemenu", this.keepMenuOpen ? "none" : "auto");
+      /* eslint-disable tabmix/balanced-listeners */
+      m.addEventListener("command", this);
+      m.addEventListener("click", this);
+      /* eslint-enable tabmix/balanced-listeners */
       if (i === 0)
         m.setAttribute("key", "key_undoCloseTab");
       aPopup.appendChild(m);
     }
 
     aPopup.appendChild(document.createElement("menuseparator"));
-    function addKey(item, id) {
-      if (document.getElementById("key_tm_" + id)) {
-        item.setAttribute("key", "key_tm_" + id);
-      }
-    }
-    // "Clear Closed Tabs List"
-    m = aPopup.appendChild(document.createElement("menuitem"));
-    m.setAttribute("id", "clearClosedTabsList");
-    m.setAttribute("label", TabmixSvc.getString("undoclosetab.clear.label"));
-    m.setAttribute("value", -1);
-    addKey(m, "clearClosedTabs");
-    m.addEventListener("command", function() {
-      TMP_ClosedTabs.restoreTab('original', -1);
-    });
 
+    const addMenu = this.addMenuItem.bind(this, aPopup);
+    // "Keep menu open"
+    m = addMenu("lockedClosedTabsList", TabmixSvc.getString("undoclosetab.keepOpen.label"), -3);
+    m.setAttribute("description", TabmixSvc.getString("undoclosetab.keepOpen.description"));
+    m.setAttribute("closemenu", "none");
+    const image = this.keepMenuOpen ? "chrome://tabmixplus/skin/pin.png" : "";
+    m.setAttribute("image", image);
+    // "Clear Closed Tabs List"
+    addMenu("clearClosedTabsList", TabmixSvc.getString("undoclosetab.clear.label"), -1, "clearClosedTabs");
     // "Restore All Tabs"
-    m = aPopup.appendChild(document.createElement("menuitem"));
-    m.setAttribute("id", "restoreAllClosedTabs");
-    m.setAttribute("label", gNavigatorBundle.getString("menuRestoreAllTabs.label"));
-    m.setAttribute("value", -2);
-    addKey(m, "ucatab");
-    m.addEventListener("command", function() {
-      TMP_ClosedTabs.restoreTab('original', -2);
-    });
+    addMenu("restoreAllClosedTabs", gNavigatorBundle.getString("menuRestoreAllTabs.label"), -2, "ucatab");
+
     return true;
-    /* eslint-enable tabmix/balanced-listeners */
   },
 
-  restoreCommand: function(aEvent) {
-    let index = aEvent.originalTarget.getAttribute("value");
-    if (index < 0) {
+  addMenuItem(popup, id, label, val, keyId) {
+    const m = popup.appendChild(document.createElement("menuitem"));
+    m.setAttribute("id", id);
+    m.setAttribute("label", label);
+    m.setAttribute("value", val);
+    m.setAttribute("class", "menuitem-iconic");
+    if (keyId && document.getElementById("key_tm_" + keyId)) {
+      m.setAttribute("key", "key_tm_" + keyId);
+    }
+    /* eslint-disable tabmix/balanced-listeners */
+    m.addEventListener("command", this);
+    return m;
+  },
+
+  handleEvent(event) {
+    switch (event.type) {
+      case "click":
+        this.checkForMiddleClick(event);
+        break;
+      case "command":
+        this.restoreCommand(event);
+        break;
+    }
+  },
+
+  restoreCommand(aEvent) {
+    const item = aEvent.originalTarget;
+    const index = Number(item.getAttribute("value"));
+    if (index == -3) {
+      this.keepMenuOpen = !this.keepMenuOpen;
+      const image = this.keepMenuOpen ? "chrome://tabmixplus/skin/pin.png" : "";
+      item.setAttribute("image", image);
+      this.populateUndoSubmenu(item.parentNode, true);
       return;
     }
-    TMP_ClosedTabs.restoreTab("original", index);
+
+    this.doCommand("restoreTab", "original", item);
   },
 
   checkForMiddleClick: function ct_checkForMiddleClick(aEvent) {
     if (aEvent.button != 1)
       return;
 
-    let index = aEvent.originalTarget.getAttribute("value");
-    if (index < 0)
-      return;
+    const deleteItem = Tabmix.prefs.getBoolPref("middleclickDelete");
+    const where = deleteItem ? "delete" : "tab";
+    this.doCommand("restoreTab", where, aEvent.originalTarget, deleteItem);
+  },
 
-    let deleteItem = Tabmix.prefs.getBoolPref("middleclickDelete");
-    TMP_ClosedTabs.restoreTab(deleteItem ? "delete" : "tab", index);
-    if (deleteItem && TMP_ClosedTabs.count > 0) {
-      aEvent.stopPropagation();
-      TMP_ClosedTabs.populateUndoSubmenu(aEvent.originalTarget.parentNode);
-    } else {
-      closeMenus(aEvent.target);
+  contextMenuOnPopupShowing(popup) {
+    const val = this.keepMenuOpen ? "single" : "auto";
+    Array.prototype.forEach.call(popup.childNodes, item => {
+      item.setAttribute("closemenu", val);
+    });
+    return popup.triggerNode.value >= 0;
+  },
+
+  contextMenuOnCommand(event) {
+    const menuItem = event.originalTarget;
+    const [command, where] = menuItem.getAttribute("commandData").split(",");
+    const popup = menuItem.parentNode;
+    this.doCommand(command, where, popup.triggerNode);
+  },
+
+  doCommand(command, where, item, keepMenuOpen) {
+    const popup = item.parentNode;
+    const index = Number(item.getAttribute("value"));
+    this[command](where || index, index);
+    const rePopulate = (keepMenuOpen || this.keepMenuOpen) && this.count > 0;
+    if (rePopulate) {
+      if (popup && command == "restoreTab") {
+        this.populateUndoSubmenu(popup, true);
+      }
+    } else if (!this.count || item.getAttribute("closemenu") == "none") {
+      closeMenus(popup);
     }
   },
 
@@ -449,7 +512,7 @@ var TMP_ClosedTabs = {
     var tabData = this.getClosedTabData[index];
     var url = this.getUrl(tabData);
     var clipboard = Components.classes["@mozilla.org/widget/clipboardhelper;1"]
-    .getService(Components.interfaces.nsIClipboardHelper);
+        .getService(Components.interfaces.nsIClipboardHelper);
 
     clipboard.copyString(url);
   },
@@ -477,7 +540,7 @@ var TMP_ClosedTabs = {
     }
   },
 
-  removeAllClosedTabs: function() {
+  removeAllClosedTabs() {
     // update our session data
     var updateRDF = TabmixSessionManager.enableBackup && Tabmix.prefs.getBoolPref("sessions.save.closedtabs");
     if (updateRDF)
@@ -487,11 +550,11 @@ var TMP_ClosedTabs = {
     this.setButtonDisableState(true);
   },
 
- /**
-  * @brief           fetch the data of closed tab, while removing it from the array
-  * @param aIndex    a Integer value - 0 or grater index to remove
-  * @returns         closed tab data at aIndex.
-  */
+  /**
+   * @brief           fetch the data of closed tab, while removing it from the array
+   * @param aIndex    a Integer value - 0 or grater index to remove
+   * @returns         closed tab data at aIndex.
+   */
   getClosedTabAtIndex: function ct_getClosedTabAtIndex(aIndex) {
     if (aIndex < 0 || aIndex >= this.count)
       return null;
@@ -541,7 +604,7 @@ var TMP_ClosedTabs = {
     }
   },
 
-  SSS_undoCloseTab: function(aIndex, aWhere, aSelectRestoredTab, aBlankTabToReuse, skipAnimation) {
+  SSS_undoCloseTab(aIndex, aWhere, aSelectRestoredTab, aBlankTabToReuse, skipAnimation) {
     if (!Tabmix.prefs.getBoolPref("undoClose") || this.count === 0)
       return null;
 
@@ -571,11 +634,11 @@ var TMP_ClosedTabs = {
         (!Tabmix.isVersion(490) ||
         aBlankTabToReuse.getAttribute("usercontextid") == (userContextId || ""));
     let newTab = reuseExisting ? aBlankTabToReuse :
-        gBrowser.addTab("about:blank", {
-          skipAnimation: tabToRemove || skipAnimation,
-          dontMove: true,
-          userContextId: userContextId,
-        });
+      gBrowser.addTab("about:blank", {
+        skipAnimation: tabToRemove || skipAnimation,
+        dontMove: true,
+        userContextId,
+      });
     if (!reuseExisting && aBlankTabToReuse) {
       gBrowser.removeTab(aBlankTabToReuse, {animate: false});
     }
@@ -648,7 +711,7 @@ var TabmixConvertSession = {
     TabmixSessionManager.saveStateDelayed();
     var callBack = function(aResult) {
       if (aResult.button == Tabmix.BUTTON_OK) {
-        setTimeout(function(a, b) {
+        setTimeout((a, b) => {
           TabmixConvertSession.convertFile(a, b);
         }, 50, null, true);
       }
@@ -659,10 +722,10 @@ var TabmixConvertSession = {
   selectFile: function cs_selectFile(aWindow) {
     const nsIFilePicker = Ci.nsIFilePicker;
     var fp = Cc["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
-    var fpCallback = function fpCallback_done(aResult) {
+    var fpCallback = aResult => {
       if (aResult == nsIFilePicker.returnOK)
         this.convertFile(fp.fileURL.spec);
-    }.bind(this);
+    };
 
     fp.init(aWindow, this.getString("selectfile"), nsIFilePicker.modeOpen);
     fp.defaultString = "session.rdf";
@@ -686,7 +749,7 @@ var TabmixConvertSession = {
   confirm: function cs_confirm(aMsg, aCallBack) {
     let buttons = TabmixSvc.getDialogStrings("Yes", "No").join("\n");
     return Tabmix.promptService([Tabmix.BUTTON_OK, Tabmix.HIDE_MENUANDTEXT, Tabmix.HIDE_CHECKBOX],
-                                [this.getTitle, aMsg, "", "", buttons], window, aCallBack);
+      [this.getTitle, aMsg, "", "", buttons], window, aCallBack);
   },
 
   getSessionState: function cs_getSessionState(aPath, internal) {
@@ -711,7 +774,7 @@ var TabmixConvertSession = {
     return {
       windows: _windows,
       selectedWindow: _windows.indexOf(selected) + 1,
-      tabsCount: tabsCount,
+      tabsCount,
     };
   },
 
@@ -769,7 +832,7 @@ var TabmixConvertSession = {
         tabsData.push(new _tabData(rdfNodeTab));
       }
     }
-    tabsData.sort(function(a, b) {
+    tabsData.sort((a, b) => {
       return a - b;
     });
     for (let i = 0; i < tabsData.length; i++) {
@@ -805,9 +868,11 @@ var TabmixConvertSession = {
 
   getTabState: function cs_getTabState(rdfNodeTab, aClosedTab, internal) {
     var tabData = {entries: [], index: 0, zoom: 1, disallow: "", text: ""};
-    tabData.entries = this.getHistoryState(rdfNodeTab);
-    if (!tabData.entries.length)
+    const entries = this.getHistoryState(rdfNodeTab);
+    if (!entries.length) {
       return null;
+    }
+    tabData.entries = this.addTriggeringPrincipal(entries);
     tabData.image = TabmixSessionManager.getLiteralValue(rdfNodeTab, "image", null);
     let index = TabmixSessionManager.getIntValue(rdfNodeTab, "index");
     tabData.index = Math.max(1, Math.min(index + 1, tabData.entries.length));
@@ -818,7 +883,7 @@ var TabmixConvertSession = {
       // until version 0.4.1.5 textZoom was included in scroll data
       scroll = scroll.split(",").splice(0, 2).join(",");
       if (scroll != "0,0") {
-        tabData.scroll = {scroll: scroll};
+        tabData.scroll = {scroll};
       }
     }
     tabData.userContextId = TabmixSessionManager.getIntValue(rdfNodeTab, "userContextId", 0);
@@ -846,13 +911,13 @@ var TabmixConvertSession = {
       let TSTProps = properties.split('|');
       properties = TSTProps.shift();
       let PREFIX = "tmp-session-data-";
-      TSTProps.forEach(function(aProp) {
+      TSTProps.forEach(aProp => {
         if (/^([^\s=]+)=(.*)/.test(aProp) &&
             RegExp.$1.startsWith(PREFIX) && RegExp.$2)
           extData[RegExp.$1.substr(PREFIX.length)] = decodeURIComponent(RegExp.$2);
       });
       properties = properties.substr(booleanAttrLength + 1).split(" ");
-      properties.forEach(function(aAttr) {
+      properties.forEach(aAttr => {
         aAttr = TabmixSessionManager.getDecodedLiteralValue(null, aAttr);
         if (!/^([^\s=]+)=(.*)/.test(aAttr))
           return;
@@ -887,6 +952,10 @@ var TabmixConvertSession = {
             extData.tabClr = RegExp.$2;
             break;
           default:
+            // treestyletab data
+            if (RegExp.$1.startsWith("treestyletab-")) {
+              extData[RegExp.$1] = RegExp.$2;
+            }
             tabData.attributes[RegExp.$1] = RegExp.$2;
         }
       });
@@ -903,6 +972,13 @@ var TabmixConvertSession = {
   },
 
   getHistoryState: function cs_getHistoryState(rdfNodeTab) {
+    // starting with version 0.5.0.3 history data serialized with JSON.stringify
+    let isJSONData = TabmixSessionManager.nodeHasArc(rdfNodeTab, "historyData");
+    if (isJSONData) {
+      const state = TabmixSessionManager.getLiteralValue(rdfNodeTab, "historyData");
+      return JSON.parse(decodeURI(state));
+    }
+
     let decodeData = function(data, decode) {
       return decode ? TabmixSessionManager.getDecodedLiteralValue(null, data) : data;
     };
@@ -929,5 +1005,18 @@ var TabmixConvertSession = {
       }
     }
     return entries;
-  }
+  },
+
+  // add triggeringPrincipal to history entries that was saved before Firefox 54 (Bug 1307736)
+  addTriggeringPrincipal(entries) {
+    if (!TabmixSvc.SERIALIZED_SYSTEMPRINCIPAL) {
+      return entries;
+    }
+    return entries.map(entry => {
+      if (!entry.triggeringPrincipal_base64) {
+        entry.triggeringPrincipal_base64 = TabmixSvc.SERIALIZED_SYSTEMPRINCIPAL;
+      }
+      return entry;
+    });
+  },
 };
