@@ -54,6 +54,11 @@ Tabmix.beforeDelayedStartup = function() {
 
 // after TabmixSessionManager and SessionStore initialized
 Tabmix.sessionInitialized = function() {
+  // Let EmbeddedWebExtension know we're done.
+  if (this.firstWindowInSession && TabmixSvc.sm.deferredInitialized) {
+    TabmixSvc.sm.deferredInitialized.resolve();
+  }
+
   var SM = TabmixSessionManager;
   if (SM.enableManager) {
     window.restoreLastSession = function restoreLastSession() {
@@ -61,7 +66,7 @@ Tabmix.sessionInitialized = function() {
     };
 
     this.setItem("Browser:RestoreLastSession", "disabled",
-                 !SM.canRestoreLastSession || SM.isPrivateWindow);
+      !SM.canRestoreLastSession || SM.isPrivateWindow);
 
     if (TabmixSvc.isPaleMoon) {
       this.changeCode(window, "window.BrowserOnAboutPageLoad")._replace(
@@ -105,7 +110,7 @@ Tabmix.getButtonsHeight = function(setDefault) {
     let stripIsHidden = TabmixTabbar.hideMode !== 0 && !tabBar.visible;
     if (stripIsHidden)
       tabBar.visible = true;
-    this._buttonsHeight = Tabmix.visibleTabs.first.getBoundingClientRect().height;
+    this._buttonsHeight = Tabmix.getBoundsWithoutFlushing(Tabmix.visibleTabs.first).height;
     if (stripIsHidden)
       tabBar.visible = false;
 
@@ -115,6 +120,11 @@ Tabmix.getButtonsHeight = function(setDefault) {
   } else {
     this._buttonsHeight = 24;
   }
+
+  if (TabmixSvc.isPaleMoon && this.isVersion(0, 270)) {
+    this._buttonsHeight = Math.round(this._buttonsHeight);
+  }
+
   return this._buttonsHeight;
 };
 
@@ -129,8 +139,8 @@ Tabmix.getAfterTabsButtonsWidth = function TMP_getAfterTabsButtonsWidth() {
     this.tabsNewtabButton =
       document.getAnonymousElementByAttribute(tabBar, "command", "cmd_newNavigatorTab");
     this.tabsNewtabButton.setAttribute("force-display", true);
-    let openNewTabRect = this.tabsNewtabButton.getBoundingClientRect();
-    let style = window.getComputedStyle(this.tabsNewtabButton, null);
+    let openNewTabRect = Tabmix.getBoundsWithoutFlushing(this.tabsNewtabButton);
+    let style = window.getComputedStyle(this.tabsNewtabButton);
     let marginStart = style.getPropertyValue("margin-left");
     // it doesn't work when marginEnd add to buttonWidth
     // let marginEnd = style.getPropertyValue("margin-right");
@@ -146,7 +156,7 @@ Tabmix.getAfterTabsButtonsWidth = function TMP_getAfterTabsButtonsWidth() {
     // the right button
     let openNewPrivateTab = document.getElementById("privateTab-afterTabs-openNewPrivateTab");
     if (openNewPrivateTab) {
-      let openNewPrivateTabRect = openNewPrivateTab.getBoundingClientRect();
+      let openNewPrivateTabRect = Tabmix.getBoundsWithoutFlushing(openNewPrivateTab);
       this.afterTabsButtonsWidth.push(openNewPrivateTabRect.width);
       if (openNewPrivateTabRect.right > openNewTabRect.right)
         this.tabsNewtabButton = openNewPrivateTab;
@@ -158,6 +168,13 @@ Tabmix.getAfterTabsButtonsWidth = function TMP_getAfterTabsButtonsWidth() {
 };
 
 Tabmix.afterDelayedStartup = function() {
+  // focus content area if the selected tab is not blank when Firefox starts
+  setTimeout(() => {
+    if (gURLBar.focused && !gBrowser.isBlankNotBusyTab(gBrowser.selectedTab)) {
+      gBrowser.selectedBrowser.focus();
+    }
+  }, 250);
+
   TabmixTabbar._enablePositionCheck = true;
 
   TMP_TabView.init();
@@ -226,7 +243,7 @@ Tabmix.afterDelayedStartup = function() {
     let buttons = [{
       label: "Disable Debug Mode",
       accessKey: "D",
-      callback: function() {
+      callback() {
         Tabmix.prefs.setBoolPref("enableDebug", false);
       }
     }];
@@ -236,14 +253,14 @@ Tabmix.afterDelayedStartup = function() {
       "Once you disable 'Debug Mode' restart your browser.";
     const errorimage = "chrome://tabmixplus/skin/tmpsmall.png";
     gnb.appendNotification(msg, "tabmix-debugmode-enabled",
-        errorimage, gnb.PRIORITY_CRITICAL_HIGH, buttons);
+      errorimage, gnb.PRIORITY_CRITICAL_HIGH, buttons);
   }
 };
 
 var TMP_eventListener = {
   init: function TMP_EL_init() {
-    window.addEventListener("DOMContentLoaded", this, false);
-    window.addEventListener("load", this, false);
+    window.addEventListener("DOMContentLoaded", this);
+    window.addEventListener("load", this);
   },
 
   handleEvent: function TMP_EL_handleEvent(aEvent) {
@@ -252,11 +269,11 @@ var TMP_eventListener = {
         this.onTabAttrModified(aEvent);
         break;
       case "SSWindowClosing":
-        window.removeEventListener("SSWindowClosing", this, false);
+        window.removeEventListener("SSWindowClosing", this);
         TabmixSessionManager.onWindowClose(!Tabmix.numberOfWindows());
         break;
       case "SSTabRestoring":
-        this.onSSTabRestoring(aEvent);
+        this.onSSTabRestoring(aEvent.target);
         break;
       case "TabOpen":
         this.onTabOpen(aEvent);
@@ -299,7 +316,7 @@ var TMP_eventListener = {
     }
   },
 
-  toggleEventListener: function(aObj, aArray, aEnable, aHandler) {
+  toggleEventListener(aObj, aArray, aEnable, aHandler) {
     var handler = aHandler || this;
     var eventListener = aEnable ? "addEventListener" : "removeEventListener";
     aArray.forEach(function(eventName) {
@@ -309,28 +326,34 @@ var TMP_eventListener = {
 
   // ignore non-browser windows
   _onLoad: function TMP_EL_onContentLoaded(aType) {
-    window.removeEventListener(aType, this, false);
+    window.removeEventListener(aType, this);
     let wintype = window.document.documentElement.getAttribute("windowtype");
-    if (wintype == "navigator:browser")
+    if (wintype == "navigator:browser") {
       if (aType != "load") {
         Tabmix.initialization.run("onContentLoaded");
         Tabmix.initialization.run("beforeBrowserInitOnLoad");
       } else {
         Tabmix.initialization.run("onWindowOpen");
       }
-    else if (aType != "load")
-      window.removeEventListener("load", this, false);
+    } else if (aType != "load") {
+      window.removeEventListener("load", this);
+    }
   },
 
   onContentLoaded: function TMP_EL_onContentLoaded() {
+    if (Tabmix.isVersion(510) && !Tabmix.isVersion(530)) {
+      let newRule = '.tabbrowser-tab {' +
+          '-moz-binding: url("chrome://tabmixplus/content/tab/tabBindings.xml#tabmix-tabbrowser-tab-v51-52") !important;}';
+      gTMPprefObserver.insertRule(newRule);
+    }
     if (!Tabmix.isVersion(510)) {
       let newRule = '.tabbrowser-tab {' +
-          '-moz-binding: url("chrome://tabmixplus/content/tab/tabbrowser_4.xml#tabmix-tabbrowser-tab-before-v51") !important;}';
+          '-moz-binding: url("chrome://tabmixplus/content/tab/tabBindings.xml#tabmix-tabbrowser-tab-before-v51") !important;}';
       gTMPprefObserver.insertRule(newRule);
     }
     if (Tabmix.isVersion(280) && !Tabmix.isVersion(470)) {
       let newRule = '.tabbrowser-tab > .tab-stack > .tab-content > .tab-label[tabmix="true"] {' +
-        '-moz-binding: url("chrome://tabmixplus/content/tab/tabbrowser_4.xml#tabmix-tab-label") !important;}';
+        '-moz-binding: url("chrome://tabmixplus/content/tab/tabBindings.xml#tabmix-tab-label") !important;}';
       gTMPprefObserver.insertRule(newRule);
     }
 
@@ -359,6 +382,9 @@ var TMP_eventListener = {
       "TabAttrModified"];
     this.toggleEventListener(gBrowser.tabContainer, this._tabEvents, true);
 
+    gBrowser.selectedTab.tabmixKey = {};
+    this.tabWidthCache.set(gBrowser.selectedTab.tabmixKey, 0);
+
     try {
       TMP_extensionsCompatibility.onContentLoaded();
     } catch (ex) {
@@ -388,8 +414,8 @@ var TMP_eventListener = {
   },
 
   onWindowOpen: function TMP_EL_onWindowOpen() {
-    window.addEventListener("unload", this, false);
-    window.addEventListener("SSWindowClosing", this, false);
+    window.addEventListener("unload", this);
+    window.addEventListener("SSWindowClosing", this);
     window.addEventListener("fullscreen", this, true);
 
     if (Tabmix.isVersion(320)) {
@@ -415,7 +441,7 @@ var TMP_eventListener = {
       Tabmix.assert(ex);
     }
     try {
-      tablib.init();
+      Tabmix.tablib.init();
     } catch (ex) {
       Tabmix.assert(ex);
     }
@@ -462,7 +488,7 @@ var TMP_eventListener = {
         tabBar.setAttribute("classic", "v4Mac");
       else if (TabmixSvc.isLinux) {
         tabBar.setAttribute("classic", "v3Linux");
-///XXX test if this is still the case
+        ///XXX test if this is still the case
         TMP_tabDNDObserver.LinuxMarginEnd = -2;
         Tabmix.setItem(tabsToolbar, "tabmix_skin", "classic");
       } else {
@@ -529,12 +555,12 @@ var TMP_eventListener = {
     TMP_ClosedTabs.setButtonType(Tabmix.prefs.getBoolPref("undoCloseButton.menuonly"));
 
     TabmixTabbar.hideMode = Tabmix.prefs.getIntPref("hideTabbar");
-   /*
-    *  In the first time TMP is running we need to match extensions.tabmix.hideTabbar to browser.tabs.autoHide.
-    *  extensions.tabmix.hideTabbar default is 0 "Never Hide tabbar"
-    *  if browser.tabs.autoHide is true we need to make sure extensions.tabmix.hideTabbar
-    *  is set to 1 "Hide tabbar when i have only one tab":
-    */
+    /**
+     *  In the first time TMP is running we need to match extensions.tabmix.hideTabbar to browser.tabs.autoHide.
+     *  extensions.tabmix.hideTabbar default is 0 "Never Hide tabbar"
+     *  if browser.tabs.autoHide is true we need to make sure extensions.tabmix.hideTabbar
+     *  is set to 1 "Hide tabbar when i have only one tab":
+     */
     gTMPprefObserver.setAutoHidePref();
 
     if (TabmixTabbar.hideMode == 2)
@@ -554,7 +580,7 @@ var TMP_eventListener = {
 
     // apply style on tabs
     let styles = ["currentTab", "unloadedTab", "unreadTab", "otherTab"];
-    styles.forEach(function(ruleName) {
+    styles.forEach(ruleName => {
       gTMPprefObserver.updateTabsStyle(ruleName);
     });
     // progressMeter on tabs
@@ -569,44 +595,48 @@ var TMP_eventListener = {
     Tabmix.setItem("tmp_undocloseButton", "disabled", true);
     Tabmix.setItem("tmp_closedwindows", "disabled", true);
 
+    if (Tabmix.isVersion(550)) {
+      Tabmix.changeCode(tabBar, "gBrowser.tabContainer.adjustTabstrip")._replace(
+        'this.tabbrowser.visibleTabs[this.tabbrowser._numPinnedTabs];',
+        'TMP_TabView.checkTabs(this.tabbrowser.visibleTabs);'
+      ).toCode(false, tabBar, "tabmix_adjustTabstrip");
+    }
     Tabmix.setNewFunction(tabBar, "adjustTabstrip", Tabmix.adjustTabstrip);
     delete Tabmix.adjustTabstrip;
   },
 
   tabWidthCache: new WeakMap(),
-  onTabAttrModified: function(aEvent) {
+  onTabAttrModified(aEvent) {
     if (!TabmixTabbar.widthFitTitle)
       return;
 
-    // when browser.tabs.animate is true and a new tab is still opening we wait
-    // until onTabOpen_delayUpdateTabBar call updateScrollStatus
-    let lastTab = Services.prefs.getBoolPref("browser.tabs.animate") &&
-        gBrowser.getTabForLastPanel();
-    if (lastTab && !lastTab._fullyOpen)
-      return;
-
     // catch tab width changed when label attribute changed
     // or when busy attribute changed hide/show image
     var tab = aEvent.target;
-    var width = tab.boxObject.width;
-    if (this.tabWidthCache.has(tab) && this.tabWidthCache.get(tab) == width)
+    var key = tab.tabmixKey;
+    if (!this.tabWidthCache.has(key)) {
       return;
+    }
 
-    this.tabWidthCache.set(tab, width);
+    var width = tab.boxObject.width;
+    if (this.tabWidthCache.get(key) == width) {
+      return;
+    }
+
+    this.tabWidthCache.set(key, width);
 
     TabmixTabbar.updateScrollStatus();
     setTimeout(() => TabmixTabbar.updateScrollStatus(), 2500);
   },
 
-  onSSTabRestoring: function TMP_EL_onSSTabRestoring(aEvent) {
-    var tab = aEvent.target;
+  onSSTabRestoring: function TMP_EL_onSSTabRestoring(tab) {
     Tabmix.restoreTabState(tab);
     TabmixSessionManager.restoreHistoryComplete(tab);
 
     gBrowser.ensureTabIsVisible(gBrowser.selectedTab, false);
 
     // don't mark new tab as unread
-    var url = tab.linkedBrowser.currentURI.spec;
+    let url = tab.__SS_lazyData ? tab.__SS_lazyData.url : tab.linkedBrowser.currentURI.spec;
     if (url == TabmixSvc.aboutBlank || url == TabmixSvc.aboutNewtab)
       tab.setAttribute("visited", true);
   },
@@ -618,8 +648,8 @@ var TMP_eventListener = {
       if (!fullScrToggler) {
         fullScrToggler = document.createElement("hbox");
         fullScrToggler.id = "fullscr-bottom-toggler";
-        fullScrToggler.addEventListener("mouseover", this._expandCallback, false);
-        fullScrToggler.addEventListener("dragenter", this._expandCallback, false);
+        fullScrToggler.addEventListener("mouseover", this._expandCallback);
+        fullScrToggler.addEventListener("dragenter", this._expandCallback);
         fullScrToggler.hidden = true;
         let bottombox = document.getElementById("browser-bottombox");
         bottombox.appendChild(fullScrToggler);
@@ -645,7 +675,7 @@ var TMP_eventListener = {
         }
       }
       let fullScreen = Tabmix.isVersion(470) ?
-          document.fullscreenElement : document.mozFullScreen;
+        document.fullscreenElement : document.mozFullScreen;
       if (!fullScreen) {
         fullScrToggler.hidden = false;
       }
@@ -657,7 +687,7 @@ var TMP_eventListener = {
       this.updateMultiRow();
   },
 
-  showNavToolbox: function() {
+  showNavToolbox() {
     this._updateMarginBottom("");
     this.toggleTabbarVisibility(true);
     this.updateMultiRow();
@@ -671,12 +701,12 @@ var TMP_eventListener = {
     gBrowser.ensureTabIsVisible(gBrowser.selectedTab, false);
   },
 
- /**
-  * for use in Firefox 40+.
-  * update FullScreen._mouseTargetRect when in full screen and the tabbar is
-  * visible. we call this function from tabBarHeightModified and showNavToolbox
-  */
-  updateMouseTargetRect: function() {
+  /**
+   * for use in Firefox 40+.
+   * update FullScreen._mouseTargetRect when in full screen and the tabbar is
+   * visible. we call this function from tabBarHeightModified and showNavToolbox
+   */
+  updateMouseTargetRect() {
     if (!Tabmix.isVersion(400)) {
       return;
     }
@@ -710,7 +740,7 @@ var TMP_eventListener = {
   },
 
   // for tabs bellow content
-  toggleTabbarVisibility: function(aShow, aAnimate) {
+  toggleTabbarVisibility(aShow, aAnimate) {
     let fullScrToggler = document.getElementById("fullscr-bottom-toggler");
     if (TabmixTabbar.position != 1 || !fullScrToggler) {
       return;
@@ -745,7 +775,7 @@ var TMP_eventListener = {
     }
   },
 
-  updateMultiRow: function(aReset) {
+  updateMultiRow(aReset) {
     if (aReset)
       Tabmix.tabsNewtabButton = null;
     if (TabmixTabbar.isMultiRow) {
@@ -763,7 +793,7 @@ var TMP_eventListener = {
     this.setTabAttribute(tab);
     TMP_LastTab.tabs = null;
     TMP_LastTab.attachTab(tab);
-    tablib.setLoadURIWithFlags(tab.linkedBrowser);
+    Tabmix.tablib.setLoadURIWithFlags(tab.linkedBrowser);
     if (TabmixTabbar.lockallTabs) {
       tab.setAttribute("locked", "true");
       tab.tabmix_allowLoad = false;
@@ -775,6 +805,17 @@ var TMP_eventListener = {
   // when more the one tabs opened at once
   lastTimeTabOpened: 0,
   onTabOpen_delayUpdateTabBar: function TMP_EL_onTabOpen_delayUpdateTabBar(aTab) {
+    if (aTab.__SS_lazyData) {
+      this.onSSTabRestoring(aTab);
+      if (aTab.label == "about:blank") {
+        aTab.label = gBrowser.mStringBundle.getString("tabs.emptyTabTitle");
+        gBrowser._tabAttrModified(aTab, ["label"]);
+      }
+    }
+    // cache tab width, we use our own key since Pale Moon doesn't have permanentKey
+    aTab.tabmixKey = {};
+    this.tabWidthCache.set(aTab.tabmixKey, aTab.boxObject.width);
+
     let newTime = Date.now();
     if (Tabmix.tabsUtils.overflow || newTime - this.lastTimeTabOpened > 200) {
       this.onTabOpen_updateTabBar(aTab);
@@ -782,7 +823,7 @@ var TMP_eventListener = {
     } else if (!this._onOpenTimeout) {
       let self = this;
       let timeout = Tabmix.tabsUtils.disAllowNewtabbutton &&
-          Services.prefs.getBoolPref("browser.tabs.animate") ? 0 : 200;
+          TabmixSvc.tabAnimationsEnabled ? 0 : 200;
       this._onOpenTimeout = window.setTimeout(function TMP_onOpenTimeout(tab) {
         if (self._onOpenTimeout) {
           clearTimeout(self._onOpenTimeout);
@@ -820,6 +861,7 @@ var TMP_eventListener = {
     tab._tPosInGroup = TMP_TabView.getTabPosInCurrentGroup(tab);
     TMP_LastTab.tabs = null;
     TMP_LastTab.detachTab(tab);
+    TMP_Places.updateRestoringTabsList(tab);
     var tabBar = gBrowser.tabContainer;
 
     // if we close the 2nd tab and tabbar is hide when there is only one tab
@@ -833,10 +875,10 @@ var TMP_eventListener = {
       }
     }
 
-    // when browser.tabs.animate is true gBrowser._endRemoveTab calls
+    // when tab animations enabled is true gBrowser._endRemoveTab calls
     // onTabClose_updateTabBar.
     // we would like to get early respond when row height is going to change.
-    var updateNow = !Services.prefs.getBoolPref("browser.tabs.animate");
+    var updateNow = !TabmixSvc.tabAnimationsEnabled;
     if (!updateNow && tabBar.hasAttribute("multibar")) {
       let lastTab = Tabmix.visibleTabs.last;
       if (!TabmixTabbar.inSameRow(lastTab, Tabmix.visibleTabs.previous(lastTab))) {
@@ -856,8 +898,9 @@ var TMP_eventListener = {
     }
 
     // clean WeakMap
-    if (this.tabWidthCache.has(tab))
-      this.tabWidthCache.delete(tab);
+    if (this.tabWidthCache.has(tab.tabmixKey)) {
+      this.tabWidthCache.delete(tab.tabmixKey);
+    }
   },
 
   // TGM extension use it
@@ -884,8 +927,10 @@ var TMP_eventListener = {
 
     // workaround when we remove last visible tab
     if (tabBar.firstChild.pinned && TabmixTabbar.isMultiRow && Tabmix.tabsUtils.overflow &&
-        aTab._tPos >= Tabmix.visibleTabs.last._tPos)
-      tabBar.mTabstrip.ensureElementIsVisible(gBrowser.selectedTab, false);
+        aTab._tPos >= Tabmix.visibleTabs.last._tPos) {
+      const instantScroll = Tabmix.isVersion(570);
+      tabBar.mTabstrip.ensureElementIsVisible(gBrowser.selectedTab, instantScroll);
+    }
 
     if (Tabmix.tabsUtils.disAllowNewtabbutton)
       Tabmix.tabsUtils.adjustNewtabButtonVisibility();
@@ -898,6 +943,11 @@ var TMP_eventListener = {
   onTabSelect: function TMP_EL_TabSelect(aEvent) {
     var tab = aEvent.target;
 
+    if (TabmixTabbar.hideMode != 2 && TabmixTabbar.widthFitTitle &&
+        !tab.hasAttribute("width") && tab.hasAttribute("pending")) {
+      tab.setAttribute("width", Tabmix.getBoundsWithoutFlushing(tab).width);
+    }
+
     // for ColorfulTabs 6.0+
     // ColorfulTabs traps TabSelect event after we do
     // we need to set standout class before we check for getTabRowNumber
@@ -926,7 +976,7 @@ var TMP_eventListener = {
     TabmixSessionManager.tabSelected(true);
   },
 
-  updateDisplay: function(tab) {
+  updateDisplay(tab) {
     if (!tab.hasAttribute("visited"))
       tab.setAttribute("visited", true);
 
@@ -1008,26 +1058,50 @@ var TMP_eventListener = {
     }
 
     if (shouldMoveFocus) {
-      direction = direction > 0 ? 1 : -1;
-      tabBar.advanceSelectedTab(direction, true);
+      if (aEvent.mozInputSource == MouseEvent.MOZ_SOURCE_MOUSE) {
+        direction = direction > 0 ? 1 : -1;
+        tabBar.advanceSelectedTab(direction, true);
+      }
       aEvent.stopPropagation();
       aEvent.preventDefault();
     } else if (direction !== 0 && !Tabmix.extensions.treeStyleTab) {
       // this code is based on scrollbox.xml wheel/DOMMouseScroll event handler
       let scrollByDelta = function(delta) {
-        if (Tabmix.isVersion(480) &&
+        let scrollByPixels = true;
+        let instant;
+        let scrollAmount = 0;
+        if (Tabmix.isVersion(530) && TabmixTabbar.isMultiRow) {
+          delta = delta > 0 ? 1 : -1;
+          scrollAmount = delta * tabStrip.lineScrollAmount;
+        } else if (Tabmix.isVersion(480) &&
             aEvent.deltaMode == aEvent.DOM_DELTA_PIXEL) {
-          tabStrip.scrollByPixels(delta);
+          scrollAmount = delta;
+          instant = true;
         } else if (Tabmix.isVersion(490) &&
             aEvent.deltaMode == aEvent.DOM_DELTA_PAGE) {
-          tabStrip.scrollByPage(delta);
+          scrollAmount = delta * tabStrip.scrollClientSize;
+        } else if (Tabmix.isVersion(530)) {
+          scrollAmount = delta * tabStrip.lineScrollAmount;
         } else {
           // scroll the tabbar by one tab
           if (orient == "horizontal" || TabmixTabbar.isMultiRow) {
             delta = delta > 0 ? 1 : -1;
           }
+          scrollByPixels = false;
           tabStrip.scrollByIndex(delta);
         }
+
+        if (scrollByPixels) {
+          let useSmoothScroll = Tabmix.isVersion(530) && !Tabmix.isVersion(570) &&
+            aEvent.deltaMode != aEvent.DOM_DELTA_PIXEL && tabStrip.smoothScroll;
+          if (useSmoothScroll) {
+            tabStrip._smoothScrollByPixels(scrollAmount);
+          } else if (Tabmix.isVersion(570)) {
+            tabStrip.scrollByPixels(scrollAmount, instant);
+          } else {
+            tabStrip.scrollByPixels(scrollAmount);
+          }
+        }
       };
 
       if (orient == "vertical") {
@@ -1050,14 +1124,14 @@ var TMP_eventListener = {
   },
 
   onWindowClose: function TMP_EL_onWindowClose() {
-    window.removeEventListener("unload", this, false);
-    window.removeEventListener("SSWindowClosing", this, false);
+    window.removeEventListener("unload", this);
+    window.removeEventListener("SSWindowClosing", this);
 
     // notice that windows enumerator don't count this window
     var isLastWindow = Tabmix.numberOfWindows() === 0;
     // we close tabmix dialog windows on exit
     if (isLastWindow) {
-      ["tabmixopt-filetype", "tabmixopt-appearance", "tabmixopt"].forEach(function(aID) {
+      ["tabmixopt-filetype", "tabmixopt-appearance", "tabmixopt"].forEach(aID => {
         var win = Services.wm.getMostRecentWindow("mozilla:" + aID);
         if (win) {
           if (aID != "tabmixopt")
@@ -1079,8 +1153,8 @@ var TMP_eventListener = {
     window.removeEventListener("fullscreen", this, true);
     var fullScrToggler = document.getElementById("fullscr-bottom-toggler");
     if (fullScrToggler) {
-      fullScrToggler.removeEventListener("mouseover", this._expandCallback, false);
-      fullScrToggler.removeEventListener("dragenter", this._expandCallback, false);
+      fullScrToggler.removeEventListener("mouseover", this._expandCallback);
+      fullScrToggler.removeEventListener("dragenter", this._expandCallback);
     }
 
     this.toggleEventListener(gBrowser.tabContainer, this._tabEvents, false);
@@ -1128,6 +1202,9 @@ var TMP_eventListener = {
     }
     updateAttrib("class", "tab-icon-image", "role", "presentation");
     updateAttrib("class", "tab-text", "role", "presentation");
+    if (Tabmix.isVersion(570)) {
+      updateAttrib("class", "tab-background", "orient", "vertical");
+    }
   }
 
 };
@@ -1162,8 +1239,8 @@ Tabmix.initialization = {
 
     if (stopInitialization) {
       this.run = function() {};
-      window.removeEventListener("DOMContentLoaded", TMP_eventListener, false);
-      window.removeEventListener("load", TMP_eventListener, false);
+      window.removeEventListener("DOMContentLoaded", TMP_eventListener);
+      window.removeEventListener("load", TMP_eventListener);
     }
 
     delete this.isValidWindow;
