@@ -11,7 +11,7 @@ var Tabmix = {
     return (this.defaultPrefs = Services.prefs.getDefaultBranch("extensions.tabmix."));
   },
 
-  isVersion: function() {
+  isVersion() {
     return TabmixSvc.version.apply(null, arguments);
   },
 
@@ -22,13 +22,13 @@ var Tabmix = {
   },
 
   // Show/hide one item (specified via name or the item element itself).
-  showItem: function(aItemOrId, aShow) {
+  showItem(aItemOrId, aShow) {
     var item = typeof (aItemOrId) == "string" ? document.getElementById(aItemOrId) : aItemOrId;
     if (item && item.hidden == Boolean(aShow))
       item.hidden = !aShow;
   },
 
-  setItem: function(aItemOrId, aAttr, aVal) {
+  setItem(aItemOrId, aAttr, aVal) {
     var elem = typeof (aItemOrId) == "string" ? document.getElementById(aItemOrId) : aItemOrId;
     if (elem) {
       if (aVal === null || aVal === undefined) {
@@ -43,7 +43,7 @@ var Tabmix = {
     }
   },
 
-  setAttributeList: function(aItemOrId, aAttr, aValue, aAdd) {
+  setAttributeList(aItemOrId, aAttr, aValue, aAdd) {
     let elem = typeof (aItemOrId) == "string" ? document.getElementById(aItemOrId) : aItemOrId;
     let att = elem.getAttribute(aAttr);
     let array = att ? att.split(" ") : [];
@@ -58,7 +58,25 @@ var Tabmix = {
       elem.removeAttribute(aAttr);
   },
 
-  getTopWin: function() {
+  getBoundsWithoutFlushing(element) {
+    if (!("_DOMWindowUtils" in this)) {
+      try {
+        this._DOMWindowUtils =
+          window.QueryInterface(Ci.nsIInterfaceRequestor)
+              .getInterface(Ci.nsIDOMWindowUtils);
+        if (!this._DOMWindowUtils.getBoundsWithoutFlushing) {
+          this._DOMWindowUtils = null;
+        }
+      } catch (ex) {
+        this._DOMWindowUtils = null;
+      }
+    }
+    return this._DOMWindowUtils ?
+      this._DOMWindowUtils.getBoundsWithoutFlushing(element) :
+      element.getBoundingClientRect();
+  },
+
+  getTopWin() {
     return Services.wm.getMostRecentWindow("navigator:browser");
   },
 
@@ -72,7 +90,7 @@ var Tabmix = {
     return this.prefs.getBoolPref("singleWindow");
   },
 
-  isNewWindowAllow: function(isPrivate) {
+  isNewWindowAllow(isPrivate) {
     // allow to open new window if:
     //   user are not in single window mode or
     //   there is no other window with the same privacy type
@@ -80,11 +98,11 @@ var Tabmix = {
       !this.RecentWindow.getMostRecentBrowserWindow({private: isPrivate});
   },
 
-  lazy_import: function(aObject, aName, aModule, aSymbol, aFlag, aArg) {
+  lazy_import(aObject, aName, aModule, aSymbol, aFlag, aArg) {
     if (aFlag)
       this[aModule + "Initialized"] = false;
     var self = this;
-    XPCOMUtils.defineLazyGetter(aObject, aName, function() {
+    XPCOMUtils.defineLazyGetter(aObject, aName, () => {
       let tmp = {};
       Components.utils.import("resource://tabmixplus/" + aModule + ".jsm", tmp);
       let Obj = tmp[aSymbol];
@@ -98,13 +116,13 @@ var Tabmix = {
     });
   },
 
-  backwardCompatibilityGetter: function(aObject, aOldName, aNewName) {
+  backwardCompatibilityGetter(aObject, aOldName, aNewName) {
     if (aOldName in aObject)
       return;
 
     var self = this;
     Object.defineProperty(aObject, aOldName, {
-      get: function() {
+      get() {
         self.informAboutChangeInTabmix(aOldName, aNewName);
         delete aObject[aOldName];
         return (aObject[aOldName] = self.getObject(window, aNewName));
@@ -113,18 +131,18 @@ var Tabmix = {
     });
   },
 
-  informAboutChangeInTabmix: function(aOldName, aNewName) {
+  informAboutChangeInTabmix(aOldName, aNewName) {
     let err = Error(aOldName + " is deprecated in Tabmix, use " + aNewName + " instead.");
     // cut off the first lines, we looking for the function that trigger the getter.
     let stack = Error().stack.split("\n").slice(3);
-    let file = stack[0] ? stack[0].split(":") : null;
-    if (file) {
-      let [path, line] = file;
-      let index = path.indexOf("/", 2) - 3;
+    let stackData = stack[0] ? stack[0].split("@") : null;
+    if (stackData && stackData.length == 2) {
+      let [path, line] = stackData[1].replace("chrome://", "").split(":");
+      let index = path.indexOf("/") - 1;
       let extensionName = index > -1 ?
-         path.charAt(2).toUpperCase() + path.substr(3, index) + " " : "";
+        path.charAt(0).toUpperCase() + path.substr(1, index) + " " : "";
       this.clog(err.message + "\n\n" + extensionName + "extension call " + aOldName +
-                 " from:\nfile: chrome:" + path + "\nline: " + line +
+                 " from:\nfile: chrome://" + path + "\nline: " + line +
                  "\n\nPlease inform Tabmix Plus developer" +
                  (extensionName ? (" and " + extensionName + "developer.") : "."));
     } else {
@@ -132,15 +150,15 @@ var Tabmix = {
     }
   },
 
-  promptService: function(intParam, strParam, aWindow, aCallBack) {
+  promptService(intParam, strParam, aWindow, aCallBack) {
     var dpb = Cc["@mozilla.org/embedcomp/dialogparam;1"]
-                            .createInstance(Ci.nsIDialogParamBlock);
+        .createInstance(Ci.nsIDialogParamBlock);
     // intParam[0] - default button accept=0, cancel=1, extra1=2
     // intParam[1] - show menuList= 1 , show textBox= 0, hide_both= 2
     // intParam[2] - set checkbox checked  true=1 , false=0, hide=2
     // intParam[3] - flag  - for menuList contents: flag to set menu selected item
     //                     - for textBox rename: 1 , save: 0
-///XXX temp fix
+    ///XXX temp fix
     // intParam[4] - flag  - 1 - use Tabmix.Sessions.createMenuForDialog
 
     // we use non modal dialog when we call for prompt on startup
@@ -164,7 +182,7 @@ var Tabmix = {
 
     // we add dependent to features to make this dialog float over the window on start
     var dialog = Services.ww.openWindow(aWindow,
-           "chrome://tabmixplus/content/dialogs/promptservice.xul", "", "centerscreen" +
+      "chrome://tabmixplus/content/dialogs/promptservice.xul", "", "centerscreen" +
            (modal ? ",modal" : ",dependent"), dpb);
     if (!modal)
       dialog._callBackFunction = aCallBack;
@@ -219,7 +237,7 @@ var Tabmix = {
     return item.boxObject.screenX + (end ? item.getBoundingClientRect().width : 0);
   },
 
-  show: function(aMethod, aDelay, aWindow) {
+  show(aMethod, aDelay, aWindow) {
     TabmixSvc.console.show(aMethod, aDelay, aWindow || window);
   },
 
@@ -237,26 +255,27 @@ var Tabmix = {
     return null;
   },
 
-  installChangecode: function() {
+  installChangecode() {
     Services.scriptloader.loadSubScript("chrome://tabmixplus/content/changecode.js", window);
     this.installChangecode = function() {};
   },
 
-  _init: function() {
+  _init() {
     Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
     Components.utils.import("resource://gre/modules/Services.jsm");
     XPCOMUtils.defineLazyModuleGetter(this, "RecentWindow",
-                                      "resource:///modules/RecentWindow.jsm");
+      "resource:///modules/RecentWindow.jsm");
 
-    window.addEventListener("unload", function tabmix_destroy() {
-      window.removeEventListener("unload", tabmix_destroy, false);
+    const destroy = () => {
+      window.removeEventListener("unload", destroy);
       this.destroy();
-    }.bind(this), false);
+    };
+    window.addEventListener("unload", destroy);
 
     var methods = ["changeCode", "setNewFunction", "nonStrictMode",
-                   "getObject", "log", "getCallerNameByIndex", "callerName",
-                   "clog", "isCallerInList", "callerTrace",
-                   "obj", "assert", "trace", "reportError"];
+      "getObject", "log", "getCallerNameByIndex", "callerName",
+      "clog", "isCallerInList", "callerTrace",
+      "obj", "assert", "trace", "reportError"];
     methods.forEach(function(id) {
       this[id] = function TMP_console_wrapper() {
         return this._getMethod(id, arguments);
@@ -273,4 +292,4 @@ var Tabmix = {
 };
 
 Tabmix._init();
-Tabmix.lazy_import(window, "TabmixSvc", "Services", "TabmixSvc");
+Tabmix.lazy_import(window, "TabmixSvc", "TabmixSvc", "TabmixSvc");
